<?php

/** This file is the entry-point for incoming emails */

require_once("initialize.php");
require_once(_CLASSES_."email.class.php");
require_once(_CLASSES_."mail.class.php"); // legacy

/* {{{ Function reject */
/**
 *
 * Used when mail is rejected by parser. Send back the original message to user
 *
 * @param string $message the rejection message
 */
function reject($msg, $error)
{
  if (empty($error)) {
    echo "OK";
    exit;
  }
  
  $_ENV['error']->logger("Sending rejection: $error", "parser");

  $error .= "\n\n";
  $error .= "Original Message:\n";
  $error .= "-----------------\n";
  $error .= "Subject: ".$msg->subject."\n\n";
  $error .= "Body:\n".$msg->body;

  $mailer = new MAILER;
  # XXX can optionally specify a group address here, for when it's known?
  $mailer->set("email_from", $_ENV['config']['email']['sender']);
  $mailer->subject("Re: " . $msg->subject);
  $mailer->to($msg->sources[0]);
  $mailer->message($error);
  $mailer->send();

  // everything got submitted properly
  echo "OK";
  exit;
}
/* }}} */

/* {{{ Function update_permission_check */
/**
 * function update_permission_check
 *
 * Check if the message should trigger an update to an issue.
 *
 * @param object $msg the message object
 * @return integer $gid the group ID of the group which permitted this access;
 *         does not return if permission is not available
 */
function update_permission_check(&$msg)
{
  // First, if there's a user and that user is an admin, it's all good
  if ($msg->userid && is_admin($msg->userid)) {
    // use the GID under which this message came in
    if ($msg->gid) return $msg->gid;

    // or just pick a GID from the issue
    $groups = issue_groups($msg->issueid);
    return $groups[0]['gid'];
  }

  // If there's a specific group for this email, use it; otherwise, test
  // all of the issues's groups
  if ($msg->gid) {
    $groups_to_check = array($msg->gid);
  } else {
    // extract the GID from each of the issue groups
    $issue_groups = issue_groups($msg->issueid);
    $groups_to_check = array();
    foreach ($issue_groups as $ig) {
      $groups_to_check[] = $ig['gid'];
    }
  }
  // TODO if being permissive, check them all?

  foreach ($groups_to_check as $gid) {
    // Public groups are all good
    if (group_public($gid)) { return $gid; }
    // Otherwise, make sure this user can post here
    if ($msg->userid && permission_check('create_issues', $gid, $msg->userid)) { return $gid; }
  }

  // If we get this far, the user didn't have permission.
  reject($msg, sprintf("You do not have permission to update Issue #%s", $msg->issueid));
}
/* }}} */

/* {{{ Function create_permission_check */
/**
 * function create_permission_check
 *
 * Check if the message should trigger a new issue
 *
 * @param object $msg the message object
 * @return null only on success; otherwise the function does not return.
 */
function create_permission_check(&$msg)
{
  // If the group is not public, we need to check the user's permissions
  if (group_public($msg->gid)) { return; }

  // If the user is not an admin, keep checking
  if (is_admin($msg->userid)) { return; }

  // If the user is not a member of the group, reject
  $group_members = group_members($msg->gid);
  if (!array_key_exists($msg->userid, $group_members)) {
    reject($msg, sprintf("You are not a member of the '%s' group." . "  " . "Your issue was not created.", group_name($msg->gid)));
  }

  // Finally, actually check the user's permissions
  if (permission_check("create_issues", $msg->gid, $msg->userid)) { return; }

  // no good
  reject($msg, sprintf("You do not have permission to add issues to group '%s'." . "  " . "Your issue was not created.", group_name($msg->gid)));
}
/* }}} */

/* {{{ Function process */
function process()
{
  ////
  // Collect and parse the email
  
  // Confirm the POSTed shared secret
  if ($_POST['shared_secret'] != $_ENV['config']['email']['shared_secret']) die("Shared secrets do not match");
  
  // Get any hints
  $hints = array();
  if (!empty($_POST['gid'])) { $hints['gid'] = $_POST['gid']; }
  if (!empty($_POST['issueid'])) { $hints['issueid'] = $_POST['issueid']; }
  if (!empty($_POST['userid'])) { $hints['userid'] = $_POST['userid']; }
  
  // Get the POSTed message
  $fn = $_FILES['message']['tmp_name'];
  $rawdata_fp = fopen($_FILES['message']['tmp_name'], "r") or die("Could not open submitted file");
  $rawdata = @fread($rawdata_fp, filesize($_FILES['message']['tmp_name']));
  @fclose($rawdata_fp);
  
  // A single timestamp for this action
  $currtime = time();
  
  $msg = &new Email($rawdata, $hints);
  
  if ($msg->error) {
    reject($msg, $msg->error);
  }
  
  ////
  // Start processing the message

  $sources = implode(', ', $msg->sources);

  $_ENV['error']->logger("Received message from $sources with subject '{$msg->subject}'", "parser");
  $_ENV['error']->logger("decoded: $decode", "parser");
  
  // If the message has an issueid, add an event to that issue
  if ($msg->issueid) {
  
    // Make sure this issue exists
    if (!issue_exists($msg->issueid)) {
      reject($msg, sprintf("The issue number you tried to update (Issue #%s) does not exist.", $msg->issueid));
    }
  
    // get the Issue object
    $issue = &get_issue($msg->issueid);

    // Check that the user has permission, and get the gid
    // that grants the user that permission
    $permitting_gid = update_permission_check($msg);
  
    // Check to see if the issue was closed
    if (closed($msg->issueid)) {
      if (!is_admin($msg->userid) && !group_auto_reopen($permitting_gid)) {
        // they can't reopen it
        reject($msg, sprintf("The issue you are trying to update (Issue #%s) has been closed.", $msg->issueid));
      } else {
        // go ahead and reopen it
        $issue->reopen();
      }
    }
  
    // TODO parse any commands out of the message, either in the body
    // or in headers

    // attach any files necessary
    $fid = null;
    if (!empty($msg->files)) {
      foreach ($msg->files as $file) {
        if (!($fid = $issue->add_file($msg->issueid, $msg->userid, $file['body'], $file['filename']))) {
          reject(sprintf("Error uploading file '%s'.", $file['filename']));
        }
      }
    }

    // Finally , create the event
    if (!$issue->add_event($msg->body, $msg->userid, $issue->status(), 0, FALSE, $fid, null, $msg->gid)) {
      reject($msg, sprintf("There was an unknown problem adding your event to Issue #%s.", $msg->issueid));
    }
  
    // Add this user as a client (this won't turn a tech into a client, don't worry)
    $issue->add_client($msg->userid);
  
    $_ENV['error']->logger("Successfully updated issue #{$msg->issueid}", "parser");
  } else {
    // Message has no issueid -> add a new issue
    // If message has no discovered group id, bail.
    if (!$msg->gid) {
      reject($msg, sprintf("EnterTrack could not determine the group to which your email applies.  
      Please contact %s regarding sending emails to this system.", $_ENV['config']['email']['admin']));
    }
  
    create_permission_check($msg);
  
    // If we made it this far, the user has permission to add a new issue.  Get to it.
    $issue = add_new_issue($msg->userid, $msg->gid, $msg->subject, $msg->body, SEV_NORMAL);
    if ($issue === false) {
      reject($msg, "An error occurred while processing your message.");
    }
    $issueid = $issue->issueid();
  
    // Add this user as a client
    $issue->add_client($msg->userid);

    // upload files, if we have any attached
    if (!empty($msg->files)) {
      foreach ($msg->files as $file) {
        if (!$issue->add_file($issueid, $msg->userid, $file['body'], $file['filename'])) {
          reject($msg, "Error uploading file '{$file['filename']}'.");
        }
      }
    }
  
    // Send out the autoreply
    // XXX this should get squirreled away somewhere
    $sql  = "SELECT name,autoreply,email ";
    $sql .= "FROM groups ";
    $sql .= "WHERE gid=?";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($msg->gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $data = $result->fetchRow(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($data) and $data['autoreply']) {
      $message = str_replace("[TID]",$issueid,$data['autoreply']);
      $subject = "[{$data['name']}] Issue #$issueid";
      foreach ($msg->sources as $sender) {
        $mailer = new MAILER;
        $mailer->set("email_from", $data['email']);
  
        $mailer->subject($subject);
        $mailer->to($sender);
        $mailer->message($message);
        $mailer->send();
      }
    }
  
    $_ENV['error']->logger("Successfully created issue #$issueid", "parser");
  }
  
  // Let's send some notifications
  $_ENV['notif']->send_notifications();

  // Send a flag back to the submission script that everything worked out
  echo "OK";
}
/* }}} */

// XXX parse the incoming message, then initialize e.g., $_SESSION['userid']
// and the language stuff(?)

// Initialize
initialize_entertrack("email");

// process the incoming message
process();
?>
