"""poll.py

Copyright (C) 2005 Dustin J. Mitchell

You should have received a copy of the GNU General Public License along
with this file; see the file LICENSE.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

Poll an inbox on an external mail server (either via POP or IMAP)
and send any messages discovered to EnterTrack.

REQUIREMENTS

This script has been Tested with Python 2.4.

This script can run in daemon mode (polling at a specific interval), or it may
be configured to poll once and exit (e.g., when run from a crontab).

To avoid exposing email passwords on the command-line, this script
runs from a configuration file.  The file is parsed by Python's
ConfigParser module, which parses files formatted after the Windows
.INI format.  The file begins with a [global] section which specifies the contact information for the EnterTrack site:

[global]
# url of the 'email.php' script in your EnterTrack installation
submission-url: http://myserver.hk/entertrack/email.php
# shared_secret as configured in EnterTrack's conf/config.php
shared_secret: I-M-!-telling
# if this directive is specified, the script will not exit but
# will re-poll all accounts at this interval (in seconds)
daemon-poll-time: 60

This is followed by one or more account specifications, each of which
looks like this:

[mymailserver]
# type is "pop3", "pop3s" (secure), "imap" or "imap_ssl" (secure)
type: pop3
# server is the hostname or IP of the mailserver
server: my.mailserver.com
# port is the TCP port, if different from the default
# default ports:
# pop3: 110
# pop3s 995
# imap: 143
# imap_ssl: 993
port: 110
# username and password are the email login credentials
username: tracker
password: somethinghardtoguess
# optionally, a gid, issueid, and/or userid may be sent as a "hint":
gid = 4
issueid = 1029
userid = 19

Finally, the script is invoked with the path of the configuration file:

 poll.py /path/to/my/config/file.conf

"""

_usage = """Usage: %s /path/to/my/config/file.conf

See the text of this script for more detail on configuration and
invocation.
"""

import sys, urllib, os, time, string
import ConfigParser, imaplib, poplib
from email.Message import Message
from email.Generator import Generator
from cStringIO import StringIO
import urllib2, getopt

class Defer(Exception): pass
class Bounce(Exception): pass

# default TCP ports
default_ports = {
  'pop3' : 110,
  'pop3s' : 995,
  'imap' : 143,
  'imap_ssl' : 993,
  }

def main():
  """
  Read the configuration file and call out to the relevant functions
  """

  if len(sys.argv) != 2: die(_usage)

  # Load the configuration
  config_file = sys.argv[1]
  if not os.path.isfile(config_file): die("'%s' not found" % config_file)

  if hasattr(ConfigParser, 'RawConfigParser'):
    # use the safer "raw" config parser
    config = ConfigParser.RawConfigParser()
  else:
    # fallback for pre-2.3 Python
    config = ConfigParser.ConfigParser()
  config.read([config_file])

  load_global(config)
  accounts = {}
  for account in config.sections():
    if account != 'global':
      accounts[account] = load_account(config, account)

  # depending on the daemon config, check once or repeatedly
  if daemon_poll_time:
    while 1:
      check_accounts(accounts)
      time.sleep(daemon_poll_time)
  else:
    check_accounts(accounts)

####
# Error handling

# TODO - these could be spruced up to do a better job of notifying someone
# when running as a daemon?

def die(msg):
  print >>sys.stderr, msg
  sys.exit(1)

def warn(msg):
  print >>sys.stderr, msg

####
# Configuration handling

def load_global(config):
  global submission_url, shared_secret, daemon_poll_time
  # look at the global section
  if not config.has_section('global'): die("config: [global] section not found")

  if not config.has_option('global', 'submission-url'): die("config: [global] submission-url not specified")
  if not config.has_option('global', 'shared_secret'): die("config: [global] shared_secret not specified")
  submission_url = config.get('global', 'submission-url')
  shared_secret = config.get('global', 'shared_secret')

  daemon_poll_time = 0 # disabled by default
  if config.has_option('global', 'daemon-poll-time'):
    try: daemon_poll_time = config.getint('global', 'daemon-poll-time')
    except (ConfigParser.Error, ValueError): die("config: [global] daemon-poll-time must be an integer number of seconds")

def load_account(config, account):
  class Account(object): pass # cheap OO -- I just want something with attributes
  acct = Account()

  if not config.has_option(account, 'type'): die("config: [%s] type not specified" % account)
  if not config.has_option(account, 'server'): die("config: [%s] type not specified" % account)
  if not config.has_option(account, 'username'): die("config: [%s] type not specified" % account)
  if not config.has_option(account, 'password'): die("config: [%s] type not specified" % account)

  acct.type = config.get(account, 'type')
  acct.server = config.get(account, 'server')
  acct.username = config.get(account, 'username')
  acct.password = config.get(account, 'password')

  if acct.type not in default_ports: die("config: [%s] type '%s' unknown" % (account, acct.type))

  acct.port = default_ports[acct.type]
  if config.has_option(account, 'port'):
    try: acct.port = config.getint(account, 'port')
    except (ConfigParser.Error, ValueError): die("config: [%s] port must be an integer" % account)

  acct.hints = {}
  for hint in 'gid', 'iisueid', 'userid':
    if config.has_option(account, hint):
      try: acct.hints[hint] = config.getint(account, hint)
      except (ConfigParser.Error, ValueError): die("config: [%s] %s must be an integer" % (account, hint))

  return acct

####
# Account handling
  
def check_accounts(accounts):
  for name, account in accounts.items():
    if account.type == 'pop3' or account.type == 'pop3s':
      check_pop_account(account)
    elif account.type == 'imap' or account.type == 'imap_ssl': # account.type == 'imap'
      check_imap_account(account)
    else:
      die ('Invalid account type')

def check_imap_account(account):
  try:
    if account.type == 'imap':
       imap = imaplib.IMAP4(account.server, account.port)
    else:
       imap = imaplib.IMAP4_SSL(account.server, account.port)
    
    imap.login(account.username, account.password)
    imap.select()
    typ, data = imap.search(None, 'ALL')
    if typ != 'OK': die(str(data))
    for num in data[0].split():
      # fetch the message
      typ, data = imap.fetch(num, '(RFC822)')
      if typ != 'OK': die(str(data))

      # send it
      message = data[0][1]
      try:
        print "sending message"
        send_message(message, shared_secret, submission_url, account.hints)
      except (Defer, Bounce), e: # defer / bounce are about the same: ignore the message
        warn(str(e) + " -> message temporarily ignored")
        continue

      # and mark it deleted (it will be expunged when the mailbox is closed)
      imap.store(num, '+FLAGS', '\\Deleted')

    imap.close()
    imap.logout()

  except imaplib.IMAP4.error, e:
    die(str(e))

def check_pop_account(account):
  try:
    if account.type == 'pop3':
      pop = poplib.POP3(account.server, account.port)
    else:
      pop = poplib.POP3_SSL(account.server, account.port)
    pop.user(account.username)
    pop.pass_(account.password)
    l = pop.list()
    if l[0][1:3] != 'OK': die("POP error: %s" % l[0])

    count = len(l[1])

    for i in range(count):
      l = pop.retr(i+1)
      if l[0][1:3] != 'OK': die("POP error: %s" % l[0])
      message = string.join(l[1], "\n");
      try:
        send_message(message, shared_secret, submission_url, account.hints)
      except (Defer, Bounce), e: # defer / bounce are about the same: ignore the message
        warn(str(e) + " -> message temporarily ignored")
        continue
      
      resp = pop.dele(i+1)

    pop.quit()
  except poplib.error_proto, e: die(str(e))

####
# Message sending
  
def send_message(message_text, shared_secret, submission_url, hints):
  """
  Send MESSAGE to the web server, using the configuration variables
  """
  secret = Message()
  secret.add_header("Content-Disposition", "form-data", name="shared_secret")
  secret.set_payload(shared_secret)

  message = Message()
  message.add_header("Content-Disposition", "form-data", name="message", filename="message.txt")
  message.add_header("Content-type", "text/plain")
  message.set_payload(message_text)

  post = Message()
  post.add_header("Content-type", "multipart/form-data")
  for name, value in hints.items():
    hint = Message()
    hint.add_header("Content-Disposition", "form-data", name=name)
    hint.set_payload(str(value))
    post.attach(hint)
  post.attach(secret)
  post.attach(message)

  # Get the message, formatted as MIME
  output = StringIO()
  generator = Generator(output)
  generator.flatten(post)
  post_contents = output.getvalue()

  # split off the MIME header from the rest of the body (because we have to
  # send the headers separately)
  post_headers, post_data = post_contents.split("\n\n", 1)

  request = urllib2.Request(submission_url)
  request.add_data(post_data)
  # re-add all of the headers
  for header, value in post.items():
    request.add_header(header, value)

  # finally, open the URL
  try:
    rv = urllib2.urlopen(request)
    result = rv.read()
    if result.strip() != 'OK':
      raise Bounce(str(result))
  except urllib2.HTTPError, e:
    raise Defer(str(e))

if __name__ == "__main__":
  main()
