"""submit.py

Copyright (C) 2004 Dustin J. Mitchell

You should have received a copy of the GNU General Public License along
with this file; see the file LICENSE.  If not, write to the Free
Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

Submit an incoming email to EnterTrack.  This script should be run
by an MTA in response to incoming emails.

REQUIREMENTS

This script has been Tested with Python 2.4.

The script expects to find the message available on its standard input.
It makes no further assumptions about its input.

Most UNIX MTA's (at least Sendmail and Postfix) process the exit
code of invoked scripts according to <sysexits.h>, which defines a
number of numeric constants.  Among these constants is EX_TEMPFAIL
a which indicates a temporary failure (so the MTA should defer and
re-attempt delivery).  Simply returning 1 indicates a permanent failure
(which should result in a bounce).  This script returns the appropriate
values, as defined by the 'os' module or as hard-coded herein.  """

_usage = """Usage: %s [-g|--gid gid] [-i|--issueid issueid] [-u|--userid userid]
          [-d|--debug] shared_secret submission-url

Submit the email message on standard input to the given SUBMISSION URL using
the given SHARED SECRET.  If specified, send the given gid, issueid, or userid
as hints.  If -d or --debug is specified, verbose error messages will be
displayed.  If -d or --debug is specified twice, extra debugging information 
will also be displayed.
"""

# generally necessary
import sys, urllib, traceback

# used for the form submission
from email.Message import Message
from email.Generator import Generator
from cStringIO import StringIO
import urllib2, getopt

try:
  from os import EX_TEMPFAIL # try to get the system's value
except:
  EX_TEMPFAIL = 75 # the usual value for this exit code

class Defer(Exception): pass
class Bounce(Exception): pass

def main():
  """
  Read the message and send it on to the web server; also
  handles any errors that come up
  """

  try:
    options, args = getopt.getopt(sys.argv[1:], 'g:i:u:d', ['gid=', 'issueid=', 'userid=', 'debug'])
  except getopt.GetoptError, g:
    print _usage
    return

  if len(args) != 2:
    print _usage
    return

  shared_secret, submission_url = tuple(args)
  hints = {}
  debug = 0
  for option, value in options:
    if option in ('-g', '--gid'): hints['gid'] = value
    if option in ('-i', '--issueid'): hints['issueid'] = value
    if option in ('-u', '--userid'): hints['userid'] = value
    if option in ('-d', '--debug'): debug += 1

  try:
    message = read_message()
    send_message(message, shared_secret, submission_url, hints, debug)
  except Defer, e:
    sys.stdout.write("Delivery can be re-tried later")
    if debug:
      sys.stdout.write('\nError message: ' + str(e))
    sys.exit(EX_TEMPFAIL)
  except Bounce, e:
    sys.stdout.write("Permanent faliure - message bounced")
    if debug:
      sys.stdout.write('\nError message: ' + str(e))
    sys.exit(1)
  except:
    sys.stdout.write("Unknown submission error")
    if debug:
      sys.stdout.write("\nTraceback:\n")
      traceback.print_exc()
    sys.exit(1)

def read_message():
  """
  Read the message from standard input
  """
  message = sys.stdin.read()
  return message

def send_message(message_text, shared_secret, submission_url, hints, debug):
  """
  Send MESSAGE to the web server, using the configuration variables
  """
  secret = Message()
  secret.add_header("Content-Disposition", "form-data", name="shared_secret")
  secret.set_payload(shared_secret)

  message = Message()
  message.add_header("Content-Disposition", "form-data", name="message", filename="message.txt")
  message.add_header("Content-type", "text/plain")
  message.set_payload(message_text)

  post = Message()
  post.add_header("Content-type", "multipart/form-data")
  for name, value in hints.items():
    hint = Message()
    hint.add_header("Content-Disposition", "form-data", name=name)
    hint.set_payload(str(value))
    post.attach(hint)
  post.attach(secret)
  post.attach(message)

  # Get the message, formatted as MIME
  output = StringIO()
  generator = Generator(output)
  generator.flatten(post)
  post_contents = output.getvalue()
  if debug >= 2:
    print "POST TO SCRIPT:"
    for line in post_contents.split('\n'):
      print ">", line

  # split off the MIME header from the rest of the body (because we have to
  # send the headers separately)
  post_headers, post_data = post_contents.split("\n\n", 1)

  request = urllib2.Request(submission_url)
  request.add_data(post_data)
  # re-add all of the headers
  for header, value in post.items():
    request.add_header(header, value)

  # finally, open the URL
  try:
    rv = urllib2.urlopen(request)
    result = rv.read()
    if debug >= 2:
      print "RESULT FROM SCRIPT:"
      for line in result.split('\n'):
        print "<", line
    if result.strip() != 'OK':
      raise Bounce(str(result))
  except urllib2.HTTPError, e:
    raise Defer(str(e))

if __name__ == "__main__":
  main()
