<?php
/**
 * @package Framework
 * @subpackage Classes
 */

/* {{{ class DBObject */
/**
 * DBObject class
 *
 * The base class for the "business objects" of EnterTrack; this class handles some
 * of the mechanics of the write-through caching that these objects implement.  Most
 * of this mechanics is in handling 'types' of values.  Available types are:
 *
 *  string
 *  number (integer, float)
 *  boolean
 *  time (UNIX timestamp)
 *  issueid
 *  userid
 *  gid
 *  eid
 *  status
 *  category
 *  product
 *  severity
 *
 * Values can be in one of three formats: native (PHP values), db (with quoting),
 * and human (a textual description).
 */
class DBObject {

  /* {{{ Function _native_to_db */
  /**
   * _native_to_db
   *
   * Convert a value from native to db format.
   *
   * @param string $value the new value (as a PHP value)
   * @param string $type the type of the value
   */
  function _native_to_db($value, $type)
  {
    if (is_null($value)) { return null; }

    switch ($type) {
      case 'string': return $value;
      case 'number': return ($value+0) . ''; // ensure it's not a string
      case 'boolean': return $value?'t':'f';
      case 'time': return $value;

      // all ID types are literal
      case 'issueid':
      case 'userid':
      case 'gid':
      case 'eid':
      case 'status':
      case 'category':
      case 'product':
      case 'severity':
      default: return $value;
    }
  }
  /* }}} */

  /* {{{ Function _db_to_native */
  /**
   * _db_to_native
   *
   * Convert a value from db to native format.
   *
   * @param string $value the new value (as a PHP value)
   * @param string $type the type of the value
   */
  function _db_to_native($value, $type)
  {
    if (is_null($value)) { return null; }

    switch ($type) {
      case 'string': return $value;
      case 'number': return $value+0;
      case 'boolean': return $value=='t';
      case 'time': return $value;

      // all ID types are literal
      case 'issueid':
      case 'userid':
      case 'gid':
      case 'eid':
      case 'status':
      case 'category':
      case 'product':
      case 'severity':
      default: return $value;
    }
  }
  /* }}} */

  /* {{{ Function _db_type */
  /**
   * _db_type
   *
   * Return the proper value for an MDB2 'type'.
   *
   * @param string $type the type of the value
   */
  function _db_type($type)
  {
    /* note: see MDB2/Driver/Datatype/Common.php for the list of types */
    switch ($type) {
      case 'string': return 'text';

      case 'boolean': return 'text'; // ('t' or 'f')

      case 'number': return 'float';

      case 'time': return 'integer';

      // all ID types are integers
      case 'issueid':
      case 'userid':
      case 'gid':
      case 'eid':
      case 'status':
      case 'category':
      case 'product':
      case 'severity':
      default: return 'integer';
    }
  }
  /* }}} */

  /* {{{ Function _native_to_human */
  /**
   * _native_to_human
   *
   * Convert a value from native to human format.
   *
   * @param string $value the new value (as a PHP value)
   * @param string $type the type of the value
   */
  function _native_to_human($value, $type)
  {
    switch ($type) {
      case 'string':
        return '"' . $value . '"';

      case 'number':
        return $value;

      case 'boolean':
        return $value?'True':'False';

      case 'time':
        return $_ENV['i18n']->format_datetime($value);

      case 'issueid':
        $issue = &get_issue($value);
        if ($issue) {
          return "Issue #$value: \"" . $issue->subject() . "\"";
        } else {
          return "Issue #$value";
        }
      case 'userid':
        return ident( $value ) . " (User #$value)";

      case 'gid':
        return group_name($value) . " (Group #$value)";

      case 'eid':
        return "Event #$value";

      case 'status':
        return status($value);

      case 'category':
        return category($value);

      case 'product':
        return product($value);

      case 'severity':
        return severity_text($value);

      default:
        return $value;
    }
  }
  /* }}} */
}
/* }}} */
