<?php
/**
 * @package Framework
 * @subpackage Classes
 */
require_once("Mail/mimeDecode.php");
require_once("Mail/RFC822.php");

/* {{{ Class Email */
/**
 * Email Class
 *
 * This class implements the functionality to break an incoming
 * email message down into the components needed to deal with it as
 * an EnterTrack event.
 *
 * @author Dustin Mitchell (based on work of Edwin Robertson)
 * @version 1.0
 * @package Framework
 * @subpackage Classes
 */
class Email {
  /* {{{ Variables */
  /** Body of incoming mail message */
  var $body = null;
  
  /** Headers of incoming mail message */
  var $headers = array();

  /** Subject of incoming mail message */
  var $subject = null;

  /** Complete unmodified subject of incoming mail message */
  var $orig_subject = null;

  /** Source addresses of this message */
  var $sources = array();

  /** Destinations of this message */
  var $destinations = array();

  /** Files attached to the message */
  var $files = array();

  /** Group ID associated with this message */
  var $gid = null;

  /** Issue ID associated with this message */
  var $issueid = null;

  /** User ID associated with this message */
  var $userid = null;

  /** Parse arguments in the subject of this message */
  var $args = array();

  /** Any error that has occurred in processing **/
  var $error = FALSE;
  /* }}} */
  
  /* {{{ Constructor */
  /**
   * Initialize parser
   *
   * @param string $raw_data the raw text of the email, headers included
   * @param array $hints suggestions on how to deal with this message
   *
   * Recognized 'hints' keys:
   *  'gid' -- the gid to which this message relates
   *  'issueid' -- the issueid to which this message relates
   *  'userid' -- the userid posting this message
   */
  function Email($raw_data, $hints)
  {
    // Decode the message into member variables 
    $this->decode($raw_data);

    // Now interpret those results to figure out what kind of event this is
    if (!$this->error) { $this->interpret($hints); }
  }
  /* }}} */

  /* {{{ Function interpret */
  /**
   * Interpret tracker-specific information in the parsed message (subject
   * encodings, etc.)
   *
   * @param array $hints suggestions on how to deal with this message
   */
  function interpret($hints)
  {
    $this->args = $this->interpret_args();
    $this->issueid = $this->interpret_issueid($hints);
    $this->userid = $this->interpret_userid($hints);
    $this->gid = $this->interpret_gid($hints);
  }
  /* }}} */

  /* {{{ Function interpret_args */
  /**
   * Interpret the subject and remove arguments that are specified in the format
   * [<variable>=<value>]
   *
   * @return array
   */
  function interpret_args()
  {
    # Initialized the $args array
    $args = array();
    $subject = $this->subject;

    preg_match_all("/\[(.*?)]/",$subject,$parsedArgs);
    if (is_array($parsedArgs)) {
      $parsedArgs = preg_grep("/=/",$parsedArgs[1]);
      foreach ($parsedArgs as $arg) {
        $parts = split("=",$arg);
        $args[strtolower(trim($parts[0]))] = strtolower(trim($parts[1]));
        $subject = str_replace("[$arg]","",$subject);
      }
    }
    $this->subject = $subject;
    return $args;
  }
  /* }}} */

  /* {{{ Function interpret_issueid */
  /**
   * Interpret any signs of the issueid for this message
   *
   * @param array $hints suggestions on how to deal with this message
   */
  function interpret_issueid($hints)
  {
    if (array_key_exists('issueid', $hints)) {
       // this hint might be used to tie a specific email addy to a long-lived ticket
       return $hints['issueid'] + 0;
    }

    // Get the issue number from an 'issue #xx' string in the subject
    if (eregi("issue #",$this->subject)) {
       preg_match("/Issue\ #([0-9]+)/i", $this->subject, $issue);
       return $issue[1] + 0;
    }

    // Issueid given in subject?
    if (array_key_exists('issue', $this->args)) {
       return $this->args['issue'] + 0;
    }

    // Otherwise, no such luck
    return null;
  }
  /* }}} */

  /* {{{ Function interpret_gid */
  /**
   * Interpret any signs of the gid for this message
   *
   * @param array $hints suggestions on how to deal with this message
   */
  function interpret_gid($hints)
  {
    if (array_key_exists('gid', $hints)) {
      return $hints['gid'] + 0;
    }
    
    // Group given in subject?
    if (array_key_exists('group', $this->args)) {
      return group_from_name($this->args['group']);
    }

    // Could be the first value in parenthesis in Subject
    if (preg_match("/\(.*?\)/",$this->subject)) {
	    $test=group_from_name(preg_replace("/[^\(]*\((.*?)\).*/","$1",$this->subject));
	    if ( isset($test) ) {
		    return $test;
	    }
    }

    // Fetch group based on userid.
    $user = get_user($this->userid);
    if ($user) {
      $groups = $user->groups();

      // if user belongs to only one group, just use that
      if (count($groups) == 1) {
        return $groups[0];
      }
    }

    // Check whether this issue has only one group; if so, use it!
    if ($this->issueid) {
      $issue_groups = issue_groups($this->issueid);
      if (count($issue_groups) == 1) {
        return $issue_groups[0]['gid'];
      }
    }

    // Check whether a group owns any of the destination addresses
    foreach ($this->destinations as $destination) {
      if ($gid = group_from_email($destination)) {
        return $gid;
      }
    }

    // As a last resort, use the system default group, if one was
    // specified.
    if (isset($_ENV['config']['email']['default_groupid'])) {
      return $_ENV['config']['email']['default_groupid'];
    }

    // No luck -- return null
    return null;
  }
  /* }}} */

  /* {{{ Function interpret_userid */
  /**
   * Interpret any signs of the userid of the user submitting this message;
   * if none are found, this function adds a new "email-only" user to the
   * database and returns the resulting userid.
   *
   * @param array $hints suggestions on how to deal with this message
   * @return integer the userid for this message
   */
  function interpret_userid($hints)
  {
    if (array_key_exists('userid', $hints)) {
      return $hints['userid'] + 0;
    }

    // Check whether any of the source addresses correspond to
    // a real user
    foreach ($this->sources as $source) {
      if ($user = &get_user_by_email($source)) {
        return $user->userid();
      }
    }

    // No userid found -- create one
    $source = $this->sources[0];
    $user_data = array(
      'primary_email' => $source,
      'admin' => false,
    );
    $user = create_user($user_data);
    return $user->userid();
  }
  /* }}} */

  /* {{{ Function deocde */

  /**
   * Takes the raw data of an email and separates headers from body
   * and assigns the correct values to the header variables
   *
   * @param string $raw_data the raw message, including headers
   *
   */
  function decode($raw_data)
  {
    // If we didn't get any data to parse just stop now
    if (empty($raw_data)) {
      $this->error = "no data";
      return;
    }

    $decoded = Mail_MimeDecode::decode(array(
            'include_bodies' => 1,
            'decode_bodies' => 1,
            'decode_headers' => 1,
            'input' => $raw_data ) );

    // extract some easy info from the decoded message
    $this->headers = $decoded->headers;
    $this->subject = $decoded->headers['subject'];

    /* danda: This should not be necessary when passing 'decode_headers' to 
     * Mail_MimeDecode::decode()
     * $this->subject = mb_decode_mimeheader($this->subject);  # Issue 7958 marc@gridshield.net
     */

    if (!$this->subject) { $this->subject = '(no subject)'; }
    $this->orig_subject = $this->subject;
    
    // Get a list of canonical email addresses for the sender
    $source_headers = array('from', 'resent-from', 'sender');
    $this->sources = array();
    foreach ($source_headers as $source_header) {
      $this->append_addresses($this->sources, $this->headers[$source_header]);
    }

    if (!$this->sources) {
      // if there's no sender, ignore the message
      $this->error = "no senders";
      return;
    }

    // Same for the destinations
    $destination_headers = array('to', 'cc', 'resent-to');
    $this->destinations = array();
    foreach ($destination_headers as $destination_header) {
      $this->append_addresses($this->destinations, $this->headers[$destination_header]);
    }

    if (!$this->destinations) {
      // if there are no recipients, ignore the message
      $this->error = "no recipients";
      return;
    }

    // Now take a look at the MIME structure to see what we can get
    // for a body and for files
    $this->body = '';
    $this->files = array();

    // recurse over the message structure, building up $this->body and
    // $this->files in the process.
    if (strtolower($decoded->ctype_primary) == 'multipart') {
      $this->parse_multipart($decoded);
    } else {
      $this->parse_leaf_part($decoded);
    }
  }
  /* }}} */

  /* {{{ Function append_addresses */
  /**
   * utility function for decode()
   *
   * a recursive function to append all email addresses found in
   * $values to $arr.
   *
   * @param ref $arr reference to output array
   * @param mixed $values contents of a header
   */
  function append_addresses(&$arr, $values)
  {
    if ($values) {
      if (is_array($values)) {
        foreach ($values as $key => $value) {
          $this->append_addresses($arr, $value);
        }
      } else {
        # call parseAddressList with validation turned off (yep, we get invalid data..
        # our job is to deal with it, not put up a fuss)
        $addresses = Mail_RFC822::parseAddressList($values, 'localhost', FALSE, FALSE);
        if (!PEAR::isError($addresses)) {
          foreach ($addresses as $address_arr) {
            $arr[] = $address_arr->mailbox . '@' . $address_arr->host;
          }
        } else {
          die($addresses->getMessage() . "\n");
        }
      }
    }
  }
  /* }}} */

  /* {{{ Function parse_leaf_part */
  /**
   * Parse a leaf MIME object; if it's a file, keep it;
   * if it's a part of the body, append it to $this->body.
   *
   * @param object $decoded
   */
  function parse_leaf_part($decoded)
  {
    /* an attachment either has an 'attachment' disposition,
     * or at least a filename */
    if ($decoded->disposition == 'attachment' or
        $decoded->d_parameters['filename'] or
        /* QuickMail, at least passes the filename in the "name"
           parameter to the content-type header.. */
        $decoded->ctype_parameters['name']) {

      /* get the filename from the relevant header */
      if ($decoded->d_parameters['filename']) {
        $filename = $decoded->d_parameters['filename'];
      } else if ($decoded->ctype_parameters['name']) {
        $filename = $decoded->ctype_parameters['name'];
      } else {
        $filename = 'download.txt';
      }

      $this->files[] = array("filename" => $filename,
                             "body" => $decoded->body);
    /* Otherwise it's not an attachment, so it's part of the body */
    } else {

      if (strtolower($decoded->ctype_primary) == 'text') {
        if (strtolower($decoded->ctype_secondary) == 'plain') {
          $this->body .= $decoded->body . "\n";
        } else if (strtolower($decoded->ctype_secondary) == 'html') {
          $this->body .= strip_tags($decoded->body) . "\n";
        } else if (strtolower($decoded->ctype_secondary) == 'enriched') {
          $this->body .= strip_tags($decoded->body) . "\n";
        } else {
          $this->body .= $decoded->body;
        }
      } else {
        /* unknown type -- stick it away as an attachment */
        $num = sizeof($this->files);
        $filename = "unknown-body-part-$num.raw";
        $this->files[] = array("filename" => $filename,
                               "body" => $decoded->body);

        $_ENV['error']->logger("Un-decodable leaf part of type "
                        . "{$decoded->ctype_primary}/{$decoded->ctype_secondary} "
                        . "attached to issue as $filename.", "parser");
        $this->body .= "\n----\nun-decodable element in body; saved as '$filename'.\n----\n";
      }
    }
  }
  /* }}} */

  /* {{{ Function parse_multipart */
  /**
   * Parse a multipart MIME object, looking for files and
   * for a potential body.
   *
   * @param object $decoded
   */
  function parse_multipart($decoded)
  {
    /* mixed means we go through them sequentially */
    if (strtolower($decoded->ctype_secondary == 'mixed')) {
      foreach ($decoded->parts as $part) {
        if (strtolower($part->ctype_primary) == 'multipart') {
          $this->parse_multipart($part);
        } else {
          $this->parse_leaf_part($part);
        }
      }

    /* alternative means pick the best one -- we go for text/plain
     * or, failing that, text/html.  "related" means the parts may
     * reference each other, but for our purposes we will process 
     * it like "alternative" */
    } else if (strtolower($decoded->ctype_secondary) == 'alternative' 
            || strtolower($decoded->ctype_secondary) == 'related' ) {
      $html_part = null;
      foreach ($decoded->parts as $part) {
	if (strtolower($part->ctype_primary) == 'multipart') {
	       	$this->parse_multipart($part);
	}
        if (strtolower($part->ctype_primary) == 'text') {
          if (strtolower($part->ctype_secondary) == 'plain') {
            $this->parse_leaf_part($part);
            return;
          } else if (strtolower($part->ctype_secondary) == 'html') {
            $html_part = $part;
          }
        }
      }
      // if we get here, and have HTML text, then that's the best
      // we can do
      if ($html_part) {
        $this->parse_leaf_part($html_part);
      } else {
        $_ENV['error']->logger("Unable to find a suitable alternative in multipart/{$decoded->ctype_secondary}", "parser");
      }
    } else {
      $_ENV['error']->logger("Unknown mime type multipart/{$decoded->ctype_secondary}; ignoring part", "parser");
    }

    $this->body .= "\n----\nun-decodable multipart element in message body\n----\n";
  }
  /* }}} */
}
/* }}} */
