<?php
/* {{{ Function: is_admin */
/**
 * Determine if a user is an admin
 *
 * @param integer $userid
 * @returns boolean
 */
function is_admin($userid = null)
{
  // XXX this caching is ok, but in a bad place
  if (!isset($_ENV['admin'])) {
    $_ENV['admin'] = array();
  }

  if (empty($userid)) {
    $userid = $_SESSION['userid'];
  }
  
  if (array_key_exists($userid,$_ENV['admin'])) {
    return $_ENV['admin'][$userid];
  } else {
    // form query to get admin
    $stmt = $_ENV['dbi']->prepare("SELECT admin from users where userid=?");
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($userid);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $admin = $result->fetchOne(); 
    $result->free();
    $stmt->free();
    if ($admin == "t") {
      $_ENV['admin'][$userid] = TRUE;
      return TRUE;
    }

    $_ENV['admin'][$userid] = FALSE;
    return FALSE;
  }
  
  return FALSE;
}
/* }}} */

/* {{{ Function: is_employee */
/**
 * Determines whether or not the user is an employee
 *
 * @param integer $userid ID of user to check
 * @returns boolean
 */
function is_employee($userid = null)
{
  if (empty($userid)) {
    $userid = $_SESSION['userid'];
  }

  if (is_admin($userid)) {
    return TRUE;
  }

  $stmt = $_ENV['dbi']->prepare("SELECT userid from group_users where userid=? and gid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($userid, _EMPLOYEES_));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $emp = $result->fetchOne(); 
  $result->free();
  $stmt->free();
  if (!is_null($emp)) {
    return TRUE;
  } 

  return FALSE;
}
/* }}} */

/* {{{ Function: is_group_admin */
/**
 * Determine if a user is a group administrator
 *
 * @param integer $userid ID of user to check
 * @return boolean
 */
function is_group_admin($gid,$userid = null)
{
  if (empty($userid)) {
  	$userid = $_SESSION['userid'];
  }

  if (is_admin($userid)) {
    return TRUE;
  }

  if (permission_set_id($gid,$userid) == PSET_GADMIN) {
    return TRUE;
  }

  return FALSE;
}
/* }}} */

/* {{{ Function: is_manager */
/**
 * Determine if a user is a manager
 *
 * @param integer $userid ID of user to check
 * @return boolean
 */
function is_manager($userid = null)
{
  if (empty($userid)) {
    $userid = $_SESSION['userid'];
  }

  if (is_admin($userid)) {
    return TRUE;
  }

  if (is_employee($userid)
  and permission_check("update_group",_EMPLOYEES_,$userid)) {
    return TRUE;
  }

  return FALSE;
}
/* }}} */

/* {{{ Function: permission_set */
/**
 * Retrieve the permission set for a given user in a given group
 *
 * @param integer $gid ID of group to look in
 * @param integer $userid ID of user to get permission set for
 * @returns array
 */
function permission_set($gid,$userid = null)
{
  $pset = array();

  if (empty($userid)) {
    $userid = $_SESSION['userid'];
  }

  $sql  = "SELECT p.permissions,g.perm_set ";
  $sql .= "FROM group_users g, permission_sets p ";
  $sql .= "WHERE g.userid=? ";
  $sql .= "AND g.gid=? ";
  $sql .= "AND g.perm_set = p.permsetid";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($userid, $gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  list($pset,$id) = $result->fetchRow(MDB2_FETCHMODE_ORDERED);
  $pset = explode(",",$pset);
  $result->free();
  $stmt->free();

  if (!is_array($pset)) {
    $_ENV['error']->logger("Permission set $id returns none arrary.","permissions");
    return;
  }

  return $pset;
}
/* }}} */

/* {{{ Function: permission_set_id */
/**
 * Retrieve the permission set id for a given user in a given group
 *
 * @param integer $gid ID of group to look in
 * @param integer $userid ID of user to get permission set for
 * @return integer
 */
function permission_set_id($gid,$userid = null)
{
  if (empty($userid)) {
    $userid = $_SESSION['userid'];
  }

  $sql  = "SELECT perm_set ";
  $sql .= "FROM group_users ";
  $sql .= "WHERE userid=? ";
  $sql .= "AND gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($userid, $gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $id = $result->fetchOne();
  $result->free();
  $stmt->free();

  return $id;
}
/* }}} */

/* {{{ Function: permission */
/**
 * Retrieve the text of a permission
 *
 * @param integer $permid ID of permission
 */
function permission($permid)
{
  $sql  = "SELECT permission ";
  $sql .= "FROM permissions ";
  $sql .= "WHERE permid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($permid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  $permission = $result->fetchOne();
  $result->free();
  $stmt->free();

  return $permission ? $permission : "UNKNOWN";
}
/* }}} */

/* {{{ Function: permission_set_name */
/**
 * Retrieve the permission set name for a permission set id
 *
 * @param integer $psetid ID of the permission set
 * @return string
 */
function permission_set_name($psetid)
{
  $sql  = "SELECT name ";
  $sql .= "FROM permission_sets ";
  $sql .= "WHERE permsetid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($psetid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  $name = $result->fetchOne();
  $result->free();
  $stmt->free();

  return $name;
}
/* }}} */

/* {{{ Function: permission_check */
/**
 * Determine if a user has a certain permission
 *
 * @param string $priv Privilege to be checked
 * @param integer $userid ID of user to check
 * @param integer $gid ID of group to check
 * @returns boolean
 */
function permission_check($perm,$gid = null,$userid = null)
{
  if (empty($userid)) {
    $userid = $_SESSION['userid'];
  }

  if(is_admin($userid)){
    return TRUE;
  }

  $sql  = "SELECT permid,group_perm,user_perm ";
  $sql .= "FROM permissions ";
  $sql .= "WHERE permission=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($perm));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  list($permid,$group,$user) = $result->fetchRow(MDB2_FETCHMODE_ORDERED);
  $result->free();
  $stmt->free();

  if (!$permid) {
    $_ENV['error']->logger("Lookup against unknown permission $sql ($perm).","permissions");
    return FALSE;
  }

  if ($user == "t") {
    $sql  = "SELECT userid ";
    $sql .= "FROM user_permissions ";
    $sql .= "WHERE userid=? ";
    $sql .= "AND permid=? ";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($userid, $permid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $permitted = ($result->numRows() > 0);;
    $result->free();
    $stmt->free();
    // did we find the permission?
    if ($permitted) return TRUE;
  }

  if ($group == "t") {
    // DBI TODO: do this with a join instead of repeated queries (it'll be safer that way, too)
    if ($userid == $_SESSION['userid']) {
      $user_groups = $_SESSION['groups'];
    } else {
      $user = &get_user($userid);
      $user_groups = $user->groups();
    }

    foreach ($user_groups as $key => $val) {
      $sql  = "SELECT permid ";
      $sql .= "FROM group_permissions ";
      $sql .= "WHERE permid=? ";
      $sql .= "AND gid=? ";
      $stmt = $_ENV['dbi']->prepare($sql); 
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute(array($permid, $val));
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $permitted = ($result->numRows() > 0);;
      $result->free();
      $stmt->free();
      if ($permitted) return TRUE;
    }

    return FALSE; // XXX is this wrong?
  }

  if ($gid) {
    $gids = array($gid);
  } else {
    $gids = $_SESSION['groups'];
  }
  foreach ($gids as $gid) {
    $pset = permission_set($gid,$userid);
    
    if (in_array($perm,$pset)) {
      return TRUE;
    }
  
    $sql  = "SELECT s.parent_gid ";
    $sql .= "FROM sub_groups s,groups g ";
    $sql .= "WHERE s.parent_gid=g.gid ";
    $sql .= "AND s.child_gid=? ";
    $sql .= "AND g.prop_user='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $permitted = FALSE;
    while ($parent = $result->fetchOne()) {
      $pset = permission_set($parent,$userid);
   
      if (in_array($perm,$pset)) {
        $permitted = TRUE;
        break;
      }
    }
    $result->free();
    $stmt->free();
    if ($permitted) return TRUE;

    $sql  = "SELECT child_gid ";
    $sql .= "FROM sub_groups ";
    $sql .= "WHERE parent_gid=? ";
    $sql .= "AND prop_issue='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $permitted = FALSE;
    while ($child = $result->fetchOne()) {
      $pset = permission_set($child,$userid);

      if (in_array($perm,$pset)) {
        $permitted = TRUE;
        break;
      }
    }
    $result->free();
    $stmt->free();
    if ($permitted) return TRUE;

  }

  return FALSE;
}
/* }}} */

/* {{{ Function: authenticate */
/**
 * Attempt to authenticate a user
 *
 * See active_directory_authenticate() for a description
 * of ActiveDirectory integration and config parameters.
 *
 * @param string $username Username of user
 * @param string $password Password of user
 * @return boolean|integer
 */
function authenticate($username,$password)
{
  // The login actually allows users to login by
  // their username or email address, both of
  // which should be unique to each user
  // so now we have to determine which one we're
  // looking up
  $field = ereg("@",$username) ? "primary_email" : "username";

  if (!$username) { return FALSE; }
  $rv = TRUE;
  $result = null;

  // Should we do active directory authentication at all?
  $use_active_directory = 
      isset( $_ENV['config']['auth']['active_directory']['enabled'] ) &&
      $_ENV['config']['auth']['active_directory']['enabled'] == true;

  // Should we require that active directory auth succeed?
  //  ( or else fall back to native auth )
  $active_directory_required = 
      isset( $_ENV['config']['auth']['active_directory']['required'] ) &&
      $_ENV['config']['auth']['active_directory']['required'] == true;

  $ad_authed = false;
  if( $use_active_directory )  {

      // Authenticate against Active Directory.
      $ad_authed = active_directory_authenticate( $username, $password );

      // If unauthed, and A.D. is required, then there's nothing left to do.
      if( !$ad_authed && $active_directory_required ) {
          if (defined("BROWSER")) {
              $_ENV['error']->push_error(_("Invalid ActiveDirectory login and/or password."));
          }
          return false;
      }
  }

  // If we authenticatd with A.D. then we just need to verify that user
  // exists locally.  We do NOT need to check local password.
  if( $ad_authed ) {
      $sql  = "SELECT username,userid,theme ";
      $sql .= "FROM users ";
      $sql .= "WHERE LOWER($field)=LOWER(?) "; // safe: see above
      $sql .= "AND active='t'";
      $stmt = $_ENV['dbi']->prepare($sql); 
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute(array($username));
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  }

  // If we did not authenticate with A.D. then proceed with regular auth.
  // This can happen in the case that A.D. auth is not enabled, or if
  // it is enabled, but user failed to auth with A.D.
  else {
      $sql  = "SELECT username,userid,theme ";
      $sql .= "FROM users ";
      $sql .= "WHERE LOWER($field)=LOWER(?) "; // safe: see above
      $sql .= "AND password=? ";
      $sql .= "AND active='t'";
      $stmt = $_ENV['dbi']->prepare($sql); 
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute(array($username, md5($password)));
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  }

  if (!list($username,$uid,$theme) = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
    if (defined("BROWSER")) {
      $_ENV['error']->push_error(_("Invalid login and/or password."));
    }
    $rv = FALSE;
  }
  $result->free();
  $stmt->free();

  if (!$rv) return $rv;

  // If we're calling the authentication function from
  // the browser then make sure to define the needed
  // session variables, and then redirect to the user
  // XXX this does *not* belong in a function named 'authenticate'!
  if (defined("BROWSER")) {
    $_SESSION['userid'] = $uid;
    /* Information culled from JavaScript on the browser */
    $_SESSION['javascript'] = $_POST['javascript'] == "enabled" ? "t" : "f";
    $_SESSION['browser_tz_offset'] = $_POST['browser_tz_offset'];

    $_SESSION['ip'] = $_SERVER['REMOTE_ADDR'];
    $_SESSION['theme'] = !empty($theme) ? $theme : $_SESSION['theme'];
    $_SESSION['prefs'] = user_preferences($uid);

    if (empty($_SESSION['prefs']['show_fields'])) {
      $_SESSION['prefs']['show_fields'] = array();
    } else {
      $_SESSION['prefs']['show_fields'] = explode(",",$_SESSION['prefs']['show_fields']);
    }
    if (empty($_SESSION['prefs']['sort_by'])) {
      $_SESSION['prefs']['sort_by'] = "issueid";
    }
    if (empty($_SESSION['prefs']['word_wrap'])) {
      $_SESSION['prefs']['word_wrap'] = 80;
    }

    // Misc settings
    $_SESSION['debugger'] = "off";

    $_ENV['error']->logger("$username logged in.","logins");

  } else {
    // If we're not calling authentication
    // from the browser then just return
    // the retrieved userid
    return $uid;
  }
} 
/* }}} */

/* {{{ Function: authenticate */
/**
 * Authenticate a user against Active Directory
 *
 * This function connects to an Active Directory
 * server via LDAP and binds using user-supplied
 * credentials.  If the bind succeeds, the user
 * is authenticated.
 *
 * This is intended as a temporary measure until
 * the auth subsystem branch is completed and merged.
 *
 * This implementation assumes that all usernames
 * are in the same namespace, eg: *@saga.gridshield.net.
 *
 * That allows us to use simple usernames instead of
 * requiring user to type fully qualified name when
 * logging in.
 *
 * Relevant Active Directory config options are:
 *
 *   // When enabled, active directory will be used for password authentication.
 *   // However, if AD auth fails, regular timetrex auth will proceed.
 *   $_ENV['config']['auth']['active_directory']['enabled'] = true;
 *
 *   // When required, login will fail if AD auth fails.
 *   $_ENV['config']['auth']['active_directory']['required'] = false;
 *
 *   // URL for ldap server.  Specify ldaps:// to use SSL.
 *   $_ENV['config']['auth']['active_directory']['server_uri'] = 'ldap://192.168.10.15';
 *
 *   // domain for users to login with.  eg: dan@saga.gridshield.net.
 *   $_ENV['config']['auth']['active_directory']['user_domain'] = 'saga.gridshield.net';
 *
 * Some of these parameters are used by the authenticate() function.
 *
 * @param string $username Username of user
 * @param string $password Password of user
 * @return boolean|integer
 */
function active_directory_authenticate( $username, $password ) {

    $conn = null;

    // LDAP server is required, eg: 'ldap://my.hostname' or 'ldap://127.0.0.1'
    if( !isset( $_ENV['config']['auth']['active_directory']['server_uri'] ) ) {
        $_ENV['error']->push_error(_("Active Directory server not configured."));
        return false;
    }
    $server_uri = $_ENV['config']['auth']['active_directory']['server_uri'];

    // A user domain is required. This must match the DC portion of the LDAP D.N.
    // Example: 
    //   in LDAP 'CN=dan,DC=saga,DC=gridshield,DC=net'
    //   user_domain = 'saga.gridshield.net'
    if( !isset( $_ENV['config']['auth']['active_directory']['user_domain'] ) ) {
        $_ENV['error']->push_error(_("Active Directory user domain not configured."));
        return false;
    }
    $user_domain = $_ENV['config']['auth']['active_directory']['user_domain'];

    // We connect
    $conn = @ldap_connect($server_uri);
    if( !$conn ) {
        if (defined("BROWSER")) {
            $_ENV['error']->push_error(_("Unable to connect to Active Directory server."));
        }
        return false;
    }

    //set some ldap options for talking to AD
    @ldap_set_option($conn, LDAP_OPT_PROTOCOL_VERSION, 3);
    @ldap_set_option($conn, LDAP_OPT_REFERRALS, 0);


    $ad_username = $username . '@' . $user_domain;
    $bind_ok = @ldap_bind($conn,$ad_username,$password);

    @ldap_close ($conn);

    return $bind_ok;
}
/* }}} */


/* {{{ Function: group_permission */
/**
 * Determine if a permission is a group permission
 *
 * @param integer $permid ID of permission
 * @return boolean
 */
function group_permission($permid)
{
  $permitted = FALSE;

  $sql  = "SELECT group_perm ";
  $sql .= "FROM permissions ";
  $sql .= "WHERE permid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($permid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  if ($group = $result->fetchOne(MDB2_FETCHMODE_ORDERED)) {
    $permitted = ($group == "t");
  }
  $result->free();
  $stmt->free();

  return $permitted;
}
/* }}} */
?>
