<?php

/*
 * error.php
 * 
 * The error-handling subsystem examines the EnterTrack configuration and
 * configures PHP's error handling to match.
 */

/* {{{ Function error_init_early */
/**
 * Initialize error handling with the given context.  In the 'email' context, 
 * errors are never sent to the "browser".
 *
 * @param string $context the context
 */
function error_init_early($context)
{
  if ($context == 'web') {
    $_ENV['error'] =& new WebErrorApi();

  } else if ($context == 'css') {
    $_ENV['error'] =& new NullErrorApi();

  } else if ($context == 'email') {
    $_ENV['error'] =& new EmailErrorApi();
  }
}
/* }}} */

/* {{{ Function error_init_late */
/**
 * Initialize error handling with the given context.  In the 'email' context, 
 * errors are never sent to the "browser".
 *
 * @param string $context the context
 */
function error_init_late($context)
{
  if ($context == 'web') {
    ini_set('display_errors', $_ENV['config']['debug']['display_errors']);
    error_reporting($_ENV['config']['debug']['error_reporting']);

  } else if ($context == 'css') {
    // be really quiet in CSS mode
    ini_set("display_errors", FALSE);
    error_reporting(E_ERROR);

  } else if ($context == 'email') {
    // never display errors in email
    ini_set("display_errors", FALSE);
    error_reporting($_ENV['config']['debug']['error_reporting']);
  }

  // set up a proper logfile
  ini_set("log_errors", TRUE);
  ini_set("error_log", _LOGS_."phperrors");
}
/* }}} */

/* {{{ Class ErrorApi */
/**
 * The $_ENV['error'] API; this class implements an error handling system which
 * manages to communicate the error to the user one way or another.   Debugging
 * support is also implemented here.
 */
class ErrorApi
{
  /* {{{ Function: push_error */
  /**
   * Add a error message to be displayed later.
   *
   * @param string $message The error message to add
   * @param boolean $notify_admin Notify the administrator of the 
   *        error (default: FALSE)
   */
  function push_error($message, $notify_admin=FALSE) { }
  /* }}} */

  /* {{{ Function: errors_pending */
  /**
   * Determine if there are errors to be displayed
   *
   * @return boolean
   */
  function errors_pending() { }
  /* }}} */

  /* {{{ Function: get_errors */
  /**
   * Return an array of the errors that have been pushed.
   *
   * @return array
   */
  function get_errors() { }
  /* }}} */
  
  /* {{{ Function: clear_errors */
  /**
   * Clear all pending errors.
   */
  function clear_errors() { }
  /* }}} */
  
  /* {{{ Function: check_pear_error */
  /**
   * Check for a PEAR error and, if one occurred, log it and
   * call fatal_error()
   *
   * @param $result: PEAR result
   * @param $file: pass __FILE__
   * @param $line: pass __LINE__
   * @return (may not return)
   */
  function check_pear_error($result, $file, $line)
  {
    if (PEAR::isError($result)) {
      $errmsg = $result->getMessage();
      $strform = $result->toString();
      $debuginfo = $result->getDebugInfo();
      $this->fatal_error("At $file:$line:<br/>$errmsg<br/>".
            "Details:<blockquote>$debuginfo<br />$strform</blockquote>");
    }
  }
  /* }}} */

  /* {{{ Function: fatal_error */
  /**
   * Display an error message regarding a fatal error.
   *
   * @return (does not return)
   */
  function fatal_error($message) { }
  /* }}} */

  /* {{{ Function: debug */
  /**
   * Send debug message to debug file
   *
   * @param mixed $data
   * @param string $description
   */
  function debug($data,$description = null) {}
  /* }}} */

  /* {{{ Function: logger */
  /**
   * Log a message to the specified logfile; such messages are not
   * necessarily presented to the user, unless debugging is enabled
   *
   * @param string $message Message to be logged to file
   * @param string $type Type of logging
   */
  function logger($message,$type)
  {
    $_ENV['error']->debug($message,ucwords(str_replace("_"," ",$type)));

    $date = "[".strftime("%b %d %Y %H:%M:%S", time())."] ";

    if (!empty($_SESSION['userid'])) {
      $user = $_SESSION['userid'];
      if (function_exists("username")) { $user = username($user); }
      $user = "[User: $user] ";
    } else {
      $user = "";
    }

    // log it to a file with PHP
    error_log("$date$user$message\n", 3, _LOGS_."/".$type);
  }
  /* }}} */
}
/* }}} */

/* {{{ Class NullErrorApi */
/**
 * Ignore all errors.
 */
class NullErrorApi extends ErrorApi
{
  /* {{{ Function: errors_pending */
  /**
   * Determine if there are errors to be displayed
   *
   * @return boolean
   */
  function errors_pending() { return false; }
  /* }}} */

  /* {{{ Function: get_errors */
  /**
   * Return an array of the errors that have been pushed.
   *
   * @return array
   */
  function get_errors() { return array(); }
  /* }}} */

  /* {{{ Function: fatal_error */
  /**
   * Display an error message regarding a fatal error.
   *
   * @return (does not return)
   */
  function fatal_error($message) { echo $message; exit(1); }
  /* }}} */
}
/* }}} */

/* {{{ Class WebErrorApi */
/**
 * An implementation of ErrorApi for the web context.  In this case, errors are
 * stored in $_SESSION for retrieval on the next page load.
 */
class WebErrorApi extends ErrorApi
{
  /* {{{ Function: constructor */
  function WebErrorApi()
  {
    if (!isset($_SESSION['errors']) || !is_array($_SESSION['errors'])) {
      $_SESSION['errors'] = array();
    }
  }
  /* }}} */

  /* {{{ Function: push_error */
  function push_error($message, $notify_admin=FALSE)
  {
    if ($notify_admin) {
      $message .= _(" The system administrator has been notified.");
    }

    if (!isset($_SESSION['errors'])) {
      $_SESSION['errors'] = array();
    }
  	array_push($_SESSION['errors'],$message);

    if ($notify_admin) {
      ob_start();
      print_r($GLOBALS);
      $contents = ob_get_contents();
      ob_end_clean();
      
      $message .= "\n\n$contents";
      admin_notify($message);
    }
  }
  /* }}} */
  
  /* {{{ Function: errors_pending */
  function errors_pending()
  {
  	return isset($_SESSION['errors']) && count($_SESSION['errors']) != 0;
  }
  /* }}} */
  
  /* {{{ Function: get_errors */
  function get_errors()
  {
    return $_SESSION['errors'];
  }
  /* }}} */
  
  /* {{{ Function: clear_errors */
  function clear_errors() 
  {
    $_SESSION['errors'] = array();
  }
  /* }}} */
  
  /* {{{ Function: fatal_error */
  function fatal_error($message)
  {
    // Don't want to use Smarty here, because it might not be loaded
    echo "<html><head><title>ERROR</title></head>";
    echo "<body>";
    echo "<h1>Fatal Error</h1>";
    echo "<p>EnterTrack has encountered a fatal error:";
    echo "<blockquote>";
    echo $message;
    echo "</blockquote>";
    echo "</body>";
    echo "</html>";
    exit;
  }
  /* }}} */

  /* {{{ Function: debug */
  function debug($data,$description = null) 
  { 
    $buffer = '';
    if (isset($_SESSION['debugger']) && $_SESSION['debugger'] == "on") {
      if (is_array($data)) {
        ob_start(); 
        print_r($data); 
        $data = ob_get_contents(); 
        ob_end_clean();
        $data = explode("\n",$data);
      }
     
      if (function_exists("debug_backtrace")) {
        $backtrace = debug_backtrace();
      }
  
      $buffer .= "<tr><td class=\"debug\">\n";
      $buffer .= "<pre class=\"fixed\">\n";
  
      $buffer .= $_ENV['i18n']->format_datetime(time())."<br />\n";
  
      if (!empty($description)) {
        $buffer .= "<b><u>$description</u></b>\n";
      }
      
      if (is_array($backtrace)) {
        foreach ($backtrace as $val) {
          $buffer .= sprintf("    <b>File</b>: %-70.70s",$val['file'])."\n";
          $buffer .= sprintf("    <b>Line</b>: %-6.6d",$val['line'])."\n";
          $buffer .= sprintf("<b>Function</b>: %-32.32s",$val['function'])."\n";
          $buffer .= "<b>----------</b>\n";
        }
      }
      
      if (is_array($data)) {
        foreach ($data as $line) {
          $buffer .= $this->colorize_debug($line)."\n";
        }
      } else {
        $data = wordwrap($data,80,'<br />',1);
        $buffer .= $data."\n";
      }
     
      $buffer .= "</pre>\n";
      $buffer .= "</td></tr>\n";
  
      print "<script type=\"text/javascript\" language=\"javascript\">\n";
      $lines = split("\n",$buffer);
      foreach ($lines as $line) {
        print "debugwin.document.writeln('$line');\n";
      }
      print "</script>\n";
    }
  }
  /* }}} */
  
  /* {{{ Function: colorize_debug */
  /**
   * Colorize the debug output of arrays
   *
   * @param string $array Array to colorize
   */ 
  function colorize_debug($array) 
  { 
    $array = str_replace('[','<font color="magenta">[</font><font color="red">',$array); 
    $array = str_replace(']','</font><font color="magenta">]</font>',$array); 
    $array = str_replace('Array','<font color="green">Array</font>',$array); 
    $array = str_replace('=>','<font color="navy">=></font>',$array); 
    return $array; 
  }
  /* }}} */
}
/* }}} */

/* {{{ Class EmailErrorApi */
/**
 * An implementation of ErrorApi for the email context.  In this case, errors
 * are accumulated locally, and will be picked up and dealt with during this
 * HTTP request.
 */
class EmailErrorApi extends ErrorApi
{
  /* {{{ Function: constructor */
  function EmailErrorApi()
  {
    $this->errors = array();
  }
  /* }}} */

  /* {{{ Function: push_error */
  function push_error($message, $notify_admin=FALSE)
  {
	  array_push($this->errors, $message);
  }
  /* }}} */

  /* {{{ Function: errors_pending */
  /**
   * Determine if there are errors to be displayed
   *
   * @return boolean
   */
  function errors_pending()
  {
  	return count($this->errors) != 0;
  }
  /* }}} */
  
  /* {{{ Function: get_errors */
  function get_errors()
  {
    return $this->errors;
  }
  /* }}} */
  
  /* {{{ Function: clear_errors */
  function clear_errors() 
  {
    echo 'CLEAR!';
    $_SESSION['errors'] = array();
  }
  /* }}} */
  
  /* {{{ Function: fatal_error */
  function fatal_error($message)
  {
    echo "FATAL ERROR: " . $message;
    exit;
  }
  /* }}} */

  /* {{{ Function: debug */
  function debug($data,$description = null)
  {
      $backtrace_buf = '';
      if (function_exists("debug_print_backtrace")) {
        ob_start();
        debug_print_backtrace();
        $backtrace_buf = "\n\nBacktrace: " . ob_get_contents();
        ob_end_clean();
      }

      $data_buf = is_array($data) ? implode( "\n", $data ) : $data;
      $data_buf = $data_buf ? ("\n\nData:\n" . $data_buf) : '';

      $buf = "\n----\n" . date('c') . ' -- Entertrack Email Debug Msg -- ' . $description . $data_buf . $backtrace_buf;

      error_log( $buf );
  }
  /* }}} */
}
/* }}} */
?>
