<?php

/*
 * i18n.php
 * 
 * This subsystem serves as an API for all of the internationalization
 * functionality, including locale, timezone, and date/time formats.
 *
 * These are all different concerns, but they're lumped here because
 * they're interrelated, and it produces somewhat more compact code.
 *
 * The API is available through an object at $_ENV['i18n'].
 */

// Use the PEAR Date module
include_once("Date.php"); // if this doesn't work, then hopefully index will send the user to setup/
include_once(_INCLUDES_."smarty-gettext.php");

/* Define these functions if PHP does not include gettext support */
$_i18n_gettext_installed = true;
if (!function_exists('gettext')) {
  $_i18n_gettext_installed = false;
  // Define "gettext" and "_" to just return the input argument
  function gettext($x) { return $x; }
  function _($x) { return $x; }
}

/* {{{ Function i18n_init */
/**
 * Initialize the internationalization subsystem in the given context.  In
 * 'web' context, we determine the user's language using various info from the
 * browser.  In 'email', we just use the default language.
 *
 * @param string @context the context
 */
function i18n_init($context)
{
  /* Fire up the subsystem */
  $_ENV['i18n'] = &new I18nSubsystem($context);

  // And add some template functions for translation, if the template engine
  // is loaded
  if (isset($_ENV['tpl'])) {
    $_ENV['tpl']->register_block('t', 'smarty_translate');
    $_ENV['tpl']->register_modifier('t', 'gettext');
  }
}
/* }}} */

/* {{{ Class: I18nSubsystem */
class I18nSubsystem
{
  // Format strings for the various date formats
  var $_DATE_FORMATS; // initialized in constructor

  // a Date_TimeZone object representing the current user's timezone
  var $timezone;

  /* {{{ Function: I18nSubsystem */
  /**
   * Constructor
   *
   * Initialize internationalization stuff, including locale (for translation),
   * timezone, and date/time format.
   *
   * @param string $context the context
   */
  function I18nSubsystem($context) {
    global $_i18n_gettext_installed;

    if ($_i18n_gettext_installed) {
      ////
      // Locale settings

      // Only look into the session if this is a web request
      if ($context == 'web') {
        if (isset($_SESSION['prefs']['locale'])) {
          $selected_locale = $_SESSION['prefs']['locale'];
        } else {
          // try to determine locale from the browser
          if (!empty($_SERVER['HTTP_ACCEPT_LANGUAGE'])) {
            $templocale = split(",",$_SERVER['HTTP_ACCEPT_LANGUAGE']);
            $selected_locale = $templocale[0];
      
            if (!file_exists(_LOCALE_."/$selected_locale")) {
              $selected_locale = $_ENV['config']['i18n']['locale'];
            }
          } else {
            // Failing that, use the configured default
            $selected_locale = $_ENV['config']['i18n']['locale'];
          }
          // record that in the session
          $_SESSION['prefs']['locale'] = $selected_locale;
        }
      } else {
        // just use the config locale, since we don't know who the user is
        $selected_locale = $_ENV['config']['i18n']['locale'];
      }

      // format the locale correctly
      if (strpos($selected_locale,"-")) {
        $parts = split("-",$selected_locale);
        $selected_locale = $parts[0]."_".strtoupper($parts[1]);
      }
    
      // set up the C locale functionality correctly
      bindtextdomain('messages',_LOCALE_);
      textdomain('messages');
      putenv("LANG=$selected_locale");
        
      if (setlocale(LC_ALL,$selected_locale) === FALSE) {
        $_ENV['error']->logger("Couldn't set locale (to ".$selected_locale.").","system_errors");
      } 
    }

    // Now that gettext is set up, initialize the date/time format list
    $this->_DATE_FORMATS = array(
      // each value is array(long name, date/time format)
      // supported conversions are %m, %d, %Y, %I, %H, %M, %S, %p
      'us'   => array(_("US (mm/dd/yyyy)"), '%m/%d/%Y %I:%M:%S %p'),
      'euro' => array(_("European (dd/mm/yyyy)"), '%d/%m/%Y %H:%M:%S'),
      'intl' => array(_("International (yyyy/mm/dd)"), '%Y/%m/%d %H:%M:%S'),
    );

    // Sart with a null timezone
    $this->timezone = null;
  }
  /* }}} */

  /*****
   * Date/Time Functions
   */


  /* {{{ Function: parse_datetime */
  /**
   * Parse the given date in the current user's date format.
   *
   * @param string $datetime the date/time as a string
   * @return the corresponding UNIX timestamp
   */
  function parse_datetime($datetime)
  {
    list($date_name, $time_format) = $this->_DATE_FORMATS[$_SESSION['date_format']];

    $year = 0;
    $month = 0;
    $day = 0;
    $hour = 0;
    $minute = 0;
    $second = 0;
    $pm = 0;

    // don't call ord() too many times
    $zero = ord('0');
    $nine = ord('9');

    // $i is an index into $datetime, $j is an index into $time_format
    $seen_percent = false;
    for ($i = $j = 0; $j < strlen($time_format); $j++) {
      $tfc = $time_format{$j};
      if ($seen_percent) {
        // Process a format character
        if ($tfc == 'p') {
          // parse 'am' or 'pm'
          if ($i+1 >= strlen($datetime)) return null; // enough space?
          if      (strtolower($datetime{$i}) == 'a' && strtolower($datetime{$i+1}) == 'm') { $pm = 0; }
          else if (strtolower($datetime{$i}) == 'p' && strtolower($datetime{$i+1}) == 'm') { $pm = 1; }
          else return null; // didn't match
          $i += 2;
        } else {
          // parse a number, then decide where to put it
          $num = '';
          while ($i < strlen($datetime)) {
            $c = $datetime{$i};
            $oc = ord($c);
            if ($oc < $zero || $oc > $nine) break;

            // got a digit -- pop it into num
            $num .= $c;
            $i++;
          }

          if (strlen($num) == 0) return null; // no digits!
          $num = $num + 0; // convert to integer

          switch ($tfc) {
            case 'm': $month = $num; break;
            case 'd': $day = $num; break;
            case 'Y': $year = $num; break;
            case 'H': 
            case 'I': $hour = $num; break;
            case 'M': $minute = $num; break;
            case 'S': $second = $num; break;
            default:
              // a programming error -> just die
              die('unknown format character \'' . $tfc . '\' in date format "' . $time_format . '".');
          }
        }
        $seen_percent = false;
      } else {
        // Not a format character
        if ($tfc == '%') {
          // Next character will be a format character
          $seen_percent = true;
        } else {
          // Otherwise, must match exactly
          if ($i >= strlen($datetime) || $tfc != $datetime{$i++}) return null; // didn't match
        }
      }
    }

    // format that into a time, *subtracting* the current TZ offset to get to a GMT time
    $localdate = gmmktime($hour + ($pm * 12), $minute, $second, $month, $day, $year);
    $gmtdate = $localdate;

    // Adjust for timezone if given
    if ($this->timezone) {
      $gmtdate -= $this->timezone->getOffset(new Date($localdate)) / 1000;
    }

    return $gmtdate;
  }
  /* }}} */

  /* {{{ Function: enumerate_date_formats */
  /**
   * Return an associative array (id => name) of all supported date formats.
   *
   * @return the array
   */
  function enumerate_date_formats()
  {
    $rv = array();
    foreach ($this->_DATE_FORMATS as $id => $fmt) {
      list($date_name, $time_format) = $fmt;
      $rv[$id] = $date_name;
    }

    return $rv;
  }
  /* }}} */

  /* {{{ Function: get_date_format_name */
  /**
   * Return the name of the current date format.
   *
   * @return string the name of the format
   */
  function get_date_format_name()
  {
    list($date_name, $time_format) = $this->_DATE_FORMATS[$_SESSION['date_format']];

    return $date_name;
  }
  /* }}} */

  /* {{{ Function: get_date_format_spec */
  /**
   * Return the format specification of the current date format.
   *
   * @return string the specification of the format
   */
  function get_date_format_spec()
  {
    list($date_name, $time_format) = $this->_DATE_FORMATS[$_SESSION['date_format']];

    return $time_format;
  }
  /* }}} */

  /* {{{ Function: format_datetime */
  /**
   * Format the specified date according to the current user's
   * locale settings, or according to the system default if
   * the user's settings are not defined.
   *
   * @param integer $timestamp (optional) UNIX timestamp
   * @return string
   */
  function format_datetime($timestamp = null)
  {
    /* use the current time if no other time was specified */
    if (empty($timestamp)) {
      $timestamp = time();
    }

    /* adjust for timezone, if it's defined */
    if ($this->timezone) {
      $timestamp += $this->timezone->getOffset(new Date($timestamp)) / 1000;
    }

    /* We use gmstrftime() instead of strftime() because the latter
     * adjusts according to the local timezone (of the server).  We
     * are already doing timezone adjustment in the line above, so
     * all we need to do is format according to format spec.
     *
     * See: http://php.net/manual/en/function.gmstrftime.php
     */
    return gmstrftime($this->get_date_format_spec(), $timestamp);
  }
  /* }}} */

  /* {{{ Function: setup_date_format */
  /**
   * Set the current date_format in the session; should only be called
   * at initial session setup and when prefs change.
   *
   * @param string $date_format_pref the user's preference as loaded from the database
   */
  function setup_date_format($date_format_pref)
  {
    // use the system default if no user pref is set
    if (empty($date_format_pref) || $date_format_pref == 'default' || !array_key_exists($date_format_pref, $this->_DATE_FORMATS)) {
      // use system default (from config)
      $date_format_pref = $_ENV['config']['i18n']['company_date_format'];
      if (!array_key_exists($date_format_pref, $this->_DATE_FORMATS)) {
        // configuration error -- bail
        die("\$_ENV['config']['i18n']['company_date_format'] does not specify a valid date format");
      }
    }

    $_SESSION['date_format'] = $date_format_pref;
  }
  /* }}} */

  /*****
   * Timezone functions
   */

  /* {{{ Function: enumerate_timezones */
  /**
   * List the available timezones by ID.
   *
   * @return array available timezones
   */
  function enumerate_timezones()
  {
    $tzs = Date_TimeZone::getAvailableIDs();
    sort($tzs);
    return $tzs;
  }
  /* }}} */

  /* {{{ Function: setup_timezone */
  /**
   * Set the current timezone.  This is called at every request, as 
   * it sets internal PHP state (its results cannot merely be stored
   * in _SESSION).
   *
   * @param string $timezone_pref the user's timezone preference
   */
  function setup_timezone($timezone_pref)
  {
    // use the system default if no user pref is set
    if (empty($timezone_pref) || $timezone_pref == 'default') {
      // use system default (from config)
      if ($_ENV['config']['i18n']['use_browser_timezone']) {
        $timezone_pref = 'browser';
      } else {
        $timezone_pref = $_ENV['config']['i18n']['company_timezone'];
      }
    }

    // Now autodetect if it's not explicit, and create a Date_TimeZone object
    if (empty($timezone_pref) || $timezone_pref== 'browser') {
      $this->timezone = &$this->autodetect_timezone();
    } else {
      $this->timezone = &new Date_TimeZone($timezone_pref);
    }

    // XXX need to cache the *name* of the timezone in the session to avoid
    // a big loop in autodetect_timezone in every request
  }
  /* }}} */

  /* {{{ Function: autodetect_timezone */
  /**
   * Using the 'tz' cookie, which is set by JavaScript on the browser,
   * determine the browser's timezone.  Since all we get from the browser
   * is an offset, we have to work backward to determine the name of the
   * corresponding timezone (well, *a* corresponding timezone -- if several
   * match, this will pick one of them arbitrarily).  This function falls
   * back to defaults as necessary.
   *
   * @return object the appropriate Date_TimeZone object
   */
  function autodetect_timezone()
  {
    // look at the info we got from the browser's JavaScript
    $browser_tz_offset = $_SESSION['browser_tz_offset'];
    if ($browser_tz_offset != 'unknown') {
      // Do a search of all of the IDs on hand to see which matches the supplied
      // timezone offset
      $all_ids = Date_TimeZone::getAvailableIDs();
      $now = &new Date();
      foreach ($all_ids as $try_id) {
        $tz = &new Date_TimeZone($try_id);
          // (convert offset from ms to minutes)
          // (JavaScript's TZ offsets are negative of Date_TimeZone's)
        if ($tz->getOffset($now) / 60000 == -$browser_tz_offset) {
          return $tz;
        }
      }
    }
    
    // can't autodetect -- fallback to the company default
    if ($_ENV['config']['i18n']['company_timezone']) 
      return new Date_TimeZone($_ENV['config']['i18n']['company_timezone']);
  
    // otherwise use the system default
    return Date_TimeZone::getDefault();
  }
  /* }}} */
}

/* }}} */
?>
