<?php
include_once(_SUBSYSTEMS_.'notif/notice.php');

/*
 * notif.php
 *
 * This subsystem manages notification settings and the formulation
 * and transmission of notifications.  Documentation of this subsystem
 * is available in the documentation Wiki.
 */

/* {{{ REASON_ Constants */
define("REASON_ISSUE_TECH", 1);
define("REASON_ISSUE_CLIENT", 2);
define("REASON_GROUP_TECH", 3);
define("REASON_GROUP_CLIENT", 4);
define("REASON_GROUP_SUB", 5); /* NOTE: this value is used independently of the constant in setup/util/schema_upgrades.php */
define("REASON_ISSUE_ASSIGNEE", 6); /* NOTE: this value is used independently of the constant in setup/util/schema_upgrades.php */

$_notif_reason_names = array(
  REASON_ISSUE_TECH => "Issue Tech",
  REASON_ISSUE_CLIENT => "Issue Client",
  REASON_GROUP_TECH => "Group Tech",
  REASON_GROUP_CLIENT => "Group Client",
  REASON_GROUP_SUB => "Group Subscriber",
  REASON_ISSUE_ASSIGNEE => "Issue Assignee"
);

/* }}} */

/* {{{ Function notif_init */
/**
 * Initialize the configuration.
 *
 * @param string $context the context
 */
function notif_init($context)
{
  $_ENV['notif'] =& new NotifApi($context);
}
/* }}} */

/* {{{ Class NotifApi */
/**
 * The API available at $_ENV['notif']
 */
class NotifApi
{
  /* {{{ Function constructor */
  function NotifApi($context)
  {
    // initialize some member variables
    $this->context = $context;

    // an array { userid => [ Notif_Notice objects ] }
    $this->notices = array();

    $this->channels_loaded = false;
    $this->channel_names = array();
    $this->channel_objects = array();
  }
  /* }}} */

  ////
  // General Information about Notifications

  /* {{{ Function enumerate_channels */
  /**
   * Enumerate the supported notification channels
   *
   * @return array, with keys the channelids and values the display names
   */
  function enumerate_channels()
  {
    $this->load_channels();
    return $this->channel_names;
  }
  /* }}} */

  /* {{{ Function enumerate_reasons */
  /**
   * Enumerate the supported notification reasons
   *
   * @return array, with keys the reasons and values the display names
   */
  function enumerate_reasons()
  {
    global $_notif_reason_names;
    return $_notif_reason_names;
  }
  /* }}} */

  /* {{{ Function get_reason_name */
  /**
   * Return the name for the given reason
   *
   * @param integer $reason
   * @return string
   */
  function get_reason_name($reason)
  {
    global $_notif_reason_names;
    if (isset($_notif_reason_names[$reason])) {
      return $_notif_reason_names[$reason];
    } else {
      return "(unknown reason #$reason)";
    }
  }
  /* }}} */

  /* {{{ Function get_channel */
  /**
   * Get a channel object specified by ID.
   *
   * @return array, with keys the channelids and values the display names
   */
  function get_channel($channelid)
  {
    $this->load_channels();
    return $this->channel_objects[$channelid];
  }
  /* }}} */

  ////
  // Functions to manipulate user preferences

  /* {{{ Function get_blank_prefs */
  /**
   * Return a blank array of notification preferences (all false)
   *
   * @return array such that $returnval[$reasonid][$channelid] is
   *  true if the given user wants to be notified via $channelid
   *  for $reasonid
   */
  function get_blank_prefs()
  {
    $channels = $this->enumerate_channels();
    $reasons = $this->enumerate_reasons();

    // First, set up an array filled with false values
    $rv = array();
    foreach ($reasons as $reasonid => $reasonname) {
      foreach ($channels as $channelid => $channel) {
        $rv[$reasonid][$channelid] = 0;
      }
    }

    return $rv;
  }
  /* }}} */

  /* {{{ Function get_user_prefs */
  /**
   * Return a user's notification preferences for a specific group.
   *
   * @param integer $userid the userid
   * @return array such that $returnval[$reasonid][$channelid] is
   *  true if the given user wants to be notified via $channelid
   *  for $reasonid
   */
  function get_user_prefs($userid)
  {
    $rv = $this->get_blank_prefs();

    // Now query the database to find any existing preferences
    $sql  = "SELECT channelid, reason as reasonid ";
    $sql .= "FROM notification_prefs ";
    $sql .= "WHERE userid=? ";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($userid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $prefs = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();

    if ($prefs) {
      foreach ($prefs as $row) {
        $rv[$row['reasonid']][$row['channelid']] = 1;
      }
    }

    return $rv;
  }
  /* }}} */

  /* {{{ Function set_user_prefs */
  /**
   * Set a user's notification preferences.
   *
   * @param integer $userid the userid
   * @param array $new_prefs the user's preferences as returned from get_user_prefs
   */
  function set_user_prefs($userid, $new_prefs)
  {
    $existing_prefs = $this->get_user_prefs($userid);

    $channels = $this->enumerate_channels();
    $reasons = $this->enumerate_reasons();

    foreach ($reasons as $reasonid => $reasonname) {
      foreach ($channels as $channelid => $channel) {
        $existing_pref = $existing_prefs[$reasonid][$channelid];
        $new_pref = $new_prefs[$reasonid][$channelid];

        if ($new_pref && !$existing_pref) {
          $insert = array();
          $types = array();
          $insert['userid'] = $userid;
          $types[] = 'integer';
          $insert['channelid'] = $channelid;
          $types[] = 'integer';
          $insert['reason'] = $reasonid;
          $types[] = 'integer';
          $result = $_ENV['dbi']->extended->autoExecute("notification_prefs",
            $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
          $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
        } elseif (!$new_pref && $existing_pref) {
          $wc  = "userid=" . $_ENV['dbi']->quote($userid, 'integer');
          $wc .= " AND channelid=" . $_ENV['dbi']->quote($channelid, 'integer');
          $wc .= " AND reason=" . $_ENV['dbi']->quote($reasonid, 'integer');
          $result = $_ENV['dbi']->extended->autoExecute("notification_prefs",
            null, MDB2_AUTOQUERY_DELETE,
            sprintf("userid=%s and channelid=%s and reason=%s",
              $_ENV['dbi']->quote($userid, 'integer'),
              $_ENV['dbi']->quote($channelid, 'integer'),
              $_ENV['dbi']->quote($reasonid, 'integer')));
          $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
        }
      }
    }
  }
  /* }}} */

  ////
  // Loading channel modules

  /* {{{ Function load_channels */
  /**
   * Load all channels in the notif/channels directory.
   */
  function load_channels() {
    if ($this->channels_loaded) { return; }

    // Load all channels located in _SUBSYSTEMS_.'notif/channels/'
    if ($dir = @opendir(_SUBSYSTEMS_.'notif/channels/')) {
      while (($item = readdir($dir)) !== FALSE) {
        if ($item{0} == "." or $item == ".." or $item == "CVS" or $item == "base.php") {
          continue;
        }

        if (is_file(_SUBSYSTEMS_.'notif/channels/'.$item) and eregi("\.php$",$item)) {
          include_once(_SUBSYSTEMS_.'notif/channels/'.$item);
        }
      }
    }

    $this->channels_loaded = true;
  }
  /* }}} */

  /* {{{ Function register_channel */
  /**
   * Register a channel class for later enumeration
   *
   * @param integer $channelid the assigned ID for this channel
   * @param string $name the name of this channel
   * @param object $object the channel object
   */
  function register_channel($channelid, $name, $object)
  {
    $this->channel_names[$channelid] = $name;
    $this->channel_objects[$channelid] = $object;
  }
  /* }}} */


  ////
  // Notifications

  /* {{{ Function record_new_issue */
  /**
   * Record the creation of a new issue, with the attendant information.
   * Any further settings for the issue are recorded as settings changes.
   *
   * @param integer $userid
   * @param integer $gid
   * @param integer $issueid
   * @param string $summary
   * @param string $problem
   */
  function record_new_issue($userid, $gid, $issueid, $summary, $problem)
  {
    include_once(_SUBSYSTEMS_."notif/changes/new_issue.php");

    $ch = &new Notif_Change_New_Issue($userid, $gid, $issueid, $summary, $problem);
    $this->record_change($ch);
  }
  /* }}} */

  /* {{{ Function record_settings_change */
  /**
   * Record a change in any of the settings for an issue.
   *
   * @param integer $userid -- the userid making the change
   * @param integer $issueid
   * @param string $setting the name of the setting
   * @param string $old_value the old value (or null if there was none)
   * @param string $new_value the new value for the setting
   */
  function record_settings_change($userid, $issueid, $setting, $old_value, $new_value)
  {
    include_once(_SUBSYSTEMS_."notif/changes/settings_change.php");

    $ch = &new Notif_Change_Settings_Change($userid, $issueid, $setting, $old_value, $new_value);
    $this->record_change($ch);
  }
  /* }}} */

  /* {{{ Function record_new_event */
  /**
   * Record the creation of a new event on an existing ticket.
   *
   * @param integer $userid -- the userid making the change
   * @param integer $issueid
   * @param integer $eid
   * @param string $action the text of the event
   * @param boolean $private
   */
  function record_new_event($userid, $issueid, $eid, $action, $private)
  {
    include_once(_SUBSYSTEMS_."notif/changes/new_event.php");

    $ch = &new Notif_Change_New_Event($userid, $issueid, $eid, $action, $private);
    $this->record_change($ch);
  }
  /* }}} */

  ////
  // Change-handling functions (private)

  /* {{{ Function record_change */
  /**
   * Record a new change object.  This function distributes the change to
   * all of the users that might be interested in hearing about it -- the
   * details of who that is depend on the change object.
   *
   * @param object $change
   */
  function record_change(&$change)
  {
    // Accumulate an array of Notif_Notice objects, at most one per user.
    $notices = array();

    $recips = $change->get_recipients();
    foreach ($recips as $l) {
      list($userid, $reason) = $l;
      if (!isset($notices[$userid])) {
        $notices[$userid] = &new Notif_Notice($userid, $change);
      }

      $notices[$userid]->add_reason($reason);
    }

    // and add those notice objects to the relevant users
    foreach ($notices as $userid => $notice) {
      if (!isset($notices[$userid])) {
        $this->notices[$userid] = array();
      }
      $this->notices[$userid][] = $notice;
    }
  }
  /* }}} */

  ////
  // End-of-request functions

  /* {{{ Function send_notifications */
  /**
   * Actually transmit/record the notifications. This function
   * is called at the end of a request.
   */
  function send_notifications()
  {
    // If there are no changes, bail out before even loading the channels
    if (sizeof($this->notices) == 0) {
      return;
    }

    // Load channels, if they're not already loaded
    $this->load_channels();

    // start through the changes on a user-by-user
    $sent_addresses_list = array();
    foreach ($this->notices as $userid => $notices) {
      $user = &get_user($userid);
      if ($user) {
        // $sent_addresses_by_channel is passed by reference.
        $sent_addresses_by_channel = array();
        $user->send_notifications($notices, $sent_addresses_by_channel);

        // Append to the list of sent addresses, by issue_id.
        foreach( $sent_addresses_by_channel as $channel_id => $sent_addresses_by_issue ) {
            foreach( $sent_addresses_by_issue as $issue_id => $sent_addresses ) {
                foreach( $sent_addresses as $addr ) {
                    // If channel is not 'Email', then we list the address as <channel>:<addr> (<channel>) to differentiate.
                    $channel_name = strtolower($this->channel_names[$channel_id]);
                    $sent_addresses_list[$issue_id][] = $channel_name == 'email' ? $addr : sprintf( '%s:%s', $channel_name, $addr );
                }
            }
        }
      }
    }
    // For each issue we sent notifications for, add sent addresses to issue log
    foreach( $sent_addresses_list as $issue_id => $sent_list ) {
        $sent_list_unique = array_unique( $sent_list );
        $maillist = implode(", ",$sent_list_unique);
        issue_log($issue_id,sprintf(_("Notifications sent to %s"),$maillist));
    }

  }
  /* }}} */
}
/* }}} */
