<?php

/*
 * base.php
 *
 * The base class for channels.
 */

/* {{{ Class Notif_Channel_Base */
class Notif_Channel_Base
{
  ////
  // Abstract functions (override these)

  /* {{{ Function send_notifications */
  /**
   * sent_notifications
   *
   * Send out any recorded notifications
   *
   * @param integer $userid the user to be notified
   * @param array $changes array of the changes to be shown
   * @param array &$sent_addresses_by_issue_id array of issues, each with a list of the addresses (email or otherwise) that notifications were sent to.
   */
  function send_notifications($userid, $changes, &$sent_addresses_by_issue_id) { }
  /* }}} */

  ////
  // Change-formatting support (of general use to most subclasses)

  /* {{{ Function format */
  /**
   * format
   *
   * Format the given changes according to $format.
   *
   * @param array $changes the change objects to format
   * @param string $format the name of the template to use for formatting
   * @return string the formatted changes
   */
  function format($changes, $format)
  {
    $by_issueid = $this->sort_by_issueid($changes);

    // first, format changes with no issueid (none of the known
    // change types lack an issueid, so this is just calls to
    // to_text
    $issueless_changes = array();
    if (isset($by_issueid[null])) {
      $issueless_changes = $by_issueid[null];
    }

    $issueful_changes = array();
    // now format all of the issue-related changes
    foreach ($by_issueid as $issueid => $issue_changes) {
      if (is_null($issueid)) { continue; } // already did those

      // further divide the changes by type
      $new_issue = null;
      $new_events = array();
      $settings_changes = array();
      $unknown_changes = array();

      foreach ($issue_changes as $ch) {
        switch (get_class($ch)) {
          case 'Notif_Change_New_Issue': $new_issue = $ch; break;
          case 'Notif_Change_New_Event': $new_events[] = $ch; break;
          case 'Notif_Change_Settings_Change': $settings_changes[] = $ch; break;
          default: $unkonwn_changes[] = $ch; break;
        }
      }

      // set them up in order
      $issue = &get_issue($issueid);

      $assignees = $issue->assignees();
      $assignee_names = array();
      foreach( $assignees as $userid_assignee ) {
          $assignee_names[] = ident( $userid_assignee );
      }
      $assigned_to_names = count($assignee_names) ? implode( ', ', $assignee_names ) : _( 'Unassigned' );

      $settings_changes_messages = array();

      // Create messages for display in the template.
      // Using sprintf() we can make i18n friendly messages.
      foreach( $settings_changes as $change ) {
          $msg = sprintf( _('Note: %1$s changed %2$s from %3$s to %4$s.'), ident($change->userid()), $change->setting, $change->old_value, $change->new_value );

          // Use special formatting when 'assigned_to' changes.
          if( strstr($change->setting, 'assigned_to' ) ) {
             $msg = sprintf( _('Note: %1$s assigned this issue to %2$s (in group %3$s).'), ident($change->userid()), $change->new_value, group_name($change->issue->primary_gid()) );
          }
          $settings_changes_messages[] = $msg;
      }

      $issueful_changes[$issueid] = array(
        'issue' => $issue,
        'assigned_to_names' => $assigned_to_names,
        'new_issue' => $new_issue,
        'new_events' => $new_events,
        'settings_changes' => $settings_changes,
        'settings_changes_messages' => $settings_changes_messages,
        'unkonwn_changes' => $unknown_changes,
      );
    }

    // now send it off to the template, first clearing out any previous
    // assignments in Smarty
    $_ENV['tpl']->clear_all_assign();
    $_ENV['tpl']->assign('issueless_changes', $issueless_changes);
    $_ENV['tpl']->assign('issueful_changes', $issueful_changes);
    $_ENV['tpl']->assign('config',$_ENV['config']);
    $buf = $_ENV['tpl']->fetch($format);
    return $buf;
  }
  /* }}} */

  /* {{{ Function sort_by_issueid */
  /**
   * sort_by_issueid
   *
   * Sort the given changes by issueid
   *
   * @param array $changes the change objects to format
   * @return array of lists of change objects, keyed by issueid
   */
  function sort_by_issueid($changes)
  {
    // organize the changes by issueid
    $by_issueid = array();

    foreach ($changes as $change) {
      $issueid = $change->issueid();
      $by_issueid[$issueid][] = $change;
    }

    return $by_issueid;
  }
  /* }}} */
}
/* }}} */
?>

