<?php
////
// Group Objects

/* {{{ Function get_group */
/**
 * Look up a group by gid
 *
 * @param integer $gid the group id
 * @return mixed a group object or null
 */
function get_group($gid)
{
  static $group_cache = array();

  if (!array_key_exists($gid, $group_cache)) {
    $sql  = "SELECT * ";
    $sql .= "FROM groups ";
    $sql .= "WHERE gid=? ";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);  
    $group_row = $result->fetchRow(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();

    if (is_null($group_row)) { return null; }
    $group_cache[$gid] = &new Group($group_row['gid'], $group_row);
  }

  return $group_cache[$gid];
}
/* }}} */

////
// Old, unreformed functions

/* {{{ Function: group_exists */
/**
 * Verifies that a group actually exists with given gid
 *
 * @param integer $gid ID of group to verify
 * @return boolean
 */
function group_exists($gid)
{
  $exists = FALSE;

	$sql  = "SELECT count(gid) ";
	$sql .= "FROM groups ";
	$sql .= "WHERE gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
	if ($result->fetchOne() > 0) {
		$exists = TRUE;
	}
  $result->free();
  $stmt->free();

	return $exists;
}
/* }}} */

/* {{{ Function: group_name */
/**
 * Get the name of a group from the database and return it
 *
 * @param integer $gid ID of group to get name of
 * @return string
 */
function group_name($gid)
{
  global $groups_name_cache;

  if (is_array($groups_name_cache)
  and $_ENV['cache']['groups']
  and @array_key_exists($gid,$groups_name_cache)) {
    return $groups_name_cache[$gid];
  } else {
    // Since we didn't find the name in cache
    // make sure to generate new cache file
    gen_cache("groups","gid","name");
    
    $rv = 'UNKNOWN';
    $sql  = "SELECT name ";
    $sql .= "FROM groups ";
    $sql .= "WHERE gid=? ";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    if ($name = $result->fetchOne()) {
      $rv = $name;
    }
    $result->free();
    $stmt->free();

    return $rv;
  }
}
/* }}} */

/* {{{ Function: group_from_name */
/**
 * Return a gid given a group name
 *
 * @param string $name the group name
 */
function group_from_name($name)
{
  $rv = null;

  $sql  = "SELECT gid ";
  $sql .= "FROM groups ";
  $sql .= "WHERE LOWER(name) = LOWER(?)";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($name));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  if ($gid = $result->fetchOne()) {
    $rv = $gid;
  }
  $result->free();
  $stmt->free();

  return $rv;
}
/* }}} */

/* {{{ Function: group_from_email */
/**
 * Return a gid given an email address
 *
 * @param string $email the email address
 */
function group_from_email($email)
{
  $rv = null;

  $sql  = "SELECT gid ";
  $sql .= "FROM groups ";
  $sql .= "WHERE LOWER(email) = LOWER(?)";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($email));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  if ($gid = $result->fetchOne()) {
    $rv = $gid;
  }
  $result->free();
  $stmt->free();

  return $rv;
}
/* }}} */

/* {{{ Function: group_visibility_add */
/**
 * Add a group for issue visibility
 *
 * @param integer $pgid ID of issue owner group
 * @param integer $cgid ID of issue viewer group
 * @param string $flag 'f' = read, 't' = write
 */
function group_visibility_add($pgid,$cgid,$flag = 'f')
{
  // check to make sure this group isn't already an viewer
  $sql  = "SELECT viewer_gid ";
  $sql .= "FROM group_issue_visibility ";
  $sql .= "WHERE owner_gid=? ";
  $sql .= "AND viewer_gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($pgid, $cgid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $already_a_viewer = ($result->numRows() > 0);
  $result->free();
  $stmt->free();

  if ($already_a_viewer) {
	  $result = $_ENV['dbi']->extended->autoExecute("group_issue_visibility",
      array("can_write" => $flag),
      MDB2_AUTOQUERY_UPDATE,
      sprintf("owner_gid=%s and viewer_gid=%s", 
            $_ENV['dbi']->quote($pgid, 'integer'), 
            $_ENV['dbi']->quote($cgid, 'integer')),
      array('text')); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  } else {
    $insert = array();
    $types = array();
  	$insert['owner_gid']  = $pgid;
    $types[] = 'integer';
  	$insert['viewer_gid'] = $cgid;
    $types[] = 'integer';
  	$insert['can_write']  = $flag;
    $types[] = 'text';
  	$result = $_ENV['dbi']->extended->autoExecute("group_issue_visibility",
      $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  }

  $cgname  = group_name($cgid);
  $pgname  = group_name($pgid);
  $_ENV['error']->logger("$cgname added as viewer of $pgname issues.","groups");
}
/* }}} */

/* {{{ Function: group_visibility_del */
/**
 * Remove a viewer group from issue visbility
 *
 * @param integer $pgid ID of group to remove viewer from
 * @param integer $cgid ID of group to remove as viewer
 * @return boolean
 */
function group_visibility_del($pgid,$cgid)
{
  $cgname  = group_name($cgid);
  $pgname  = group_name($pgid);
	
  $result = $_ENV['dbi']->extended->autoExecute("group_issue_visibility",
    null, MDB2_AUTOQUERY_DELETE,
    "owner_gid=" . $_ENV['dbi']->quote($pgid, "integer") . " AND " .
    "viewer_gid=" . $_ENV['dbi']->quote($cgid, "integer")); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);

  // did we change anything?
  if ($result != 0) {
    $_ENV['error']->logger("$pgname removed $cgname from issue visibility","groups");	
    return TRUE;
  }
  return FALSE;
}
/* }}} */

/* {{{ Function: group_assign_add */
/**
 * Add a group for issue reassignment
 *
 * @param integer $pgid ID of assignor group
 * @param integer $cgid ID of assignee group
 */
function group_assign_add($pgid,$cgid)
{
  // check to make sure this group isn't already an assignee
  $sql  = "SELECT * ";
  $sql .= "FROM assign_groups ";
  $sql .= "WHERE assignor_gid=? ";
  $sql .= "AND assignee_gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($pgid, $cgid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $already_assignee = $result->numRows();
  $result->free();
  $stmt->free();

  if ($already_assignee) return;

  // form insert array
  $insert = array();
  $types = array();
  $insert["assignor_gid"] = $pgid;
  $types[] = 'integer';
  $insert["assignee_gid"] = $cgid;
  $types[] = 'integer';
  $result = $_ENV['dbi']->extended->autoExecute("assign_groups",
    $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  $cgname  = group_name($cgid);
  $pgname  = group_name($pgid);
  $_ENV['error']->logger("$cgname added as issue assignee of $pgname.","groups");
}
/* }}} */

/* {{{ Function: group_assign_del */
/**
 * Remove an assignee from an assignor group
 *
 * @param integer $pgid ID of group to remove assignee from
 * @param integer $cgid ID of group to remove as assignee
 * @return boolean
 */
function group_assign_del($pgid,$cgid)
{
	$cgname  = group_name($cgid);
	$pgname  = group_name($pgid);
	
  $sql  = "DELETE FROM assign_groups ";
  $sql .= "WHERE assignor_gid='$pgid' ";
  $sql .= "AND assignee_gid='$cgid'";
  $result = $_ENV['dbi']->extended->autoExecute("assign_groups",
    null, MDB2_AUTOQUERY_DELETE,
    "assignor_gid=" . $_ENV['dbi']->quote($pgid, "integer") . " AND " .
    "assignee_gid=" . $_ENV['dbi']->quote($cgid, "integer")); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  // did we change anything?
	if ($result != 0) {
		$_ENV['error']->logger("$pgname removed $cgname","groups");	
		return TRUE;
	} 
  return FALSE;
}
/* }}} */

/* {{{ Function: group_assigns */
/**
 * Retrieve assignable groups for issue reassignment
 *
 * @param integer $gid ID of group to add user to
 * @return array
 */
function group_assigns($gid)
{
	$assign_groups = array();
	
	$sql = "SELECT a.assignee_gid,g.name ";
  $sql .= "FROM assign_groups AS a ";
  $sql .= "LEFT JOIN groups AS g ON g.gid=a.assignee_gid ";
  $sql .= "WHERE a.assignor_gid=? ";
  $sql .= "ORDER BY g.name ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  while(list($gid,$name) = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
	  $assign_groups[$gid] = $name;
  }
  $result->free();
  $stmt->free();
	
	return $assign_groups;
}
/* }}} */

/* {{{ Function: group_fields */
/**
 * Retrieve group-defined fields
 *
 * @param integer $gid ID of group to add user to
 * @return array
 */
function group_fields($gid)
{
	$fields = array();

  $sql  = "SELECT g.gfid,c.type,g.name,g.label,c.tpl_path AS tpl ";
  $sql .= "FROM group_fields AS g ";
  $sql .= "LEFT JOIN custom_field_types AS c ON c.typeid=g.typeid ";
  $sql .= "WHERE g.gid=?  ";
  $sql .= "ORDER BY g.gfid";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
	while ($row = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
 	  $fields[$row['gfid']] = array("label" => $row['label'],
 								                  "name"  => $row['name'],
 								                  "type"  => $row['type'],
 								                  "tpl"	  => _THEMES_ . $row['tpl']);
  }
  $result->free();
  $stmt->free();

	return $fields;
}
/* }}} */	

/* {{{ Function: group_field_values */
/**
 * Retrieve group-defined fields
 *
 * @param integer $gfid ID of group-defined field
 * @return mixed
 */
function group_field_values($gfid)
{
    $stmt = $_ENV['dbi']->prepare("SELECT value_idx,value,is_default FROM group_field_values WHERE gfid=?");
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gfid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $vals = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if(count($vals) > 0){
    	foreach($vals as $key => $value){
    		$values[$value['value_idx']]['value']   = $value['value'];
    		$values[$value['value_idx']]['default'] = $value['is_default'];
    	}
    } else {
    	$values = null;
    }
    return $values;
}
/* }}} */	

/* {{{ Function: group_useradd */
/**
 * Add a user to a group
 *
 * @param integer $userid ID of user to add to group
 * @param integer $gid ID of group to add user to
 */
function group_useradd($userid,$gid)
{
  // check to make sure this user isn't already in the group
  $sql  = "SELECT userid ";
  $sql .= "FROM group_users ";
  $sql .= "WHERE gid=? ";
  $sql .= "AND userid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid, $userid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $already_in_group = ($result->numRows() > 0);
  $result->free();
  $stmt->free();

  if ($already_in_group) return;

  // form insert array
  $insert = array();
  $types = array();
  $insert["userid"]       = $userid;
  $types[] = 'integer';
  $insert["gid"]          = $gid;
  $types[] = 'integer';
  $insert["show_group"]   = "t";
  $types[] = 'text';
  $result = $_ENV['dbi']->extended->autoExecute("group_users",
    $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  $uname  = username($userid);
  $gname  = group_name($gid);
  $_ENV['error']->logger("$uname added to $gname.","users");
}
/* }}} */

/* {{{ Function: group_userdel */
/**
 * Remove a user from a group
 *
 * @param integer $userid ID of user to remove from group
 * @param integer $gid ID of group to remove user from
 * @return boolean
 */
function group_userdel($userid,$gid)
{
	$uname  = username($userid);
	$gname  = group_name($gid);
	
  $result = $_ENV['dbi']->extended->autoExecute("group_users",
    null, MDB2_AUTOQUERY_DELETE,
    "gid=" . $_ENV['dbi']->quote($gid, "integer") . " AND " .
    "userid=" . $_ENV['dbi']->quote($userid, "integer")); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  // did we change anything?
	if ($result != 0) {
		$_ENV['error']->logger("$uname removed $gname","users");	
		return TRUE;
	} else {
		return FALSE;
	}
}
/* }}} */

/* {{{ Function: group_members */
/**
 * Retrieve array of uids for group
 *
 * @param integer $gid ID of group to get members list for
 * @param string $ignore INTERNAL FUNCTION USE ONLY
 * @return array
 */
function group_members($gid,$ignore = null)
{
  // initialize empty array
  $members = array();

  // retrieve users that are directly part of group
  $sql  = "SELECT userid ";
  $sql .= "FROM group_users ";
  $sql .= "WHERE gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  while ($row = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
    array_push($members,$row[0]);
  }
  $result->free();
  $stmt->free();

  // retrieve users from parent group
  if ($ignore != "parent") {
    $sql  = "SELECT s.parent_gid ";
    $sql .= "FROM sub_groups s,groups g ";
    $sql .= "WHERE s.parent_gid=g.gid ";
    $sql .= "AND s.child_gid=? ";
    $sql .= "AND g.prop_user='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $parent = $result->fetchRow(MDB2_FETCHMODE_ORDERED);
    $result->free();
    $stmt->free();

    if ($parent) {
      $parent_members = group_members($parent[0],"child");
  
      foreach ($parent_members as $key => $val) {
        if (!in_array($key,$members)) {
          array_push($members,$key);
        }
      }
    }
  }

  // retrieve users from child groups that are set
  // to propogate to parent group
  if ($ignore != "child") {
    $sql  = "SELECT child_gid ";
    $sql .= "FROM sub_groups ";
    $sql .= "WHERE parent_gid=? ";
    $sql .= "AND prop_user='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

    while ($child = $result->fetchOne()) {
      $child_members = group_members($child[0],"parent");

      foreach ($child_members as $key => $val) {
        if (!in_array($key,$members)) {
          array_push($members,$key);
        }
      }
    }

    $result->free();
    $stmt->free();
  }

  $group_users = array();

  // now turn $members array into associative array
  // of userid/username pairs
  $mbrs = implode(",",$members);
  if ($mbrs) {
    $sql  = "SELECT userid,username ";
    $sql .= "FROM users ";
    $sql .= "WHERE userid IN ($mbrs) "; // safe -- this is from db queries above
    $sql .= "ORDER BY username";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute();
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  
    $group_users = array();
    while ($user = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
      $userid	= $user[0];
      $username = $user[1];
      $group_users[$userid] = $username;
    }
    $result->free();
    $stmt->free();
  }

  return $group_users;
}
/* }}} */

/* {{{ Function: group_child */
/**
 * Retrieve all descendants of a group
 *
 * @param integer $gid ID of group to use as parent
 * @param array $groups Reference to array of related groups
 */
function group_child($gid, &$assign_groups)
{
	$sql = "SELECT sg.child_gid,g.name
			FROM sub_groups AS sg
			LEFT JOIN groups AS g ON g.gid=sg.child_gid
			WHERE sg.parent_gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  while (list($cgid,$name) = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
    $assign_groups[$cgid] = $name;
    group_child($cgid, $assign_groups);
	}
  $result->free();
  $stmt->free();
}
/* }}} */

/* {{{ Function: group_parent */
/**
 * Retrieve parent (higher tree-node) of a group
 *
 * @param integer $gid ID of group to use as parent
 * @param array $groups Reference to array of related groups
 */
function group_parent($gid, &$assign_groups)
{
	$sql = "SELECT sg.parent_gid,g.name
			FROM sub_groups AS sg
			LEFT JOIN groups AS g ON g.gid=sg.parent_gid
			WHERE sg.child_gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
	if (list($pgid,$name) = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
		$assign_groups[$pgid] = $name;
		group_parent($pgid, $assign_groups);
	}
  $result->free();
  $stmt->free();
}
/* }}} */

/* {{{ Function: possible_viewer_groups */
/**
 * Retrieve parents/siblings/children (related tree-nodes) of a group
 *
 * @param integer $gid ID of group
 * @return array
 */
function possible_viewer_groups($gid)
{
	$groups = array();
	
	$sql  = "SELECT gid,name ";
	$sql .= "FROM groups ";
	$sql .= "WHERE gid <> ? ";
	$sql .= "AND active='t'";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
	$group_results = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();
	foreach($group_results as $key => $row){
		$groups[$row['gid']] = $row['name'];
	}
	
	ksort($groups);
	return $groups;
}
/* }}} */

/* {{{ Function: group_active */
/**
 * Determine if a group is active
 *
 * @param integer $gid ID of group to check
 * @return boolean
 */
function group_active($gid)
{
  $rv = FALSE;

	$sql  = "SELECT count(active) ";
	$sql .= "FROM groups ";
	$sql .= "WHERE active='t' ";
	$sql .= "AND gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
	if ($result->fetchOne() > 0) $rv = TRUE;
  $result->free();
  $stmt->free();

  return $rv;
}
/* }}} */

/* {{{ Function: group_auto_reopen */
/**
 * Determine if a group is has auto_reopen enabled
 *
 * @param integer $gid ID of group to check
 * @return boolean
 */
function group_auto_reopen($gid)
{
  $rv = FALSE;

	$sql  = "SELECT count(auto_reopen) ";
	$sql .= "FROM groups ";
	$sql .= "WHERE auto_reopen='t' ";
	$sql .= "AND gid=?";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  if ($result->fetchOne() != 0) $rv = TRUE;
  $result->free();
  $stmt->free();

	return $rv;
}
/* }}} */

/* {{{ Function: group_public */
/**
 * Determine if a group is public
 *
 * @param integer $gid ID of group to check
 * @return boolean
 */
function group_public($gid)
{
  $rv = FALSE;

	$sql  = "SELECT count(public) ";
	$sql .= "FROM groups ";
	$sql .= "WHERE public='t' ";
	$sql .= "AND gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
	if ($result->fetchOne() > 0) $rv = TRUE;

  $result->free();
  $stmt->free();
	return $rv;
}
/* }}} */

/* {{{ Function: show_group */
/**
 * Determine if we should show the group to the user or not
 *
 * @param integer $gid ID of group
 * @return boolean
 */
function show_group($gid)
{
  $rv = TRUE;

	$sql  = "SELECT show_group ";
	$sql .= "FROM group_users ";
	$sql .= "WHERE gid=? ";
	$sql .= "AND userid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($gid, $_SESSION['userid']));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $show = $result->fetchOne();
  if ($show == "f") $rv = FALSE;
  $result->free();
  $stmt->free();

	return $rv;
}
/* }}} */

/* {{{ Function: escalation_groups */
/**
 * Retrieve array of escalation groups for a group
 *
 * @param integer $gid ID of group
 * @return array
 */
function escalation_groups($gid)
{
	$egroups = array();

	$sql  = "SELECT g.name,e.egid ";
	$sql .= "FROM groups g, escalation_points e ";
	$sql .= "WHERE e.gid=? ";
  $sql .= "AND e.egid=g.gid";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);  
  $groups = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();
  if (!is_null($groups)) {
    foreach ($groups as $group) {
      $egroups[$group['egid']] = $group['name'];
		}
	}

	return $egroups;
}
/* }}} */

/* {{{ Function: group_statuses */
/**
 * Retrieve array of statuses for a group
 *
 * @param integer $gid ID of group
 * @param string $ignore INTERNAL FUNCTION USE ONLY
 * @return array
 */
function group_statuses($gid,$ignore = null)
{
  $allowed = implode(",",fetch_status(array(
    TYPE_WAITING,
    TYPE_LONG_TERM,
    TYPE_CLOSED
  )));

  $statuses = array();

  $sql  = "SELECT g.sid,s.status ";
  $sql .= "FROM group_statuses g, statuses s ";
  $sql .= "WHERE g.gid=? ";
  $sql .= "AND g.sid = s.sid ";
  $sql .= "AND s.sid IN ($allowed) "; // safe: constructed above
  $sql .= "ORDER BY s.status";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  while (list($sid,$status) = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
    $statuses[$sid] = $status;
  }
  $result->free();
  $stmt->free();

  if ($ignore != "parent") {
    $sql  = "SELECT s.parent_gid ";
    $sql .= "FROM sub_groups s,groups g ";
    $sql .= "WHERE s.parent_gid=g.gid ";
    $sql .= "AND s.child_gid=? ";
    $sql .= "AND g.prop_status='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    if ($parent = $result->fetchOne()) {
      $s = group_statuses($parent,"child");
      
      if (count($s) > 0) {
        foreach ($s as $sid => $status) {
          if (!array_key_exists($sid,$statuses)) {
            $statuses[$sid] = $status;
          }
        }
      }
    }
    $result->free();
    $stmt->free();
  }

  if ($ignore != "child") {
    $sql  = "SELECT child_gid ";
    $sql .= "FROM sub_groups ";
    $sql .= "WHERE parent_gid=? ";
    $sql .= "AND prop_status='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

    while ($child = $result->fetchOne()) {
      $s = group_statuses($child);

      if (count($s) > 0) {
        foreach ($s as $sid => $status) {
          if (!array_key_exists($sid,$statuses)) {
            $statuses[$sid] = $status;
          }
        }
      }
    }

    $result->free();
    $stmt->free();
  }

  // sort the statuses by name, naturally
  natcasesort($statuses);
  return $statuses;
}
/* }}} */

/* {{{ Function: group_istatuses */
/**
 * Retrieve array of internal statuses for a group
 *
 * @param integer $gid ID of group
 * @param string $ignore INTERNAL FUNCTION USE ONLY
 * @return array
 */
function group_istatuses($gid,$ignore = null)
{
  $statuses = array();

  $sql  = "SELECT g.sid,s.status ";
  $sql .= "FROM group_istatuses g, statuses s ";
  $sql .= "WHERE g.gid=? ";
  $sql .= "AND g.sid = s.sid ";
  $sql .= "ORDER BY s.status";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  while (list($sid,$status) = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
    $statuses[$sid] = $status;
  }
  $result->free();
  $stmt->free();

  if ($ignore != "parent") {
    $sql  = "SELECT s.parent_gid ";
    $sql .= "FROM sub_groups s,groups g ";
    $sql .= "WHERE s.parent_gid=g.gid ";
    $sql .= "AND s.child_gid=? ";
    $sql .= "AND g.prop_status='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    if ($parent = $result->fetchOne()) {
      $s = group_istatuses($parent,"child");
      
      if (count($s) > 0) {
        foreach ($s as $sid => $status) {
          if (!array_key_exists($sid,$statuses)) {
            $statuses[$sid] = $status;
          }
        }
      }
    }
    $result->free();
    $stmt->free();
  }

  if ($ignore != "child") {
    $sql  = "SELECT child_gid ";
    $sql .= "FROM sub_groups ";
    $sql .= "WHERE parent_gid=? ";
    $sql .= "AND prop_status='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    while ($child = $result->fetchOne()) {
      $s = group_istatuses($child);

      if (count($s) > 0) {
        foreach ($s as $sid => $status) {
          if (!array_key_exists($sid,$statuses)) {
            $statuses[$sid] = $status;
          }
        }
      }
    }
    $result->free();
    $stmt->free();
  }

  // sort the statuses alphabetically
  natcasesort($statuses);
  return $statuses;
}
/* }}} */

/* {{{ Function: group_categories */
/**
 * Retrieve array of categories for a group
 *
 * @param integer $gid ID of group
 * @param string $ignore INTERNAL FUNCTION USE ONLY
 * @return array
 */
function group_categories($gid, $ignore = null)
{
  $categories = array();

  $sql  = "SELECT g.cid,c.category ";
  $sql .= "FROM group_categories g, categories c ";
  $sql .= "WHERE g.gid=? ";
  $sql .= "AND g.cid = c.cid ";
  $sql .= "ORDER BY c.category";

  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  while(list($cid,$category) = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
    $categories[$cid] = $category;
  }
  $result->free();
  $stmt->free();

  if ($ignore != "parent") {
    $sql  = "SELECT s.parent_gid ";
    $sql .= "FROM sub_groups s,groups g ";
    $sql .= "WHERE s.parent_gid=g.gid ";
    $sql .= "AND s.child_gid=? ";
    $sql .= "AND g.prop_category='t'";

    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    if ($parent = $result->fetchOne()) {
      $c = group_categories($parent,"child");
      
      if (count($c) > 0) {
        foreach ($c as $sid => $category) {
          if (!array_key_exists($sid,$categories)) {
            $categories[$sid] = $category;
          }
        }
      }
    }
    $result->free();
    $stmt->free();
  }

  if ($ignore != "child") {
    $sql  = "SELECT child_gid ";
    $sql .= "FROM sub_groups ";
    $sql .= "WHERE parent_gid=? ";
    $sql .= "AND prop_category='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    while ($child = $result->fetchOne()) {
      $c = group_categories($child,"parent");

      if (count($c) > 0) {
        foreach ($c as $sid => $category) {
          if (!array_key_exists($sid,$categories)) {
            $categories[$sid] = $category;
          }
        }
      }
    }
    $result->free();
    $stmt->free();
  }

  // sort the categories alphabetically
  natcasesort($categories);
  return $categories;
}
/* }}} */

/* {{{ Function: group_products */
/**
 * Retrieve array of products for a group
 *
 * @param integer $gid ID of group
 * @param string $ignore INTERNAL FUNCTION USE ONLY
 * @return array
 */
function group_products($gid,$ignore = null)
{
	$products = array();

	$sql  = "SELECT p.pid,p.product ";
	$sql .= "FROM group_products g, products p ";
  $sql .= "WHERE p.pid=g.pid ";
	$sql .= "AND g.gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
	$result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  while(list($pid,$product) = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
    $products[$pid] = $product;
  }
  $result->free();
  $stmt->free();

  if ($ignore != "parent") {
    $sql  = "SELECT s.parent_gid ";
    $sql .= "FROM sub_groups s,groups g ";
    $sql .= "WHERE s.parent_gid=g.gid ";
    $sql .= "AND s.child_gid=? ";
    $sql .= "AND g.prop_product='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    if ($parent = $result->fetchOne()) {
      $p = group_products($parent, "child");
      
      if (count($p) > 0) {
        foreach ($p as $sid => $product) {
          if (!array_key_exists($sid,$products)) {
            $products[$sid] = $product;
          }
        }
      }
    }
    $result->free();
    $stmt->free();
  }

  if ($ignore != "child") {
    $sql  = "SELECT child_gid ";
    $sql .= "FROM sub_groups ";
    $sql .= "WHERE parent_gid=? ";
    $sql .= "AND prop_product='t' ";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    while ($child = $result->fetchOne()) {
      $p = group_products($child, "parent");

      if (count($p) > 0) {
        foreach ($p as $sid => $product) {
          if (!array_key_exists($sid,$products)) {
            $products[$sid] = $product;
          }
        }
      }
    }
    $result->free();
    $stmt->free();
  }

  // sort products
  natcasesort($products);
	return $products;
}
/* }}} */

/* {{{ Function: user_in_group */
/**
 * Determine if a user is a member of given group
 *
 * @param integer $gid ID of group
 * @param integer $userid ID of user
 * @return boolean
 */
function user_in_group($gid,$userid = null)
{
  if (empty($userid)) {
    $userid = $_SESSION['userid'];
  }

  $members = group_members($gid);
  if (is_array($members)) {
    if (array_key_exists($userid,$members)) {
      return TRUE;
    }
  }

  return FALSE;
}
/* }}} */

/* {{{ Function: is_parent */
/**
 * Determines if a group is the parent group of another
 *
 * @param integer $parent Possible parent group
 * @param integer $child Possible child group
 * @return boolean
 */
function is_parent($parent,$child)
{
  $rv = FALSE;

  $sql  = "SELECT parent_gid ";
  $sql .= "FROM sub_groups ";
  $sql .= "WHERE parent_gid=? ";
  $sql .= "AND child_gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($parent, $child));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  if ($result->numRows() > 0) {
    $rv = TRUE;
  }
  $result->free();
  $stmt->free();

  return $rv;
}
/* }}} */

/* {{{ Function: is_child */
/**
 * Determines if a group is a child group of another
 *
 * @param integer $child Possible child group
 * @param integer $parent Possible parent group
 * @return boolean
 */
function is_child($child,$parent)
{
  $rv = FALSE;
  $sql  = "SELECT child_gid ";
  $sql .= "FROM sub_groups ";
  $sql .= "WHERE child_gid=? ";
  $sql .= "AND parent_gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($child, $parent));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  if ($result->numRows() > 0) {
    $rv = TRUE;
  }

  return $rv;
  $result->free();
  $stmt->free();
}
/* }}} */

/* {{{ Function: possible_children */
/**
 * Retrieves an array of possible sub groups
 * for the given group
 *
 * @param integer $gid ID of group
 * @return array
 */
function possible_children($gid)
{
  $groups = array();
  
  $sql  = "SELECT gid,name ";
  $sql .= "FROM groups ";
  $sql .= "ORDER BY name";
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute();
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  while (list($id,$name) = $result->fetchRow(MDB2_FETCHMODE_ORDERED)) {
    if (is_parent($id,$gid) or is_child($id,$gid) or $id == $gid) {
        continue;
    }

    $groups[$id] = $name;
  }
  $result->free();
  $stmt->free();
   
  return $groups;
}
/* }}} */

/* {{{ Function: group_issues */
/**
 * Retrieve array of issues that should be viewable by given group
 *
 * @param integer $gid ID of group
 * @param integer $limit Number of issues to return
 * @param boolean $show_registered Whether or not to show registered issues
 * @param string $ignore INTERNAL FUNCTION USE ONLY
 * @return array
 */
function group_issues($gid,$limit = null,$show_registered = TRUE,$ignore = null)
{
	if (empty($_GET['sort'])) {
		if ($_SESSION['prefs']['sort_by'] == 'g_name'){
			$_GET['sort'] = 'primary_gid';
		}else{
			$_GET['sort'] = $_SESSION['prefs']['sort_by'];
		}
  }

  // Check User Permisions
  if (!in_array($gid, $_SESSION['groups'])) {
     $_ENV['error']->push_error(_("You don't have permission to access this group. Contact the administrator."));
     redirect();
  }

  $issues = array();
  list($registered) = fetch_status(TYPE_REGISTERED);
  $closed = implode(",",fetch_status(array(TYPE_CLOSED,TYPE_AUTO_CLOSED)));

  // santitize this, since it's used literally in SQL, below
  $gid = $gid+0;

  // add parent GIDs to the list of groups we're looking at
  if ($ignore != "parent") {
    $sql  = "SELECT s.parent_gid ";
    $sql .= "FROM sub_groups s,groups g ";
    $sql .= "WHERE s.parent_gid=g.gid ";
    $sql .= "AND s.child_gid=? ";
    $sql .= "AND g.prop_issue='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    if ($parent = $result->fetchOne()) {
      $gid .= ",$parent";
    }
    $result->free();
    $stmt->free();
  }

  // add child GIDs
  if ($ignore != "child") {
    $sql  = "SELECT child_gid ";
    $sql .= "FROM sub_groups ";
    if (strpos($gid, ",") !== false) {
      $sql .= "WHERE parent_gid IN ($gid) ";
    } else {
      $sql .= "WHERE parent_gid=$gid ";
    }
    $sql .= "AND prop_issue='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute();
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    while ($child = $result->fetchOne()) {
      $gid .= ",$child";
    }
    $result->free();
    $stmt->free();
  }

  $params = array();
  $sql  = "SELECT i.issueid,i.summary,i.private";

  foreach ($_SESSION['prefs']['show_fields'] as $val) {
    switch ($val) {
    case "flags":
      continue;
      break;
    case "assigned_to":
      $sql .= ",g.assigned_to";
      break;
    case "time_open":
      $sql .= ",i.opened";    
      break;
    default:
      // sanitize $val first..
      $val = preg_replace('/[^a-zA-Z0-9]/', '_', $val);
      $sql .= ",i.$val";
      break;
    }
  }

  $sql .= " FROM issues i, issue_groups g";
  $sql .= in_array("opened_by",$_SESSION['prefs']['show_fields']) ? ", users u" : "";
  if (strpos($gid, ",") !== false) {
    $sql .= " WHERE g.gid IN ($gid) "; // ugly, but safe; see above
  } else {
    $sql .= " WHERE g.gid=? ";
    $params[] = $gid;
  }
  $sql .= "AND i.issueid = g.issueid ";
  $sql .= in_array("opened_by",$_SESSION['prefs']['show_fields']) ? "AND u.userid = i.opened_by " : "";
  $sql .= !permission_check("view_private",$gid) ? "and i.private != 't' " : "";
  $sql .= !$show_registered ? "AND i.status != '$registered' " : ""; // safe..
  $sql .= $_GET['showall'] != "true" ? "AND i.status NOT IN ($closed) " : ""; // safe..
  $sql .= " AND g.show_issue = 't' ";
  if ($_GET['sort'] == "assigned_to") {
    $sql .= "ORDER BY g.assigned_to ";
  } else if ($_GET['sort'] == "opened_by") {
    $sql .= "ORDER BY u.username ";
  } else if ($_GET['sort'] == "g_name") {
      $sql .= "ORDER BY g.gid ";
    } else {
    // sanitize this before handing it off to db
    $sort = preg_replace('/[^a-zA-Z0-9]/', '_', $_GET['sort']);
    //$sql .= "ORDER BY i.".$_GET['sort']." ";
    $sql .= "ORDER BY i.$sort ";
  }

  if ($_GET['sort'] == "severity") {
    $sql .= $_GET['reverse'] == "true" ? "DESC" : "ASC";
  } else {
    $sql .= $_GET['reverse'] == "true" ? "ASC" : "DESC";
  }
  $stmt = $_ENV['dbi']->prepare($sql); 
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute($params);
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  while ($row = $result->fetchRow(MDB2_FETCHMODE_ASSOC)) {
    $issues[$row['issueid']] = $row;
  }
  $result->free();
  $stmt->free();

  return $issues;
}


/* }}} */ 

/* {{{ Function: group_over_limit */
/**
 * Determine if a group has reached their limit
 *
 * @param integer $gid ID of group
 * @param string $type Only match given limit type
 * @return boolean
 */
function group_over_limit($gid,$type = null)
{
  if (empty($gid)) {
    return FALSE;
  }

  $sql  = "SELECT group_type,bought,start_date,end_date ";
  $sql .= "FROM groups ";
  $sql .= "WHERE gid=? ";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);  
  $data = $result->fetchRow(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();
  if (!is_null($data)) {
    extract($data);

    if (!is_null($type) and $group_type != $type) {
      return FALSE;
    }

    if ($group_type == "hours") {
      $stmt = $_ENV['dbi']->prepare("SELECT issueid from issue_groups where gid=?");
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute(array($gid));
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $issues = $result->fetchCol(); 
      $result->free();
      $stmt->free();
      if (!is_null($issues)) {
        $issues = implode(",",$issues);

        $args = array();
        $sql  = "SELECT SUM(duration) ";
        $sql .= "FROM events ";
        $sql .= "WHERE issueid IN ($issues) "; // (no quoting required)
        if (!empty($start_date) and !empty($end_date)) {
          $sql .= "AND performed_on >= ?";
          $args[] = $start_date;
        }

        $stmt = $_ENV['dbi']->prepare($sql);
        $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
        $result = $stmt->execute($args);
        $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
        $hours = $result->fetchOne(); 
        $result->free();
        $stmt->free();

        if (!is_null($hours)) {
          if ($hours >= $bought) {
            return TRUE;
          }
        }
      }
    } else if ($group_type == "issues") {
      $args = array();
      $sql  = "SELECT COUNT(issueid) ";
      $sql .= "FROM issues ";
      $sql .= "WHERE primary_gid=? ";
      $args[] = $gid;
      if (!empty($start_date) and !empty($end_date)) {
        $sql .= "AND opened >= ? ";
        $args[] = $start_date;
      }
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($args);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $issues = $result->fetchOne(); 
      $result->free();
      $stmt->free();

      if (!is_null($issues)) {
        if ($issues >= $bought) {
          return TRUE;
        }
      }
    }
  }

  return FALSE;
}
/* }}} */

/* {{{ Function: get_sub_groups */
function get_sub_groups($gid,$data)
{
  // make sure we've initialized the subgroups array in the session
  if (!is_array($_SESSION['subgroups'])) {
    $_SESSION['subgroups'] = array();
  }

  // check to see if we have this lookup cached
  /*
  if (array_key_exists("$gid:$data",$_SESSION['subgroups'])) {
    if (time() < $_SESSION['subgroups']["$gid:$data"]['expire']) {
      return $_SESSION['subgroups']["$gid:$data"]['groups'];
    }
  }
  */

  // Initialize $groups array
  $groups = array();

  // Check to see if $gid is an array
  if (is_array($gid)) {
    // if it was an array then cycle through
    // the array and send each gid back to this
    // function individually
    foreach ($gid as $groupId) {
      if (!in_array($groupId,$groups)) {
        array_push($groups,$groupId);
        $subGroups = get_sub_groups($groupId,$data);
        if (is_array($subGroups)) {
          foreach ($subGroups as $subGroupId) {
            if (!in_array($subGroupId,$groups)) {
              array_push($groups,$subGroupId);
            }
          }
        }
      }
    }
  } else {
    array_push($groups,$gid);

    $sql  = "SELECT parent_gid,child_gid ";
    $sql .= "FROM sub_groups ";
    $sql .= "WHERE (parent_gid=? OR child_gid=?) ";
    $sql .= "AND prop_$data='t'"; // I think this is OK, and this code should go soon
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid, $gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $subGroups = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($subGroups)) {
      foreach ($subGroups as $subGroup) {
        if (!empty($subGroup['parent_gid'])
        and $subGroup['parent_gid'] != $gid) {
          array_push($groups,$subGroup['parent_gid']);
        }
        if (!empty($subGroup['child_gid'])
        and $subGroup['child_gid'] != $gid) {
          array_push($groups,$subGroup['child_gid']);
        }
      }
    }
    unset($subGroups);
  }

  // Only cache the results for single group lookups
  if (!is_array($gid)) {
    $_SESSION['subgroups']["$gid:$data"] = array('expire' => time() + 300,'groups' => $groups);
  }
  return $groups;
}
/* }}} */
    
/* {{{ Function: group_summary */
/**
 * Returns group rating.
 * John Wells - 08/25/2004
 **/
function group_summary($gid)
{
  $closed = implode(",",fetch_status(array(TYPE_CLOSED,TYPE_AUTO_CLOSED)));

  $group_summary['new'] = num_new_issues($gid);
  $group_summary['open'] = num_open_issues($gid);

  $gids = implode(",", fetch_subgroups($gid, 't'));

  $sql  = "SELECT COUNT(i.issueid) ";
  $sql .= "FROM issues i, issue_groups g ";
  $sql .= "WHERE g.gid IN ($gids) "; // (no quoting needed)
  $sql .= "AND i.issueid=g.issueid ";
  $sql .= "AND i.severity=? ";
  $sql .= "AND i.status NOT IN ($closed) "; // (no quoting needed)
  $sql .= "AND g.show_issue='t'";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);

  for ($x = 1;$x < 5;$x++) {
    $result = $stmt->execute(array($x));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $count = $result->fetchOne(); 
    $result->free();
    $group_summary["sev$x"] = $count;
  }
  $stmt->free();

  $group_summary['name'] = group_name($gid);

  $group_summary['rating'] = 0;
  $stmt = $_ENV['dbi']->prepare("SELECT standing from status_reports where gid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($gid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $standing = $result->fetchOne(); 
  $result->free();
  $stmt->free();

  if (!empty($standing)) {
    $group_summary['rating'] = $standing;
  }

  $group_summary['rating'] += $group_summary['sev4'] * .5;
  $group_summary['rating'] += $group_summary['sev3'] * 1;
  $group_summary['rating'] += $group_summary['sev2'] * 2;
  $group_summary['rating'] += $group_summary['sev1'] * 3;
  
  if ($group_summary['rating'] >= 50) {
    $group_summary['standing'] = $_ENV['imgs']['urgent'];
  } else if ($group_summary['rating'] >= 25) {
    $group_summary['standing'] = $_ENV['imgs']['high'];
  } else {
    $group_summary['standing'] = $_ENV['imgs']['normal'];
  }   

  return $group_summary;   
}
/* }}} */
