<?php
/**
 * modules/reports/hooks/class.php
 *
 * Report generation class for the EnterTrack system
 *
 * @package EnterTrack
 * @subpackage Reports_Module
 */

/* DBI TODO what is this is_object($this) stuff?? */
/* XXX THIS IS DUE FOR A COMPLETE REWRITE */

if (strpos(basename(__FILE__),$_SERVER['PHP_SELF'])) {
  print "Direct module access forbidden.";
  exit;
}

/**
 * Report Generator
 *
 * Class for generating group and user reports for the EnterTrack system
 *
 * @package EnterTrack
 * @subpackage Reports_Module
 * @author Edwin Robertson {TuxMonkey}
 * @version 0.1
 * @copyright Edwin Robertson
 */
class Reports {
  /**
   * Start date used for generated reports
   *
   * @see set_date_range
   * @var integer
   */
  var $startDate;

  /**
   * End date used for generated reports
   *
   * @see set_date_range
   * @var integer
   */
  var $endDate;

  /**
   * Type of report to generate
   *
   * @see set_type
   * @var string
   */
  var $reportType = "";
  
  /**
   * Kind  of report to generate. This can be one of the following:
   * 
   * 1. Summary: This is the statistical report that shows number of tickets
   * for a group and then breaks those numbers out by categories, severities,
   * technicians, etc. depending on the options selected.
   * 
   * 2. Issue: This is a report that lists the issues matching the selected
   * criteria; for example, all issues in a 'closed' status, or all issues
   * for category 'x'.
   * 
   * @see set_kind
   * @var string
   */
   var $reportKind;

  /**
   * Name given to generated report
   *
   * @see set_name
   * @var string
   */
  var $reportName = null;
  
  /**
   * Flag to indicate whether the report should be exported
   * 
   * @see set_export
   * @var string
   */
  var $reportExport = null;
  
  /**
   * Export options
   * 
   * @see set_export_options
   * @var array
   */
  var $reportExportOptions = array();
  
  /**
   * Export file format
   * 
   * @see set_export_format
   * @var string
   */
  var $reportExportFormat = null;
  
  /**
   * Options to use when generating the report
   *
   * @see set_options
   * @var array
   */
  var $reportOptions = array();
  
  /**
   * Sorting option(s) to use when generating the report.
   * 
   * @see set_sort_options
   * @var array
   */
  var $reportSortOptions = array();

  /**
   * Time unit (in seconds) to use when calculating average and maximum times
   *
   * @see set_time_unit
   * @var integer
   */
  var $reportTimeUnit = _HOUR_;

  /**
   * Array containing all data resulting from report generation functions
   *
   * @var array
   */
  var $reportData = array();
  
  /**
   * Array containing the issue selections (e.g. the categories, 
   * statuses, products, or severities selected by the user).
   * 
   * @see set_issue_selections
   * @var array
   */
  var $reportIssueSelections = array();
  
  /**
   * Array containing the fields to output in the issue report.
   * 
   * @see set_issue_fields
   * @var array
   */
  var $reportIssueFields = array();
  
  /**
   * Array containing the technicians for which we want to run a report.
   * 
   * @see set_technician
   * @var array
   */
  var $reportTechnicians = array();
  
  /**
   * Comma seperated list of subgroups of group that report is being 
   * generated for
   *
   * @see generate_group_report
   * @var string
   */
  var $subGroups;

  /**
   * Comma seperated list of all members of the group that report is
   * being generated for
   *
   * @see generate_group_report
   * @var string
   */
  var $groupMembers;
  
  /**
   * Include closed issues in the report?
   * 
   * @see set_closed_issues
   * @var boolean;
   */
  var $closedIssues = false;

  /**
   * Constructor
   */
  function Report()
  {
  }

  /**
   * Set report type
   *
   * @param string $reportType 
   */
  function set_type($reportType)
  {
    if (isset($reportType)) {
      $this->reportType = $reportType;
    }
  }
  
  /**
   * Set report kind
   * 
   * A report  can be a 'summary' or 'issue' kind.
   * 
   * @param string $reportKind
   */
  function set_kind($reportKind)
  {
    if ($reportKind == "issue") {
        $this->reportKind = $reportKind;
    } else {
        $this->reportKind = 'summary';
    }
  }
  
  /**
   * Set export flag
   * 
   * @param bool 
   */
  function set_export($flag)
  {
    if ($flag == true) {
        $this->reportExport = $flag;
    }
  }
  
  /**
   * Set export options
   * 
   * @param array $reportExportOptions
   */
  function set_export_options($reportExportOptions)
  {
    if ($this->reportExport == true) {
        $this->reportExportOptions = $reportExportOptions;
    }
  }
  
  /**
   * Set export format
   * 
   * @param array $reportExportFormat
   */
  function set_export_format($reportExportFormat)
  {
    if (isset($reportExportFormat)) {
        $this->reportExportFormat = $reportExportFormat;
    }
  }

  /**
   * Set report date range
   *
   * Set the date range to use for the report.  The times should be given
   * in unix timestamp format.
   *
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   */
  function set_date_range($startDate,$endDate)
  {
    if (ereg("[0-9]+",$startDate)) {
      $this->startDate = $startDate;
    } else {
      return;
    }
    if (ereg("[0-9]+",$endDate)) {
      $this->endDate = $endDate;
    } else {
      $this->endDate = time();
    }
  }

  /**
   * Set report options
   * 
   * Set the report options to use for the report.  This should be an array
   * of options that will be used to generate the report.  The options that
   * are currently support are:
   *  technician
   *  category
   *  status
   *  product
   *  escalation
   *  severity
   *
   * @param array $reportOptions Options to use
   */
  function set_options($reportOptions)
  {
    if (is_array($reportOptions)) {
      $this->reportOptions = $reportOptions;
    }
  }
  
  /**
   * Set report sort options
   * 
   * Set the report sort options to use for the report.  This should be an array
   * of sorting options that will be used to generate the report.  The options
   * that are currently supported are:  
   * 	by Id
   * 	by Date Opened
   * 	by Date Opened/Product/Status
   * 	by Product/Date Opened/Status
   */
  function set_sort_options($reportSortOptions)
  {
    if (is_array($reportSortOptions)) {
      $this->reportSortOptions = $reportSortOptions;
    }
  }
  
  /**
   * Set fields to include in the issue report. 
   * 
   * Set the fields to include in the issue report's output. This should be an
   * array of fields.
   */
  function set_issue_fields($reportIssueFields)
  {
    if (is_array($reportIssueFields)) {
      $this->reportIssueFields = $reportIssueFields;
    }
  }
  
  /**
   * Set issue selections for the report
   * 
   * Set the selections for an issue report.  This should be an array of
   * selections. Currently, the following selections are posible: 
   * --categories
   * --products 
   * -- statuses 
   * --severities
   * 
   * @param array $reportIssueSelections Selections for an informational report
   */
  function set_issue_selections($reportIssueSelections)
  {
    if (is_array($reportIssueSelections)) {
        $this->reportIssueSelections = $reportIssueSelections;
    }
  }
  
  /**
   * Set technicians for which we'll run the report.
   * 
   * @param array $reportTechnicians 
   */ 
  function set_technicians($reportTechnicians)
  {
    if (is_array($reportTechnicians)) {
        $this->reportTechnicians = $reportTechnicians;
    }
  }
  
  /**
   * Set report name
   * 
   * Set the name to use for this report.  The name is used primarily as
   * a reference once the report has been saved, so make sure that the
   * name you give is descriptive enough for the report you are creating.
   *
   * @param string $reportName Name of report
   */
  function set_name($reportName)
  {
    if (!empty($reportName)) {
      $this->reportName = $reportName;
    }
  }
 
  /**
   * Set time unit to use
   *
   * Set the time unit used by report data such as average resolution
   * and average response.  This unit should be specified in seconds
   * so if you wanted a result in hours your unit should be 3600
   * (60 seconds * 60 minutes) = 3600 seconds
   *
   * @param integer $timeUnit Time unit to use for reports
   */
  function set_time_unit($timeUnit)
  {
    if (ereg("[0-9]+",$timeUnit)) {
      $this->reportTimeUnit = $timeUnit;
    }
  }
  
  /**
   * Set closed issue flag
   * 
   * Set the $closedIssues flag to true or false.
   * 
   * @param boolean $closedIssues
   */
  function set_closed_issues($closedIssues) {
    $this->closedIssues = $closedIssues ? true : false;
  }
 
  /**
   * Generate report
   *
   * Generate report for given user or group.  This is just a shortcut
   * function to the function that does the real work.
   *
   * @param integer $id ID of the user or group
   * @param string $reportTemplate Template to use for report
   */
  function generate($id,$reportTemplate = null)
  {
    if ($this->reportKind == 'summary') {
      $this->generate_group_report($id,$reportTemplate);
    } else if ($this->reportKind == 'issue') {
      $this->generate_issue_report($id, $reportTemplate);
    }
  }
  
  /**
   * Generate issue report
   * 
   * Generate an issue report for a given group based on the criteria selected
   * by the user: group; [categories|statuses|products|severities|technicians];
   * and date range.  Currently, this report will return records with the
   * following columns: issue id, issue summary, status, severity, and date
   * opened.
   * 
   * @param integer $groupId ID of group
   * @param string $reportTemplate Template to use for report
   */
  function generate_issue_report($groupId, $reportTemplate = null)
  {
    $this->reportData['template'] = !is_null($reportTemplate) 
        ? $reportTemplate
        : "reports/custom/issue_report.tpl";

    if ($groupId == 'ALL')
	$groupname = "All Groups";
    else
	$groupname = group_name($groupId);
    $this->reportData['title'] = "$groupname Issues by ".ucfirst($this->reportType)." Report";
    $this->reportData['time'] = time();
    $this->reportData['user'] = $_SESSION['userid']; 
    
    if ($this->reportType == 'technician') {
        $this->group_technician_issue($groupId);
    } elseif ($this->reportType == 'category') {
        $this->group_category_issue($groupId);
    } elseif ($this->reportType == 'severity') {
        $this->group_severity_issue($groupId);
    } elseif ($this->reportType == 'product') {
        $this->group_product_issue($groupId);
    } else {
        $this->group_status_issue($groupId);
    }
    
    // Get the fields to output in the report column headings
    $col_headings = array();
    foreach ($this->reportIssueFields as $key => $field) {
        list($t,$f) = explode(".", $field);
        if ($f == 'technician') { continue; }
        $col_headings[] = ucfirst($f);
    }
    
    if ($this->closedIssues) {
        $col_headings[] = 'Closed';
    }
    
    if ($this->reportType == 'technician') {
        $col_headings[] = 'Technician';
    }
    
    // If $this->reportExport is true, then generate the export file
    if ($this->reportExport == true) {
        $this->generate_export($groupId, $col_headings);
        $host = $_SERVER['HTTP_HOST'];
        $uri = rtrim(dirname($_SERVER['PHP_SELF']), '/\\');
        if ($this->reportExportFormat == 'csv') {
            header('Content-type: text/csv'); 
            header("Location: http://$host$uri/var/EnterTrack-files/".$this->reportExportOptions[0].".".$this->reportExportFormat);
            exit();            
        } else {
            header('Content-type: application/vnd.ms-excel');
            header("Location: http://$host$uri/var/EnterTrack-files/".$this->reportExportOptions[0].".".$this->reportExportFormat);
        }
        exit();
    } else {
        // Generate the report to the screen
	$_ENV['tpl']->assign('start', $this->startDate);
	$_ENV['tpl']->assign('end', $this->endDate);
	$_ENV['tpl']->assign('fields', $this->reportIssueFields);
        $_ENV['tpl']->assign('report', $this->reportData);
	$_ENV['tpl']->assign('type', $this->reportType);
	$_ENV['tpl']->display($this->reportData['template']);
    }

    if (in_array("save_report", $this->reportOptions)) {
        $insert = array();
        $types = array();
        $insert['name'] = empty($this->reportName) ? substr($this->reportData['title'], 0, 256) : substr($this->reportName, 0, 256);
        $types[] = 'text';
        $insert['options'] = serialize($_POST);
        $types[] = 'text';
        $insert['userid'] = $_SESSION['userid'];
        $types[] = 'integer';
        $insert['report'] = serialize($this->reportData);
        $types[] = 'text';
        $insert['savedate'] = $this->reportData['time'];
        $types[] = 'integer';
        $result = $_ENV['dbi']->extended->autoExecute("reports",
          $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
        $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    }
  }
  
  /**
   * Generate export file
   * 
   * Generate an export file for the report data.  
   * 
   * @param array $col_headings Column headings for the export
   */
  function generate_export($groupId, $col_headings)
  {
    // Generate an Excel formatted report
    if ($this->reportExportFormat == 'xls') {
        
        // check for the Spreadsheet_Excel_Writer package
        $excel = 0;
        $pathArray = explode( PATH_SEPARATOR, ini_get('include_path') );
        foreach ($pathArray as $path) {
            if (file_exists($path . "/" . 'Spreadsheet/Excel/Writer.php')) { 
                $excel = 1;
                break;
            }
        }
        
        // Include the Spreadsheet_Excel_Writer package
        if ($excel == 1) {
            require_once('Spreadsheet/Excel/Writer.php');
        } else {
            $_ENV['error']->fatal_error(_("You must install the Spreadsheet_Excel_Writer package to export in Excel format."));
        }
        
        // Create the workbook and add a worksheet
        $workbook = new Spreadsheet_Excel_Writer(_FILES_.$this->reportExportOptions[0].'.xls');
        
        if (!$workbook) {
            trigger_error("Failed to open Excel workbook for writing!", E_USER_ERROR);
            exit();
        }
        
        $worksheet = & $workbook->addWorksheet('Issues by Status');
    
        // write the column headings
        foreach ($col_headings as $col => $data) {
            $worksheet->write(0, $col, $data);
        }

        // write the issue data
        $count = 1;
        $rec_count = count($this->reportData['issueData']);  
        foreach ($this->reportData['issueData'] as $key => $record) {
            if (is_array($record)) {
                $column = 0;
                $cols = count($record) - 1;
                
                foreach ($record as $col => $field) {
                    $worksheet->write($count, $column, $field);
                    $column++;
                }
            }
            $count++;
        }
        
        // Write a timestamp and the user generating the report
        $worksheet->write($rec_count + 1, 0, "Generated: ".$_ENV['i18n']->format_datetime(time()));
        $worksheet->write($rec_count + 2, 0, "Generated by: ".ident($_SESSION['userid']));
        $worksheet->write($rec_count + 3, 0, "Issues by ".ucfirst($this->reportType)." for the ".group_name($groupId)." group");
        
        // Close the workbook
        $workbook->close();
    } else {
        // Generate a CSV formatted report
        $fh = fopen(_FILES_.strip_tags($this->reportExportOptions[0].'.csv'), 'w');
    
        // Get an exclusive lock
        if (flock($fh, LOCK_EX)) {
            // write the column headings
            foreach ($col_headings as $key => $col) {
                switch ($this->reportExportOptions[1]) {
                    case ($this->reportExportOptions[1] == ','):
                        fwrite($fh, "\"$col\"".',');
                        break;
                    case ($this->reportExportOptions[1] == '|'):
                        fwrite($fh, "\"$col\"".'|');
                        break;
                    case ($this->reportExportOptions[1] == ';'):
                        fwrite($fh, "\"$col\"".';');
                        break;
                    case ($this->reportExportOptions[1] == '#'):
                        fwrite($fh, "\"$col\"".'#');
                        break;
                    case ($this->reportExportOptions[1] == '$'):
                        fwrite($fh, "\"$col\"".'$');
                        break;
                    default:
                        fwrite($fh, "\"$col\"".',');
                        break;
                }
            }
            fwrite($fh,"\n");
        
            // write the report data
            foreach ($this->reportData['issueData'] as $key => $records) {
                if (is_array($records)) {
                    foreach ($records as $record => $field) {
                        fwrite($fh, "\"$field\"");
                            if ($record < count($records) - 1) {
                                switch ($this->reportExportOptions[1]) {
                                    case ($this->reportExportOptions[1] == ','):
                                        fwrite($fh, ',');
                                        break;
                                    case ($this->reportExportOptions[1] == '|'):
                                        fwrite($fh, '|');
                                        break;
                                    case ($this->reportExportOptions[1] == ';'):
                                        fwrite($fh, ';');
                                        break;
                                    case ($this->reportExportOptions[1] == '#'):
                                        fwrite($fh, '#');
                                        break;
                                    case ($this->reportExportOptions[1] == '$'):
                                        fwrite($fh, '$');
                                        break;
                                    default:
                                        fwrite($fh, ',');
                                        break;
                                }
                            }
                    }
                    fwrite($fh,"\n");
                }
            }
            fwrite($fh, "Generated: ".$_ENV['i18n']->format_datetime(time())."\n");
            fwrite($fh, "Generated by: ".ident($_SESSION['userid'])."\n");
            fwrite($fh, "Issues by ".ucfirst($this->reportType)." for the ".group_name($groupId)." group");
        }
        flock($fh, LOCK_UN);
        fclose($fh);
    }
  }
         
  /**
   * Generate group report
   *
   * Generate report for given group using the options and date range
   * that were given.  If no options are given then only the basic
   * group information will be returned.  If no date range is given
   * then all data from creation of the group will be used.
   *
   * @param integer $groupId ID of group
   * @param string $reportTemplate Template to use for report
   */
  function generate_group_report($groupId,$reportTemplate = null)
  {
  	$_SESSION['DAJ_IGROUPID'] = $groupId;
  	$_SESSION['DAJ_AGROUPID'] = $arrGroupId = get_sub_groups($groupId,"issue");
    // retrieve sub groups and format for queries
    $this->subGroups = implode(",",$arrGroupId);
    $groupMembers = group_members($groupId);
    $this->groupMembers = implode(",",array_keys($groupMembers));

    $this->reportData['template'] = !is_null($reportTemplate)
      ? $reportTemplate
      : "reports/group_report.tpl";
    $this->reportData['title'] = group_name($groupId)." Report";
    $this->reportData['time'] = time();
    $this->reportData['user'] = $_SESSION['userid'];
   
    if (isset($this->startDate)) {
      $txtStartDate = $_ENV['i18n']->format_datetime($this->startDate);
      $txtEndDate = $_ENV['i18n']->format_datetime($this->endDate);
      $this->reportData['title'] .= " ($txtStartDate "._("to")." $txtEndDate)";
    }
    
    // 06/29/04 Dave Joyce
    // iterate through possible array.
    // 08/17/04
    //foreach ($arrGroupId as $groupId) {
    	$this->group_data($groupId);
	    if (in_array("technician",$this->reportOptions)) {
	    	$this->group_tech_data($groupId);
	    }
	    if (in_array("category",$this->reportOptions)) {
	    	$this->group_category_data($groupId);
	    }
	    if (in_array("product",$this->reportOptions)) {
	    	$this->group_product_data($groupId);
	    }
	    if (in_array("severity",$this->reportOptions)) {
	    	$this->group_severity_data($groupId);
	    }
	    if (in_array("status",$this->reportOptions)) {
	    	$this->group_status_data($groupId);
	    }
	    if (in_array("escalation",$this->reportOptions)) {
	    	$this->group_escalation_data($groupId);
	    }
    //}
    
    // 09/08/04 Dave Joyce
    // Added explicit report saving option.
	  if (in_array("save_report", $this->reportOptions)) {
      $insert = array();
      $types = array();
		  $insert['name'] = empty($this->reportName) ? substr($this->reportData['title'], 0, 256) : substr($this->reportName, 0, 256);
      $types[] = 'text';
		  $insert['options'] = serialize($_POST);
      $types[] = 'text';
		  $insert['userid'] = $_SESSION['userid'];
      $types[] = 'integer';
		  $insert['report'] = serialize($this->reportData);
      $types[] = 'text';
		  $insert['savedate'] = $this->reportData['time'];
      $types[] = 'integer';
		  $result = $_ENV['dbi']->extended->autoExecute("reports",
            $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
          $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
	  }
    $_ENV['tpl']->assign('report',$this->reportData);
    $_ENV['tpl']->display($this->reportData['template']);
  }

  /**
   * Average resolution time
   *
   * Retrieve the average resolution time for issues within a group that
   * were opened during the given date range.  If date range is not given
   * then all issues within the group.  If $breakdown is set to TRUE then
   * the resulting data will be split into the average resolution time 
   * for each member of the group.
   *
   * @param integer $groupId ID of group
   * @param boolean $breakdown Breakdown results into each group member
   * @param integer $startDate Start date for report
   * @param integer $endDate End date for report
   * @return integer|array
   */
  function group_average_resolution($groupId,$breakdown = FALSE,$startDate = null,$endDate = null)
  {
    $params = array();
    $sql  = $breakdown 
      		? "SELECT ig.assigned_to,AVG(it.closed - it.opened) AS average "
      		: "SELECT AVG(it.closed - it.opened) AS average ";
    if (in_array("use_modified", $this->reportOptions)) {
  		$sql .= "FROM 
    		(SELECT
    		  issueid,
    		  CASE WHEN real_opened IS NULL
    		   THEN opened
    		   ELSE real_opened
			  END AS opened,
    		  CASE WHEN real_closed IS NULL
    		   THEN closed
    		   ELSE real_closed
			  END AS closed
    		 FROM issues) AS it ";
    } else {
  		$sql .= "FROM issues AS it ";
    }

    $sql .= "JOIN issue_groups AS ig ON ig.issueid=it.issueid ";
    $sql .= "WHERE ";
    if ($groupId != 'ALL') {
       $sql .= "ig.gid = ? AND ";
       $params[] = $groupId;
    } 
    $sql .= "it.closed <> 0 ";

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }

    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND it.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }

    if ($breakdown) {
      $sql .= "GROUP BY ig.assigned_to";
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $average = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
      $result->free();
      $stmt->free();
      if (is_array($average)) {
        if (is_object($this)) {
          foreach ($average as $tech) {
            $this->reportData['techData'][$tech['assigned_to']]['avgResolution'] = $tech['average'];
            unset($tech);
          }
        } else {
          return $average;
        }
      }
    } else {
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $average = $result->fetchOne(); 
      $result->free();
      $stmt->free();
      if (is_object($this)) {
        $this->reportData['groupData']['avgResolution'] = $average;
      } else {
        return $average;
      }
    }
    unset($average);
  }

  /**
   * Average time for first response
   *
   * Retrieve the average time for first response for issues with a group
   * that were opened during the given date range.  If date range is not
   * given then all issues within the group.  If $breakdown is set to TRUE
   * then the resulting data will be split into the average response time
   * for each member of the group.
   *
   * @param integer $groupId ID of group
   * @param boolean $breakdown Breakdown results into each group member
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return mixed
   */
  function group_average_response($groupId,$breakdown = FALSE,$startDate = null,$endDate = null)
  {
    $params = array();
    $sql  = $breakdown
      ? "SELECT g.assigned_to,AVG(g.first_response - i.opened) AS average "
      : "SELECT AVG(g.first_response - i.opened) ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM
    			(SELECT
    			  issueid,
    			  CASE WHEN real_opened IS NULL
    			    THEN opened
    				ELSE real_opened
    			  END AS opened
    			FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";
    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND g.first_response != 0 ";

    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }

    if ($breakdown) {
      if (is_object($this)) {
        $sql .= "AND g.assigned_to IN ({$this->groupMembers}) "; // safeish
      } else {
        $groupMembers = group_members($groupId);
        $groupMembers = implode(",",array_keys($groupMembers));
        $sql .= "AND g.assigned_to IN ({$groupMembers}) "; // safeish
      }
      $sql .= "GROUP BY g.assigned_to";
      
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $average = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
      $result->free();
      $stmt->free();
      if (is_array($average)) {
        if (is_object($this)) {
          foreach ($average as $tech) {
            $this->reportData['techData'][$tech['assigned_to']]['avgResponse']
              = $tech['average'];
          }
        } else {
          return $average;
        }
      }
    } else {      
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $average = $result->fetchOne(); 
      $result->free();
      $stmt->free();
      if (is_object($this)) {
        $this->reportData['groupData']['avgResponse'] = $average;
      } else {
        return $average;
      }
    }
    unset($average);
  }

  /**
   * Maximum resolution time
   *
   * Retrieve the maximum resolution time for issues within a group that
   * were opened during the given date range.  If date range is not given
   * then all issues within the group.  If $breakdown is set to TRUE then
   * the resulting data will be split into the maximum resolution time 
   * for each member of the group.
   *
   * @param integer $groupId ID of group
   * @param boolean $breakdown Breakdown results into each group member
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return mixed
   */
  function group_maximum_resolution($groupId,$breakdown = FALSE,$startDate = null,$endDate = null)
  {
    $params = array();
    $sql  = $breakdown
      		? "SELECT g.assigned_to,MAX(i.closed - i.opened) AS maximum "
      		: "SELECT MAX(i.closed - i.opened) ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    			(SELECT
				  issueid,
    		  	  CASE WHEN real_opened IS NULL
    		       THEN opened
    		   	   ELSE real_opened
			  	  END AS opened,
    		      CASE WHEN real_closed IS NULL
    		   	   THEN closed
    		   	   ELSE real_closed
			  	  END AS closed
    		 	FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";

    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND i.closed != 0 ";
    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }

    if ($breakdown) {
      if (is_object($this)) {
        $sql .= "AND g.assigned_to IN ({$this->groupMembers}) "; // safeish
      } else {
        $groupMembers = group_members($groupId);
        $groupMembers = implode(",",array_keys($groupMembers));
        $sql .= "AND g.assigned_to IN ({$groupMembers}) "; // safeish
      }
      $sql .= "GROUP BY g.assigned_to";
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $maximum = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
      $result->free();
      $stmt->free();
      if (is_array($maximum)) {
        if (is_object($this)) {
          foreach ($maximum as $tech) {
            $this->reportData['techData'][$tech['assigned_to']]['maxResolution'] 
              = $tech['maximum'];
          }
        } else {
          return $maximum;
        }
      }
    } else {
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $maximum = $result->fetchOne(); 
      $result->free();
      $stmt->free();
      if (is_object($this)) {
        $this->reportData['groupData']['maxResolution'] = $maximum;
      } else {
        return $maximum;
      }
    }
    unset($maximum);
  }

  /**
   * Maximum time for first response
   *
   * Retrieve the maximum response time for issues within a group that
   * were opened during the given date range.  If date range is not given
   * then all issues within the group.  If $breakdown is set to TRUE then
   * the resulting data will be split into the maximum response time for
   * each member of the group.
   *
   * @param integer $groupId ID of group
   * @param boolean $breakdown Breakdown results into each group member
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return mixed
   */
  function group_maximum_response($groupId,$breakdown = FALSE,$startDate = null,$endDate = null)
  {
    $params = array();
    $sql  = $breakdown
      		? "SELECT g.assigned_to,MAX(g.first_response - i.opened) AS maximum "
      		: "SELECT MAX(g.first_response - i.opened) ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM
    			(SELECT
    			  issueid,
    			  CASE WHEN real_opened IS NULL
    			    THEN opened
    				ELSE real_opened
    			  END AS opened
    			FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";

    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND g.first_response != 0 ";
    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    if ($breakdown) {
      if (is_object($this)) {
        $sql .= "AND g.assigned_to IN ({$this->groupMembers}) "; // safeish
      } else {
        $groupMembers = group_members($groupId);
        $groupMembers = implode(",",array_keys($groupMembers));
        $sql .= "AND g.assigned_to IN ({$groupMembers}) "; // safeish
      }
      $sql .= "GROUP BY g.assigned_to";
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $maximum = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
      $result->free();
      $stmt->free();
      if (is_array($maximum)) {
        if (is_object($this)) {
          foreach ($maximum as $tech) {
            $this->reportData['techData'][$tech['assigned_to']]['maxResponse'] 
              = $tech['maximum'];
          }
        } else {
          return $maximum;
        }
      }
    } else {
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $maximum = $result->fetchOne(); 
      $result->free();
      $stmt->free();
      if (is_object($this)) {
        $this->reportData['groupData']['maxResponse'] = $maximum;
      } else {
        return $maximum;
      }
    }
    unset($maximum);
  }

  /**
   * Retrieves the number of issues assigned to each member of a group
   *
   * @param integer $groupId ID of group
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return integer
   */
  function group_number_assigned($groupId,$startDate = null,$endDate = null)
  {
    $params = array();
    $sql  = "SELECT assigned_to,COUNT(issueid) AS count ";
    $sql .= "FROM issue_groups ";
    $sql .= "WHERE 1 = 1";
    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    }

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
      $groupMembers = $this->groupMembers;
    } else {
      $groupMembers = group_members($groupId);
      $groupMembers = implode(",",array_keys($groupMembers));
    }

    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    $sql .= "AND assigned_to IN ({$groupMembers}) "; // safeish

    $sql .= "GROUP BY assigned_to ";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $assigned = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($assigned)) {
      if (is_object($this)) {
        foreach ($assigned as $tech) {
          $this->reportData['techData'][$tech['assigned_to']]['assigned'] 
            = $tech['count'];
        }
      } else {
        return $assigned;
      }
    }
    unset($assigned);
  }
  
  /**
   * Number of events entered
   *
   * Retrieve number of events entered by members of the given group during
   * the given date range, if no data range is specified then a count of all
   * events entered into the group will be returned.  If $breakdown is set 
   * to TRUE then the resulting data will be split into a count for each 
   * member of the group.
   *
   * @param integer $groupId ID of group
   * @param boolean $breakdown Breakdown results into each group member
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return mixed
   */
  function group_number_events($groupId,$breakdown = FALSE,$startDate = null,$endDate = null)
  {
    $params = array();
    $sql  = $breakdown
      ? "SELECT e.userid,COUNT(e.eid) AS count "
      : "SELECT COUNT(e.eid) ";
    $sql .= "FROM issues i, issue_groups g,events e ";
    $sql .= "WHERE g.issueid = i.issueid ";
    $sql .= "AND e.issueid = i.issueid ";

    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND e.performed_on BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }

    if ($breakdown) {
      if (is_object($this)) {
        $sql .= "AND e.userid IN ({$this->groupMembers}) "; // safeish
      } else {
        $groupMembers = group_members($groupId);
        $groupMembers = implode(",",array_keys($groupMembers));
        $sql .= "AND e.userid IN ({$groupMembers}) "; // safeish
      }
      $sql .= "GROUP BY e.userid ";
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $events = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
      $result->free();
      $stmt->free();
      if (is_array($events)) {
        if (is_object($this)) {
          foreach ($events as $tech) {
            $this->reportData['techData'][$tech['userid']]['events'] 
              = $tech['count'];
          }
        } else {
          return $events;
        }
      }
    } else {
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $events = $result->fetchOne(); 
      $result->free();
      $stmt->free();
      if (is_object($this)) {
        $this->reportData['groupData']['events'] = $events;
      } else {
        return $events;
      }
    }
    unset($events);
  }

  /**
   * Number of hours logged
   *
   * Retrieve number of hours entered by members of the given group during
   * the given date range, if no date range is specified then the number of
   * hours entered since creation of group will be returned.  If $breakdown
   * is set to TRUE then the resulting data will be split into the number
   * of hours entered by each member of the group.
   *
   * @param integer $groupId ID of group
   * @param boolean $breakdown Breakdown results into each group member
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return mixed
   */
  function group_number_hours($groupId,$breakdown = FALSE,$startDate = null,$endDate = null)
  {
    $params = array();
    $sql  = $breakdown
      ? "SELECT e.userid,SUM(e.duration) AS hours "
      : "SELECT SUM(e.duration) ";
    $sql .= "FROM issues i, issue_groups g,events e ";
    $sql .= "WHERE g.gid = ? ";
    $params[] = $gropuId;
    $sql .= "AND g.issueid = i.issueid ";
    $sql .= "AND e.issueid = i.issueid ";

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND e.performed_on BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }

    if ($breakdown) {
      if (is_object($this)) {
        $sql .= "AND e.userid IN ({$this->groupMembers}) "; // safeish
      } else {
        $groupMembers = group_members($groupId);
        $groupMembers = implode(",",array_keys($groupMembers));
        $sql .= "AND e.userid IN ({$groupMembers}) "; // safeish
      }
      $sql .= "GROUP BY e.userid ";
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $hours = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
      $result->free();
      $stmt->free();
      if (is_array($hours)) {
        if (is_object($this)) {
          foreach ($hours as $tech) {
            $this->reportData['techData'][$tech['userid']]['hours'] = $tech['hours'];
          }
        } else {
          return $hours;
        }
      }
    } else {
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $hours = $result->fetchOne(); 
      $result->free();
      $stmt->free();
      if (is_object($this)) {
        $this->reportData['groupData']['hours'] += $hours;
      } else {
        return $hours;
      }
    }
    unset($hours);
  }

  /**
   * Number of issues opened
   *
   * Retrieve number of issues opened in given group during given date range,
   * if no date range is given then all issues in the group will be counted.
   * If $breakdown is set to TRUE then the resulting data will be split into
   * the number of issues opened by each member of the group.
   *
   * @param integer $groupId ID of group
   * @param boolean $breakdown Breakdown results into each group member
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return mixed
   */
  function group_number_opened($groupId,$breakdown = FALSE,$startDate = null,$endDate = null)
  {
    $params = array();
    $sql  = $breakdown
      		? "SELECT i.opened_by,COUNT(i.issueid) AS count "
      		: "SELECT COUNT(i.issueid) ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM
    			(SELECT
    			 issueid,primary_gid,opened_by,
    			 CASE WHEN real_opened IS NULL
    			  THEN opened
    			  ELSE real_opened
    			 END AS opened
    			FROM issues) AS i "
    		: "FROM issues AS i ";
    $sql .= "WHERE 1 = 1 ";
    if ($groupId != 'ALL') {
       $sql .= "AND i.primary_gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }

    if ($breakdown) {
      if (is_object($this)) {
        $sql .= "AND i.opened_by IN ({$this->groupMembers}) "; // safeish
      } else {
        $groupMembers = group_members($groupId);
        $groupMembers = implode(",",array_keys($groupMembers));
        $sql .= "AND i.opened_by IN ({$groupMembers}) "; // safeish
      }
      $sql .= "GROUP BY i.opened_by ";
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $open = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
      $result->free();
      $stmt->free();
      if (is_array($open)) {
        if (is_object($this)) {
          foreach ($open as $tech) {
            $this->reportData['techData'][$tech['opened_by']]['opened'] += $tech['count'];
          }
        } else {
          return $open;
        }
      }
    } else {
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $open = $result->fetchOne(); 
      $result->free();
      $stmt->free();
      if (is_object($this)) {
        $this->reportData['groupData']['opened'] += $open;
      } else {
        return $open;
      }
    }
    unset($open);
  }

  /**
   * Number of issues closed
   *
   * Retrieve the number of closed issues in a group that were closed during
   * the given date range, if no date range is given then the total number
   * of closed issues is returned.  If $breakdown is set to TRUE then the
   * resulting data will be split into the the number of closed issues that
   * are assigned to each member of the group.
   *
   * @param integer $groupId ID of group
   * @param boolean $breakdown Breakdown results into each group member
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return mixed
   */
  function group_number_resolved($groupId,$breakdown = FALSE,$startDate = null,$endDate = null)
  {
    $params = array();
    // retieve all closed statuses
    $closed = implode(",",fetch_status(array(TYPE_CLOSED,TYPE_AUTO_CLOSED)));
    
    $sql  = $breakdown 
      		? "SELECT g.assigned_to,COUNT(i.issueid) AS count "
      		: "SELECT COUNT(i.issueid) ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    			(SELECT
    			  issueid,
    			  status,
    			  CASE WHEN real_closed IS NULL
    			   THEN closed
    			   ELSE real_closed
    			  END AS closed
    			FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";

    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND i.status IN ($closed) "; // safe; see above

    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 


    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.closed BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    if ($breakdown) {
      if (is_object($this)) {
        $sql .= "AND g.assigned_to IN ({$this->groupMembers}) "; // safeish
      } else {
        $groupMembers = group_members($groupId);
        $groupMembers = implode(",",array_keys($groupMembers));
        $sql .= "AND g.assigned_to IN ({$groupMembers}) "; // safeish
      }
      $sql .= "GROUP BY g.assigned_to ";
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
      $closed = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
      $result->free();
      $stmt->free();
      if (is_array($closed)) {
        if (is_object($this)) {
          foreach ($closed as $tech) {
            $this->reportData['techData'][$tech['assigned_to']]['closed'] += $tech['count'];
          }
        } else {
          return $closed;
        }
      }
    } else {
      $stmt = $_ENV['dbi']->prepare($sql);
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $result = $stmt->execute($params);
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $closed = $result->fetchOne(); 
      $result->free();
      $stmt->free();
      if (is_object($this)) {
        $this->reportData['groupData']['closed'] += $closed;
      } else {
        return $closed;
      }
    }
    unset($closed);
  }

  /**
   * Retrieve needed information for groupData array of report
   *
   * @param integer $groupId ID of group
   */
  function group_data($groupId)
  {
    // make sure the class was initialized
    if (!is_object($this)) {
      return FALSE;
    }
    
    //$this->reportData['groupData'] = array();
    $this->group_average_resolution($groupId);
    $this->group_average_response($groupId);
    $this->group_maximum_resolution($groupId);
    $this->group_maximum_response($groupId);
    $this->group_number_opened($groupId);
    $this->group_number_resolved($groupId);
    $this->group_number_events($groupId);
    $this->group_number_hours($groupId);
    $this->reportData['groupData']['formatted'] = 
      sprintf("<tr><td>%d</td><td>%d</td><td>%d</td><td>%.2f</td><td>%.2f</td><td>%.2f</td><td>%.2f</td><td>%.2f</td></tr>",
        $this->reportData['groupData']['opened'],
        $this->reportData['groupData']['closed'],
        $this->reportData['groupData']['events'],
        $this->reportData['groupData']['hours'],
        ($this->reportData['groupData']['avgResponse'] / $this->reportTimeUnit),
        ($this->reportData['groupData']['maxResponse'] / $this->reportTimeUnit),
        ($this->reportData['groupData']['avgResolution'] / $this->reportTimeUnit),
        ($this->reportData['groupData']['maxResolution'] / $this->reportTimeUnit)
      );
  }

  /**
   * Retrieve needed information for breakdown of group member
   * details for a given group
   *
   * @param integer $groupId ID of group
   */
  function group_tech_data($groupId)
  {
    // make sure the class was initialized
    if (!is_object($this)) {
      return FALSE;
    }
    
    $this->reportData['techData'];
    $this->group_average_resolution($groupId,TRUE);
    $this->group_average_response($groupId,TRUE);
    $this->group_maximum_resolution($groupId,TRUE);
    $this->group_maximum_response($groupId,TRUE);
    $this->group_number_opened($groupId,TRUE);
    $this->group_number_assigned($groupId);
    $this->group_number_resolved($groupId,TRUE);
    $this->group_number_events($groupId,TRUE);
    $this->group_number_hours($groupId,TRUE);
    $tmpData = $this->reportData['techData'];
    $techData = array();

    /* DAVE --> THIS IS WHERE THE PROBLEM IS!! */
	foreach ($tmpData as $userId => $data) {
	   $name = ident($userId);
	   $techData[$name] = $data;
	   $techData[$name]['formatted'] = 
	   sprintf("<tr><td>%s</td><td>%d</td><td>%d</td><td>%d</td><td>%d</td><td>%.2f</td><td>%.2f</td><td>%.2f</td><td>%.2f</td><td>%.2f</td></tr>",
	      $name,
	      $data['opened'],
	      $data['assigned'],
	      $data['closed'],
	      $data['events'],
	      $data['hours'],
	      ($data['avgResponse'] / $this->reportTimeUnit),
	      ($data['maxResponse'] / $this->reportTimeUnit),
	      ($data['avgResolution'] / $this->reportTimeUnit),
	      ($data['maxResolution'] / $this->reportTimeUnit)
	    );
	}
	
	ksort($techData);
    $this->reportData['techData'] = $techData;
    unset($techData);
    unset($tmpData);
    
    $_SESSION['DAJ_tmpData'] = gettype($tmpData); 
  }

  /**
   * Retrieve category data for report
   *
   * Retrieves the category data such as the number of issues for each
   * category that were opened during given date range, the average
   * resolution time for each category, and the maximum resolution time
   * for each category.  If no date range is given then all issues in
   * the group will be included.
   *
   * @param integer $groupId ID of group
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return array
   */
  function group_category_data($groupId,$startDate = null,$endDate = null)
  {
    // Initialize the categoryData array
    $categoryData = array();

    // Average Resolution Time
    $params = array();
    $sql  = "SELECT i.category,AVG(i.closed - i.opened) AS average ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,category,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened,
    			CASE WHEN real_closed IS NULL
    			 THEN closed
    			 ELSE real_closed
    			END AS closed 
    		  FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";
    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND i.closed != 0 ";

    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }

    $sql .= "GROUP BY i.category";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $average = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($average)) {
      foreach ($average as $category) {
        $categoryData[$category['category']]['avgResolution'] = $category['average'];
      }
    }
    unset($average);

    // Maximum Resolution Time
    $params = array();
    $sql  = "SELECT i.category,MAX(i.closed - i.opened) AS maximum ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,category,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened,
    			CASE WHEN real_closed IS NULL
    			 THEN closed
    			 ELSE real_closed
    			END AS closed 
    		  FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";
    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND i.closed != 0 ";

    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 


    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    $sql .= "GROUP BY i.category";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $maximum = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($maximum)) {
      foreach ($maximum as $category) {
        $categoryData[$category['category']]['maxResolution'] = $category['maximum'];
      }
    }
    unset($maximum);

    // Number of Issues Opened
    $params = array();
    $sql  = "SELECT i.category,COUNT(i.issueid) AS count ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,primary_gid,category,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened,
    			CASE WHEN real_closed IS NULL
    			 THEN closed
    			 ELSE real_closed
    			END AS closed 
    		  FROM issues) AS i "
    		: "FROM issues AS i ";
    $sql .= "WHERE 1 = 1 ";
    if ($groupId != 'ALL') {
       $sql .= "AND i.primary_gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    $sql .= "GROUP BY i.category";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $open = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($open)) {
      foreach ($open as $category) {
        $categoryData[$category['category']]['opened'] = $category['count'];
      }
    }
    unset($open);

    // if we didnt fully initialize the class just return the data now
    if (!is_object($this)) {
      return $categoryData;
    }

    $tmpData = $categoryData;
    $categoryData = array();
    foreach ($tmpData as $categoryId => $data) {
      if (empty($categoryId)) {
        continue;
      }
      $name = category($categoryId);
      $categoryData[$name] = $data;
      $categoryData[$name]['formatted'] = 
        sprintf("<tr><td>%s</td><td>%d</td><td>%.2f</td><td>%.2f</td></tr>",
          $name, 
          $data['opened'],
          ($data['avgResolution'] / $this->reportTimeUnit),
          ($data['maxResolution'] / $this->reportTimeUnit)
        );
    }

    ksort($categoryData);
    $this->reportData['categoryData'] = $categoryData;
    unset($tmpData);
    unset($categoryData);
  }
  
  /**
   * Retrieve product data for report
   *
   * Retrieves the product data such as the number of issues for each
   * product there were opened during given date range, the average
   * resolution time for each product, and the maximum resolution time
   * for each product.  If no date range is given then all issues in
   * the group will be included.
   *
   * @param integer $groupId ID of group
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return array
   */
  function group_product_data($groupId,$startDate = null,$endDate = null)
  {
    // Initialize the productData array
    $productData = array();

    // Average Resolution Time
    $params = array();
    $sql  = "SELECT i.product,AVG(i.closed - i.opened) AS average ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,product,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened,
    			CASE WHEN real_closed IS NULL
    			 THEN closed
    			 ELSE real_closed
    			END AS closed 
    		  FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";
    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND i.closed != 0 ";

    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    $sql .= "GROUP BY i.product";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $average = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($average)) {
      foreach ($average as $product) {
        $productData[$product['product']]['avgResolution'] 
          = $product['average'];
      }
    }
    unset($average);

    // Maximum Resolution Time
    $params = array();
    $sql  = "SELECT i.product,MAX(i.closed - i.opened) AS maximum ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,product,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened,
    			CASE WHEN real_closed IS NULL
    			 THEN closed
    			 ELSE real_closed
    			END AS closed 
    		  FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";
    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND i.closed != 0 ";

    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    $sql .= "GROUP BY i.product";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $maximum = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($maximum)) {
      foreach ($maximum as $product) {
        $productData[$product['product']]['maxResolution'] 
          = $product['maximum'];
      }
    }
    unset($maximum);

    // Number of Issues Opened
    $params = array();
    $sql  = "SELECT i.product,COUNT(i.issueid) AS count ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,primary_gid,product,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened,
    			CASE WHEN real_closed IS NULL
    			 THEN closed
    			 ELSE real_closed
    			END AS closed 
    		  FROM issues) AS i "
    		: "FROM issues AS i ";
    $sql .= "WHERE 1 = 1 ";
    if ($groupId != 'ALL') {
       $sql .= "AND i.primary_gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    $sql .= "GROUP BY i.product";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $open = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($open)) {
      foreach ($open as $product) {
        $productData[$product['product']]['opened'] = $product['count'];
      }
    }
    unset($open);

    // if class was not actually initialized then return data now
    if (!is_object($this)) {
      return $productData;
    }

    $tmpData = $productData;
    $productData = array();
    foreach ($tmpData as $productId => $data) {
      if (empty($productId)) {
        continue;
      }
      $name = product($productId);
      $productData[$name] = $data;
      $productData[$name]['formatted'] = 
        sprintf("<tr><td>%s</td><td>%d</td><td>%.2f</td><td>%.2f</td></tr>",
          $name,
          $data['opened'],
          ($data['avgResolution'] / $this->reportTimeUnit),
          ($data['maxResolution'] / $this->reportTimeUnit)
        );
    }

    ksort($productData);
    $this->reportData['productData'] = $productData;
    unset($tmpData);
    unset($productData);
  }

  /**
   * Retrieve severity data for report
   *
   * Retrieves the severity data such as the number of issues opened
   * for each severity, the average/maximum resolution time for
   * each severity, and the average/maximum first response time for
   * each severity.
   *
   * @param integer $groupId ID of group
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return array
   */
  function group_severity_data($groupId,$startDate = null,$endDate = null)
  {
    // Initialize the severityData array
    $severityData = array();

    // Average Resolution Time
    $params = array();
    $sql  = "SELECT i.severity,AVG(i.closed - i.opened) AS average ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,severity,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
			END AS opened,
			CASE WHEN real_closed IS NULL
			 THEN closed
			 ELSE real_closed
			END AS closed
    		  FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";
    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND i.closed != 0 ";
    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }

    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ?  AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    $sql .= "GROUP BY i.severity";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $average = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($average)) {
      foreach ($average as $severity) {
        $severityData[$severity['severity']]['avgResolution'] 
          = $severity['average'];
      }
    }
    unset($average);

    // Average First Response Time
    $params = array();
    $sql  = "SELECT i.severity,AVG(g.first_response - i.opened) AS average ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,severity,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened 
    		  FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";
    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND g.first_response != 0 ";

    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    $sql .= "GROUP BY i.severity";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $average = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($average)) {
      foreach ($average as $severity) {
        $severityData[$severity['severity']]['avgResponse'] 
          = $severity['average'];
      }
    }
    unset($average);

    // Maximum Resolution Time
    $params = array();
    $sql  = "SELECT i.severity,MAX(i.closed - i.opened) AS maximum ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,severity,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened, 
			CASE WHEN real_closed IS NULL
			 THEN closed
			 ELSE real_closed
			END AS closed
			FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";
    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND i.closed != 0 ";

    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    $sql .= "GROUP BY i.severity";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $maximum = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($maximum)) {
      foreach ($maximum as $severity) {
        $severityData[$severity['severity']]['maxResolution'] 
          = $severity['maximum'];
      }
    }
    unset($maximum);

    // Maximum First Response Time
    $params = array();
    $sql  = "SELECT i.severity,MAX(g.first_response - i.opened) AS maximum ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,severity,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened 
    		  FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";
    $sql .= "WHERE g.issueid=i.issueid ";
    $sql .= "AND g.first_response != 0 ";

    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    $sql .= "GROUP BY i.severity";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $maximum = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($maximum)) {
      foreach ($maximum as $severity) {
        $severityData[$severity['severity']]['maxResponse'] 
          = $severity['maximum'];
      }
    }
    unset($maximum);

    // Number of Issues Opened
    $params = array();
    $sql  = "SELECT i.severity,COUNT(i.issueid) AS count ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,primary_gid,severity,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened 
    		  FROM issues) AS i "
    		: "FROM issues AS i ";
    $sql .= "WHERE 1 = 1 AND ";
    if ($groupId != 'ALL') {
       $sql .= "AND i.primary_gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    $sql .= "GROUP BY i.severity";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $open = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($open)) {
      foreach ($open as $severity) {
        $severityData[$severity['severity']]['opened'] = $severity['count'];
      }
    }
    unset($open);

    // if the class was not initialized then just return data now
    if (!is_object($this)) {
      return $severityData;
    }
  
    $tmpData = $severityData;
    $severityData = array();
    foreach ($tmpData as $severity => $data) {
      if (empty($severity)) {
        continue;
      }
      $name = severity_text($severity);
      $severityData[$name] = $data;
      $severityData[$name]['formatted'] = 
        sprintf("<tr><td>%s</td><td>%d</td><td>%.2f</td><td>%.2f</td><td>%.2f</td><td>%.2f</td></tr>",
          $name,
          $data['opened'],
          ($data['avgResponse'] / $this->reportTimeUnit),
          ($data['maxResponse'] / $this->reportTimeUnit),
          ($data['avgResolution'] / $this->reportTimeUnit),
          ($data['maxResolution'] / $this->reportTimeUnit)
        );
    }

    ksort($severityData);
    $this->reportData['severityData'] = $severityData;
    unset($severityData);
    unset($tmpData);
  }
  
  /**
   * Retrieve status data for report
   *
   * Retrieves the number of issues assigned to each status.  Only issues
   * within the given date range are counted.  If no date range is given
   * then all issues within the group will be counted.
   *
   * @param integer $groupId ID of group
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return array
   */
  function group_status_data($groupId,$startDate = null,$endDate = null)
  {
    // Initialize the statusData array
    $statusData = array();

    // Number of Issues Opened
    $params = array();
    $sql  = "SELECT i.status,COUNT(i.issueid) AS count ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,primary_gid,status,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened 
    		  FROM issues) AS i "
    		: "FROM issues AS i ";
    $sql .= "WHERE 1 = 1 AND";

    if ($groupId != 'ALL') {
       $sql .= "i.primary_gid = ? ";
       $params[] = $groupId;
    } 

    if (isset($this->startDate)) {
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $this->startDate;
      $params[] = $this->endDate;
    }
    $sql .= "GROUP BY i.status";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $open = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (is_array($open)) {
      foreach ($open as $status) {
        $statusData[$status['status']]['opened'] = $status['count'];
      }
    }
    unset($open);

    $tmpData = $statusData;
    $statusData = array();
    foreach ($tmpData as $status => $data) {
      if (empty($status)) {
        continue;
      }
      $name = status($status);
      $statusData[$name] = $data;
      $statusData[$name]['formatted'] 
        = sprintf("<tr><td>%s</td><td>%d</td></tr>",$name, $data['opened']);
    }

    ksort($statusData);
    $this->reportData['statusData'] = $statusData;
    unset($statusData);
    unset($tmpData);
  }

  /**
   * Retrieve needed information for escalationData array of report
   *
   * @param integer $groupId ID of group
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   * @return array
   */
  function group_escalation_data($groupId,$startDate = null,$endDate = null)
  {
    // Number of Issues Escalated from Group
    $params = array();
    $sql  = "SELECT DISTINCT i.issueid ";
    $sql .= in_array("use_modified", $this->reportOptions)
    		? "FROM 
    		  (SELECT issueid,primary_gid,
    			CASE WHEN real_opened IS NULL
    			 THEN opened
    			 ELSE real_opened
    			END AS opened
    		  FROM issues) AS i, issue_groups AS g "
    		: "FROM issues AS i, issue_groups AS g ";
    $sql .= "WHERE i.issueid = g.issueid ";
    $sql .= "AND g.gid != i.primary_gid ";

    if ($groupId != 'ALL') {
       $sql .= "AND i.primary_gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    
    // for debugging
    //echo "<br />$sql<pre>"; var_dump($params); echo "</pre>";
    
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $issues = $result->fetchCol(); 
    $result->free();
    $stmt->free();
    if (is_array($issues)) {
      if (is_object($this)) {
        $this->reportData['escalationData']['from'] += count($issues);
      } else {
        $escalationData['from'] = count($issues);
      }
    }
    unset($issues,$params);

    // Number of Issues Escalated to Group
    $param = array();
    $sql  = "SELECT COUNT(i.issueid) ";
    if (in_array("use_modified", $this->reportOptions)) {
      $sql .= "FROM 
    		    (SELECT issueid,primary_gid,
    			  CASE WHEN real_opened IS NULL
    			  THEN opened
    			  ELSE real_opened
    			  END AS opened
    		    FROM issues) AS i, issue_groups AS g ";
    } else {
      $sql .= "FROM issues AS i, issue_groups AS g ";
    }

    $sql .= "WHERE g.gid != i.primary_gid ";
    $sql .= "AND i.issueid = g.issueid ";

    if ($groupId != 'ALL') {
       $sql .= "AND g.gid = ? ";
       $params[] = $groupId;
    } 

    if (is_object($this)) {
      if (isset($this->startDate)) {
        $sql .= "AND i.opened BETWEEN ? AND ? ";
        $params[] = $this->startDate;
        $params[] = $this->endDate;
      }
    } else {
      if (ereg("[0-9]+",$startDate)) {
        if (!ereg("[0-9]+",$endDate)) {
          $endDate = time();
        }
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $this->startDate;
      $params[] = $this->endDate;
    }
    
    // for debugging
    //echo "<br />$sql<pre>"; var_dump($params); echo "</pre>";
    
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $count = $result->fetchOne(); 
    $result->free();
    $stmt->free();
    if (is_object($this)) {
      $this->reportData['escalationData']['to'] += $count;
    } else {
      $escalationData['to'] = $count;
      return $escalationData;
    }
  }
  
  /**
   * Added by Tim Boring (tim -at- timboring dot com) 1/2006 
   * 
   * Retrieve list of issues by status. Instead of returning the results, simply
   * add them to the $this->reportData['statusIssueData'] array.
   * 
   * NOTE: The idea for the closed issues option and the method for converting
   * the timestamp from a Unix timestamp to a human-readable date/time are based
   * on code from David Joyce's cvs dump patch for ET.
   *
   * @param integer $groupId ID of group
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   */
  function group_status_issue($groupId, $startDate = null, $endDate = null)
  {
    // Get the sid for each status
    $sids = array();
    
    /**
     * This if statement was put in to handle the case where a
     * user does not choose a status and leaves the selection on 
     * 'Select one or more' from the group_issues_form.tpl template.  That
     * template sets the value of that selection to '*', which will here be used
     * to select ALL statuses.
     */
    if ($this->reportIssueSelections[0] == "*" or !isset($this->reportIssueSelections[0])) {
        $stmt = $_ENV['dbi']->prepare("SELECT sid from statuses");
        $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
        $result = $stmt->execute();
        $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
        $sids = $result->fetchCol(); 
        $result->free();
        $stmt->free();
    } else {
        foreach ($this->reportIssueSelections as $key => $val) {
            $sql = "SELECT sid ";
            $sql .= "FROM statuses ";
            $sql .= "WHERE status = ? ";
            $stmt = $_ENV['dbi']->prepare($sql);
            $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
            $result = $stmt->execute(array($val));
            $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
            $sid = $result->fetchOne(); 
            $result->free();
            $stmt->free();
            array_push($sids, $sid);   
        }
    }

    // Get the ticket info for the user-selected statuses
    $params = array();
    $sql = "SELECT ";
    foreach ($this->reportIssueFields as $key => $field) {
        if ($key + 1 < count($this->reportIssueFields)) {
            if ($field == 'i.severity') {
                $sql .= "CASE
                            WHEN i.severity = 4 then 'LOW'
                            WHEN i.severity = 3 then 'NORMAL'
                            WHEN i.severity = 2 then 'HIGH'
                            WHEN i.severity = 1 then 'URGENT'
                            ELSE 'UNKNOWN'
                         END, ";
            } else {
                $sql .= "$field,"; // DBI TODO: safe?
            }
        } else {
            if ($field == 'i.severity') {
                $sql .= "CASE
                            WHEN i.severity = 4 then 'LOW'
                            WHEN i.severity = 3 then 'NORMAL'
                            WHEN i.severity = 2 then 'HIGH'
                            WHEN i.severity = 1 then 'URGENT'
                            ELSE 'UNKNOWN'
                         END ";
            } else {
                $sql .= "$field "; // DBI TODO: safe?
            }
        }
    }        
        
    //$sql = "SELECT i.issueid, i.summary, c.category, p.product, s.status, i.severity, i.opened ";
    $sql .= ($this->closedIssues) ? ",i.closed " : ""; // DBI TODO: safe?
    $sql .= "FROM issues AS i, categories AS c, products AS p, statuses AS s ";
    $sql .= "WHERE i.category = c.cid ";
    $sql .= "AND i.product = p.pid ";
    $sql .= "AND i.status = s.sid ";

    if ($groupId != 'ALL') {
       $sql .= "AND i.primary_gid = ? ";
       $params[] = $groupId;
    } 

    $sql .= ($this->closedIssues) ? "" : "AND i.closed IS NULL ";
    $sql .= "AND (";
    
    foreach ($sids as $key => $sid) {
        $sql .= " i.status = ? ";
        $params[] = $sid;
        
        if ($key < count($sids) -1 ) {
            $sql .= "OR ";
        }
    }
    
    $sql .= ") ";
    
    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    
    // Set the sort option
    switch($this->reportSortOptions[0]) {
        case 'descending':
            $sql .= "ORDER BY s.status, i.opened DESC ";
            break;
        default:
            $sql .= "ORDER BY s.status, i.opened ";
            break;
    }
        
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $records = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    $issues = array();
    
    // Convert the opened and closed timestamps to human-readable formats
    if ($records) {
        foreach ($records as $key => $record) {
            if (isset($record['opened'])) {
                $record['opened'] = $_ENV['i18n']->format_datetime($record['opened']);
            }
            
            if (isset($record['closed'])) {
                $record['closed'] = $_ENV['i18n']->format_datetime($record['closed']);
            }
            
            $issues[] = $record;        
        }
    }
    
    $this->reportData['issueData'] = $issues;  
  }
  
  /**
   * Added by Tim Boring (tim -at- timboring dot com) 1/2006 
   * 
   * Retrieve list of issues by category. Instead of returning the results,
   * simply add them to the $this->reportData['issueData'] array.
   * 
   * NOTE: The idea for the closed issues option and the method for converting
   * the timestamp from a Unix timestamp to a human-readable date/time are based
   * on code from David Joyce's cvs dump patch for ET.
   *
   * @param integer $groupId ID of group
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   */
  function group_category_issue($groupId, $startDate = null, $endDate = null)
  {
    // Get the cid for each category
    $cids = array();
    
    /**
     * This if statement was put in to handle the case where a
     * user does not choose a category and leaves the selection 
     * on 'Select one or more' from the group_issues_form.tpl
     * template.  That template sets the value of that selection
     * to '*', which will here be used to select ALL categories.
     */
    if ($this->reportIssueSelections[0] == "*" or !isset($this->reportIssueSelections[0])) {
        $stmt = $_ENV['dbi']->prepare("SELECT cid from categories");
        $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
        $result = $stmt->execute();
        $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
        $cids = $result->fetchCol(); 
        $result->free();
        $stmt->free();
    } else {
        foreach ($this->reportIssueSelections as $key => $val) {
            $sql = "SELECT cid ";
            $sql .= "FROM categories ";
            $sql .= "WHERE category = ? ";
            $stmt = $_ENV['dbi']->prepare($sql);
            $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
            $result = $stmt->execute(array($val));
            $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
            $cid = $result->fetchOne(); 
            $result->free();
            $stmt->free();
            array_push($cids, $cid);   
        }
    }
    
    // Get the ticket info for the user-selected categories
    $params = array();
    $sql = "SELECT ";
    foreach ($this->reportIssueFields as $key => $field) {
        if ($key + 1 < count($this->reportIssueFields)) {
            if ($field == 'i.severity') {
                $sql .= "CASE
                            WHEN i.severity = 4 then 'LOW'
                            WHEN i.severity = 3 then 'NORMAL'
                            WHEN i.severity = 2 then 'HIGH'
                            WHEN i.severity = 1 then 'URGENT'
                            ELSE 'UNKNOWN'
                         END, ";
            } else {
                $sql .= "$field,"; // DBI TODO: safe?
            }
        } else {
            if ($field == 'i.severity') {
                $sql .= "CASE
                            WHEN i.severity = 4 then 'LOW'
                            WHEN i.severity = 3 then 'NORMAL'
                            WHEN i.severity = 2 then 'HIGH'
                            WHEN i.severity = 1 then 'URGENT'
                            ELSE 'UNKNOWN'
                         END ";
            } else {
                $sql .= "$field "; // DBI TODO: safe?
            }
        }
    }        
        
    //$sql = "SELECT i.issueid, i.summary, c.category, p.product, s.status, i.severity, i.opened ";
    $sql .= ($this->closedIssues) ? ",i.closed " : "";
    $sql .= "FROM issues AS i, categories AS c, products AS p, statuses AS s ";
    $sql .= "WHERE i.category = c.cid ";
    $sql .= "AND i.product = p.pid ";
    $sql .= "AND i.status = s.sid ";

    if ($groupId != 'ALL') {
       $sql .= "AND i.primary_gid = ? ";
       $params[] = $groupId;
    } 

    $sql .= ($this->closedIssues) ? "" : "AND i.closed IS NULL ";
    $sql .= "AND (";
    
    foreach ($cids as $key => $cid) {
        $sql .= " i.category = ? ";
        $params[] = $cid;
        
        if ($key < count($cids) -1 ) {
            $sql .= "OR ";
        }
    }
    
    $sql .= ") ";
    
    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    
    // Set the sort option
    switch($this->reportSortOptions[0]) {
        case 'descending':
            $sql .= "ORDER BY c.category, i.opened DESC ";
            break;
        default:
            $sql .= "ORDER BY c.category, i.opened ";
            break;
    }
        
    //print "SQL: $sql<br/>";
    
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $records = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    $issues = array();
    
    // Convert the opened and closed timestamps to human-readable formats
    if ($records) {
        foreach ($records as $key => $record) {
            if (isset($record['opened'])) {
                $record['opened'] = $_ENV['i18n']->format_datetime($record['opened']);
            }
            
            if (isset($record['closed'])) {
                $record['closed'] = $_ENV['i18n']->format_datetime($record['closed']);
            }
            
            $issues[] = $record;        
        }
    }
    
    $this->reportData['issueData'] = $issues;  
  }
  
  /**
   * Added by Tim Boring (tim -at- timboring dot com) 1/2006 
   * 
   * Retrieve list of issues by product. Instead of returning the results,
   * simply add them to the $this->reportData['issueData'] array.
   * 
   * NOTE: The idea for the closed issues option and the method for converting
   * the timestamp from a Unix timestamp to a human-readable date/time are based
   * on code from David Joyce's cvs dump patch for ET.
   *
   * @param integer $groupId ID of group
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   */
  function group_product_issue($groupId, $startDate = null, $endDate = null)
  {
    // Get the pid for each product
    $pids = array();
    
    /**
     * This if statement was put in to handle the case where a
     * user does not choose a product and leaves the selection on 'Select one or
     * more' from the group_issues_form.tpl template.  That template sets the
     * value of that selection to '*', which will here be used to select ALL
     * categories.
     */
    if ($this->reportIssueSelections[0] == "*" or !isset($this->reportIssueSelections[0])) {
        $stmt = $_ENV['dbi']->prepare("SELECT pid from products");
        $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
        $result = $stmt->execute();
        $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
        $pids = $result->fetchCol(); 
        $result->free();
        $stmt->free();
    } else {
        foreach ($this->reportIssueSelections as $key => $val) {
            $sql = "SELECT pid ";
            $sql .= "FROM products ";
            $sql .= "WHERE product = ? ";
            $stmt = $_ENV['dbi']->prepare($sql);
            $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
            $result = $stmt->execute(array($val));
            $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
            $pid = $result->fetchOne(); 
            $result->free();
            $stmt->free();
            array_push($pids, $pid);   
        }
    }
    
    // Get the ticket info for the user-selected products
    $sql = "SELECT ";
    foreach ($this->reportIssueFields as $key => $field) {
        if ($key + 1 < count($this->reportIssueFields)) {
            if ($field == 'i.severity') {
                $sql .= "CASE
                            WHEN i.severity = 4 then 'LOW'
                            WHEN i.severity = 3 then 'NORMAL'
                            WHEN i.severity = 2 then 'HIGH'
                            WHEN i.severity = 1 then 'URGENT'
                            ELSE 'UNKNOWN'
                         END, ";
            } else {             
                $sql .= "$field,";
            }
        } else {
            if ($field == 'i.severity') {
                $sql .= "CASE
                            WHEN i.severity = 4 then 'LOW'
                            WHEN i.severity = 3 then 'NORMAL'
                            WHEN i.severity = 2 then 'HIGH'
                            WHEN i.severity = 1 then 'URGENT'
                            ELSE 'UNKNOWN'
                         END ";
            } else {
                $sql .= "$field ";
            }
        }
    }  
        
    $params = array();
    $sql .= ($this->closedIssues) ? ",i.closed " : "";
    $sql .= "FROM issues AS i, categories AS c, products AS p, statuses AS s ";
    $sql .= "WHERE i.category = c.cid ";
    $sql .= "AND i.product = p.pid ";
    $sql .= "AND i.status = s.sid ";

    if ($groupId != 'ALL') {
       $sql .= "AND i.primary_gid = ? ";
       $params[] = $groupId;
    } 

    $sql .= ($this->closedIssues) ? "" : "AND i.closed IS NULL ";
    $sql .= "AND (";
    
    foreach ($pids as $key => $pid) {
        $sql .= " i.product = ? ";
        $params[] = $pid;
        
        if ($key < count($pids) -1 ) {
            $sql .= "OR ";
        }
    }
    
    $sql .= ") ";
    
    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    }
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    
    // Set the sort option
    switch($this->reportSortOptions[0]) {
        case 'descending':
            $sql .= "ORDER BY p.product, i.opened DESC ";
            break;
        default:
            $sql .= "ORDER BY p.product, i.opened ";
            break;
    }
        
    //print "SQL: $sql<br/>";
    
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $records = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    $issues = array();
    
    // Convert the opened and closed timestamps to human-readable formats
    if ($records) {
        foreach ($records as $key => $record) {
            if (isset($record['opened'])) {
                $record['opened'] = $_ENV['i18n']->format_datetime($record['opened']);
            }
            
            if (isset($record['closed'])) {
                $record['closed'] = $_ENV['i18n']->format_datetime($record['closed']);
            }
            
            $issues[] = $record;        
        }
    }
    
    $this->reportData['issueData'] = $issues;  
  }
  
  /**
   * Added by Tim Boring (tim -at- timboring dot com) 1/2006 
   * 
   * Retrieve list of issues by severity. Instead of returning the results,
   * simply add them to the $this->reportData['issueData'] array.
   * 
   * NOTE: The idea for the closed issues option and the method for converting
   * the timestamp from a Unix timestamp to a human-readable date/time are based
   * on code from David Joyce's cvs dump patch for ET.
   *
   * @param integer $groupId ID of group
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   */
  function group_severity_issue($groupId, $startDate = null, $endDate = null)
  {
    // Get the ticket info for the user-selected products
    $params = array();
    $sql = "SELECT ";
    foreach ($this->reportIssueFields as $key => $field) {
        if ($key + 1 < count($this->reportIssueFields)) {
            if ($field == 'i.severity') {
                $sql .= "CASE
                            WHEN i.severity = 4 then 'LOW'
                            WHEN i.severity = 3 then 'NORMAL'
                            WHEN i.severity = 2 then 'HIGH'
                            WHEN i.severity = 1 then 'URGENT'
                            ELSE 'UNKNOWN'
                         END, ";
            } else {             
                $sql .= "$field,"; // DBI TODO: safe?
            }
        } else {
            if ($field == 'i.severity') {
                $sql .= "CASE
                            WHEN i.severity = 4 then 'LOW'
                            WHEN i.severity = 3 then 'NORMAL'
                            WHEN i.severity = 2 then 'HIGH'
                            WHEN i.severity = 1 then 'URGENT'
                            ELSE 'UNKNOWN'
                         END ";
            } else {
                $sql .= "$field "; // DBI TODO: safe?
            }
        }
    }        
        
    $sql .= ($this->closedIssues) ? ",i.closed " : "";
    $sql .= "FROM issues AS i, categories AS c, products AS p, statuses AS s ";
    $sql .= "WHERE i.category = c.cid ";
    $sql .= "AND i.product = p.pid ";
    $sql .= "AND i.status = s.sid ";

    if ($groupId != 'ALL') {
       $sql .= "AND i.primary_gid = ? ";
       $params[] = $groupId;
    } 

    $sql .= ($this->closedIssues) ? "" : "AND i.closed IS NULL ";
    $sql .= "AND (";
    
    /**
     * The following if statement was added to handle the case where a user
     * either selects 'Select one or more' from the list of severities or
     * does select anything at all.  In this case, it gets treated as
     * '*', meaning were going to get ALL severities.
     */
    if ($this->reportIssueSelections[0] == "*" or !isset($this->reportIssueSelections[0])) {
        $sql .= " i.severity = 4 or i.severity = 3 or i.severity = 2 or i.severity = 1 ";
    } else { 
        foreach ($this->reportIssueSelections as $key => $sev) {
            $sql .= " i.severity = ? ";
            $params[] = $sev;
            
            if ($key < count($this->reportIssueSelections) -1 ) {
                $sql .= "OR ";
            }
        }
    }
    
    $sql .= ") ";
    
    if (is_object($this)) {
      $startDate = $this->startDate;
      $endDate = $this->endDate;
    } 
    if (ereg("[0-9]+",$startDate)) {
      if (!ereg("[0-9]+",$endDate)) {
        $endDate = time();
      }
      $sql .= "AND i.opened BETWEEN ? AND ? ";
      $params[] = $startDate;
      $params[] = $endDate;
    }
    
    // Set the sort option
    switch($this->reportSortOptions[0]) {
        case 'descending':
            $sql .= "ORDER BY i.severity, i.opened DESC ";
            break;
        default:
            $sql .= "ORDER BY i.severity, i.opened ";
            break;
    }
        
    //print "SQL: $sql<br/>";
    
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $records = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    $issues = array();
    
    // Convert the opened and closed timestamps to human-readable formats
    if ($records) {
        foreach ($records as $key => $record) {
            if (isset($record['opened'])) {
                $record['opened'] = $_ENV['i18n']->format_datetime($record['opened']);
            }
            
            if (isset($record['closed'])) {
                $record['closed'] = $_ENV['i18n']->format_datetime($record['closed']);
            }
            
            $issues[] = $record;        
        }
    } 
    
    $this->reportData['issueData'] = $issues;  
  }
  
  /**
   * Added by Tim Boring (tim -at- timboring dot com) 2/16/2006 
   * 
   * Retrieve list of issues by technician. Instead of returning the results,
   * simply add them to the $this->reportData['issueData'] array.
   * 
   * The method for getting the issues by technician is this: a) for each
   * technician selected by the user, get the id number for the tech
   * ($this->reportTechnicians); b) lookup the tech's name (users. firstname +
   * users.lastname); c) get all the issue IDs for the tech; d) retrieve the
   * user- selected fields for issues assigned to the tech(s); e) push the
   * tech's issues onto the $t_issues array, which will be used to populate
   * $this->reportData['issueData'].
   * 
   * NOTE: The idea for the closed issues option and the method for converting
   * the timestamp from a Unix timestamp to a human-readable date/time are based
   * on code from David Joyce's cvs dump patch for ET.
   *
   * @param integer $groupId ID of group
   * @param integer $startDate Start date of report
   * @param integer $endDate End date of report
   */
  function group_technician_issue($groupId, $startDate = null, $endDate = null)
  {

    // Get Report Technicians
    $users = implode (',',$this->reportTechnicians);
    $closestats = fetch_status(array(TYPE_CLOSED,TYPE_AUTO_CLOSED));

    if (empty($users)) {
        $_ENV['error']->push_error("You must select at least one Technician.");
        redirect('?module=reports&action=custom/tickets_by_technician');
        exit;
    }

    $params = array();
    $sql  = "SELECT ";
    foreach ($this->reportIssueFields as $field) {
       if ($field =='gid')
	 $sql .= "ig.$field, ";
       elseif ($field =='duration'){
	$sql .= "(SELECT sum($field) from events where issueid = i.issueid ";
 	$sql .= " and userid = ig.assigned_to) as duration, ";
       } else
	 $sql .= "i.$field, ";
    }
    
    if ($this->closedIssues == 'true')
	$sql .= "i.closed, ";
    $sql .= "i.summary, ig.assigned_to "; // by default the issue summary is always display it
    $sql .= "FROM issues i, issue_groups ig "; 
    $sql .= "WHERE ig.issueid = i.issueid ";
    if ($groupId != 'ALL') {
       $sql .= "AND  ig.gid = ? ";
       $params[] = $groupId;
    } 
    $sql .= "AND ig.assigned_to in ($users) ";
    
    #Don't show close issues
    if (!$this->closedIssues) {
       	$sql .= "and i.status not in ($closestats[0] ";
       	for ($i=1;$i< count($closestats); $i++) {
	 $sql .=", $closestats[$i]";
	}
	$sql .= ") ";	    
    }

    $sql .= ($this->reportSortOptions[0] == "descending") ? "ORDER BY ig.gid, ig.assigned_to DESC " : "ORDER BY ig.gid, ig.assigned_to ";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute($params);
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
    $issues = $result->fetchAll(MDB2_FETCHMODE_ASSOC);; 
    $result->free();
    $stmt->free();

    $this->reportData['issueData'] = $issues;  
    
  }

}
?>
