<?php
////
// User Objects

/* {{{ Function get_user */
/**
 * Look up a user by userid
 *
 * @param integer $userid the userid
 * @return mixed a user object or null
 */
function get_user($userid)
{
  global $_user_cache;

  if (!isset($_user_cache[$userid])) {
    $sql  = "SELECT * ";
    $sql .= "FROM users ";
    $sql .= "WHERE userid = ?";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($userid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $user_row = $result->fetchRow(MDB2_FETCHMODE_ASSOC); 
    $result->free();
    $stmt->free();
    if (!is_null($user_row)) {
      $user = &new User($user_row['userid'], $user_row);
      $_user_cache[$userid] = &$user;
    }
  }

  if (isset($_user_cache[$userid])) {
    return $_user_cache[$userid];
  } else {
    return null;
  }
}
/* }}} */

/* {{{ Function get_user_by_email */
/**
 * Look up a user by email address, matching any of
 * the user's email addresses (not just the primary)
 *
 * @param string $email the email address
 * @return mixed a user object or null
 */
function get_user_by_email($email)
{
  $sql  = "SELECT userid ";
  $sql .= "FROM alternate_emails ";
  $sql .= "WHERE LOWER(email) = LOWER(?)";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($email));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $userid = $result->fetchOne(); 
  $result->free();
  $stmt->free();
  if (!is_null($userid)) {
    return get_user($userid);
  } else {
    return null;
  }
}
/* }}} */

/* {{{ Function get_user_by_username */
/**
 * Look up a user by username
 *
 * @param string $username the username address
 * @return mixed a user object or null
 */
function get_user_by_username($username)
{
  global $_user_cache;

  $sql  = "SELECT userid ";
  $sql .= "FROM users ";
  $sql .= "WHERE LOWER(username) = LOWER(?)";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($username));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $userid = $result->fetchOne(); 
  $result->free();
  $stmt->free();
  if (!is_null($userid)) {
    return get_user($userid);
  } else {
    return null;
  }
}
/* }}} */

/* {{{ Function create_user */
/**
 * Creates a new user account, returning a user object if successful, or
 * null if unsuccessful.
 *
 * @param array $user_data Array containing user data, with keys username, password,
 *     primary_email, first_name, last_name, and admin.
 * @return mixed -- a User object or null
 */
function create_user($user_data)
{
  extract($user_data);

  // email is required
  if (empty($primary_email)) {
    $_ENV['error']->push_error(sprintf(_("Email address is required."), $username));
    return null;
  }

  if (!empty($username)) {
    $sql  = "SELECT userid ";
    $sql .= "FROM users ";
    $sql .= "WHERE LOWER(username) = LOWER(?)";
    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($username));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    $uid = $result->fetchOne(); 
    $result->free();
    $stmt->free();
    if (!is_null($uid)) {
      $_ENV['error']->push_error(sprintf(_("The username '%s' is already in use by another user."), $username));
      return null;
    }
  }

  // if this email already exists, and is email-only, we can just assign a
  // username and whatnot to it
  if ($existing_user = &get_user_by_email($primary_email)) {
    if (!$existing_user->is_email_only()) {
      $_ENV['error']->push_error(sprintf(_("The email address '%s' is already in use by another user."), $primary_email));
      return null;
    }

    if (isset($username)) { $existing_user->username($username); }
    if (isset($password)) { $existing_user->password($password); }
    if (isset($first_name)) { $existing_user->first_name($first_name); }
    if (isset($last_name)) { $existing_user->last_name($last_name); }
    if (isset($admin)) { $existing_user->admin($admin); }
    $existing_user->active(true);

    return $existing_user;
  }

  // otherwise, we can create a new user, so insert the data
  $insert = array();
  $types = array();
  $insert['primary_email'] = $primary_email;
  $types[] = 'text';
  $insert['active'] = "t";
  $types[] = 'text';

  if (!empty($username) && !empty($password)) {
    $insert['username'] = $username;
    $types[] = 'text';
    $insert['password'] = md5($password);
    $types[] = 'text';
  }

  if (!empty($first_name)) {
    $insert['first_name'] = $first_name;
    $types[] = 'text';
  }
  if (!empty($last_name)) {
    $insert['last_name'] = $last_name;
    $types[] = 'text';
  }
  if (!empty($admin)) {
    $insert['admin'] = $admin?'t':'f';
    $types[] = 'text';
  }

  $result = $_ENV['dbi']->extended->autoExecute("users",
    $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $userid = dbi_last_insert_id_fix("users", "userid");

  // add the email address to the alternate_emails table
  $insert = array();
  $types = array();
  $insert['userid'] = $userid;
  $types[] = 'integer';
  $insert['email'] = $primary_email;
  $types[] = 'text';
  $result = $_ENV['dbi']->extended->autoExecute("alternate_emails",
    $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  // load the resulting object
  $user = &get_user($userid);
  
  // setting default field view preferences - John Wells 01/21/2003
  // XXX should this move somewhere else?  Should this be a default in prefs?
  if (!is_null($user)) {
    $insert = array();
    $types = array();
    $insert['userid'] = $userid;  
    $types[] = 'integer';
    $insert['value'] = "issueid,opened_by,assigned_to,modified,status,category,severity,product,flags";
    $types[] = 'text';
    $insert['preference'] = 'show_fields';
    $types[] = 'text';
    $result = $_ENV['dbi']->extended->autoExecute("user_prefs",
      $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  }

  // workaround to set default notification channel and reason
  // this must be fix it for future releases
  if (!is_null($user)) {
     $prefs[1][101] = 1;
     $prefs[2][101] = 1;
     $_ENV['notif']->set_user_prefs($userid, $prefs);
  }
  return $user;
}
/* }}} */

/* {{{ Function join_users */
/**
 * Join two distinct user rows into the same user.
 *
 * This occurs when a full-fledged user "claims" an email address that has
 * already been used in EnterTrack, and which thus has data associated with
 * it.  This can only occur if FULL_USER is a full user and EMAIL_USER is an
 * email-only user.
 *
 * @param integer $full_user userid of the full user
 * @param integer $email_user userid of the email user
 * @return boolean indicating success; any error messages will be pushed.
 */
function join_users($full_user, $email_user)
{
  // avoid SQL injection problems
  $full_user = $full_user + 0;
  $email_user = $email_user + 0;

  // verify requirements
  $stmt = $_ENV['dbi']->prepare("select username from users where userid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($full_user)); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);  
  $is_full = $result->fetchOne();
  $result->free();
  $stmt->free();
  if (!$is_full) {
    $_ENV['error']->push_error(sprintf(_("Userid %s is not a full-fledged user"), $full_user));
    return false;
  }

  $stmt = $_ENV['dbi']->prepare("select username from users where userid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($email_user)); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);  
  $is_full = $result->fetchOne();
  $result->free();
  $stmt->free();
  if ($is_full) {
    $_ENV['error']->push_error(sprintf(_("Userid %s is not an email-only user"), $email_user));
    return false;
  }

  // Clear to go ahead.  Start by updating issue owners
  $result = $_ENV['dbi']->extended->autoExecute("issues",
    array("opened_by" => $full_user),
    MDB2_AUTOQUERY_UPDATE,
    sprintf("opened_by=%s", 
        $_ENV['dbi']->quote($email_user, 'integer')),
    array("integer")); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  // Update events
  $result = $_ENV['dbi']->extended->autoExecute("events",
    array("userid" => $full_user),
    MDB2_AUTOQUERY_UPDATE,
    sprintf("userid=%s", 
        $_ENV['dbi']->quote($email_user, 'integer')),
    array("integer")); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  // Update alternate_emails
  $result = $_ENV['dbi']->extended->autoExecute("alternate_emails",
    array("userid" => $full_user),
    MDB2_AUTOQUERY_UPDATE,
    sprintf("userid=%s", 
        $_ENV['dbi']->quote($email_user, 'integer')),
    array("integer")); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  // Update issue_subscribers, deleting any potential duplicates first
  $sql = "SELECT distinct i1.issueid as issueid ";
  $sql .= "from issue_subscribers as i1 join issue_subscribers as i2 ";
  $sql .= "using(issueid) where i1.userid = ? and i2.userid = ? ";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($full_user, $email_user));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $data = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();
  if ($data) {
    foreach ($data as $row) {
      $stmt = $_ENV['dbi']->prepare("DELETE from issue_subscribers where userid=? and issueid=?"); 
      $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
      $stmt->execute(array($email_user, $row['issueid']));
      $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
      $result->free();
      $stmt->free();
    }
  }
  $result = $_ENV['dbi']->extended->autoExecute("issue_subscribers",
    array("userid" => $full_user),
    MDB2_AUTOQUERY_UPDATE,
    sprintf("userid=%s", $_ENV['dbi']->quote($email_user, 'integer')),
    array("integer")); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  // delete the old user row
  $result = $_ENV['dbi']->extended->autoExecute("users",
    null,
    MDB2_AUTOQUERY_DELETE,
    sprintf("userid=%s", $_ENV['dbi']->quote($email_user, 'integer'))); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

  // success!
  return true;
}
/* }}} */
  
////
// Utilities

/* {{{ Function generate_password */
/**
 * Generate password string
 *
 * @return string
 */
function generate_password()
{
  $pass = substr(md5(rand(1,1000000)),0,rand(6,12));
  return $pass;
}
/* }}} */

/* {{{ Function send_new_user_email */
function send_new_user_email($email, $username, $password)
{
  // send an email to the new user
  $subject = $_ENV['config']['display']['company']." Support Portal Account Created";

  $message  = "A user was created with your email address for the ".$_ENV['config']['display']['company']." Issue Tracking System.\n";
  $message .= "You may access your account by going to:\n";
  $message .= _URL_."\n";
  $message .= "Login with the following information.\n\n";
  $message .= "Username: $username\n";
  $message .= "Password: $password\n\n";
  $message .= "Once logged in you may change your password by clicking ";
  $message .= "on the 'My Settings' link on the left side of the page.  ";
  $message .= "If you have any questions please contact ".$_ENV['config']['email']['admin'].".\n\n\n";
  $message .= "            - ".$_ENV['config']['display']['company'];

  // Make sure we have mail class and initialize it
  include_once(_CLASSES_."mail.class.php");
  if (!is_object($mailer)) {
    $mailer = new MAILER();
    $mailer->set("email_from",$_ENV['config']['email']['sender']);
  }
  $mailer->subject($subject);
  $mailer->to($email);
  $mailer->message($message);
  $mailer->send();
}
/* }}} */

/* {{{ Function username */
/**
 * Returns the user's username or, if username is empty, email.
 *
 * This function is not in the 'User' class because it is often called repeatedly
 * in a single page, when fetching entire user rows is not required.
 *
 * @param integer $userid ID of user to lookup, will default to $_SESSION['userid']
 * @param boolean $link Use link or not
 * @return string $username
 */
function username($userid)
{
  if (empty($userid)) {
    return " (none) ";
  }

  // TODO add caching here, using a new 'cache' subsystem

  // form query to get username, or email if username is null
  $sql  = "SELECT username, primary_email ";
  $sql .= "FROM users ";
  $sql .= "WHERE userid=? ";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($userid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $row = $result->fetchRow(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();
  if ($row['username']) {
    return $row['username'];
  } else {
    return $row['primary_email'];
  }
}
/* }}} */

/* {{{ Function _mkident */
/* Internal use only */
function _mkident($data)
{
  $ident = $data['first_name']." ".$data['last_name'];

  if ($ident == " ") {
    $ident = $data['username'];
  }

  if ($ident == "") {
    $ident = $data['primary_email'];
  }

  return $ident;
}
/* }}} */

/* {{{ Function ident */
/**
 * Retrieve identity of user, by default will retrieve first and last name
 * if the user has not given first and last name then the username will be
 * used in its place.  Like username(), this function is defined independently
 * of the 'User' class because it is often called in situations where the
 * entire user row need not be loaded.
 *
 * @param integer $userid ID of user
 * @return string
 */
function ident($userid)
{
  if (empty($userid)) {
    return;
  }

  $sql  = "SELECT username, first_name, last_name, primary_email ";
  $sql .= "FROM users ";
  $sql .= "WHERE userid=? ";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($userid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $data = $result->fetchRow(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();

  return _mkident($data);
}
/* }}} */

/* {{{ Function enumerate_users */
/**
 * Retrieve a list of all users, optionally including email-only users, sorted by
 * last name, first name, and email.
 *
 * @param boolean $include_email_only should the results include email-only users?
 * @return array of arrays with keys userid, username, first_name, last_name,
 *   primary_email, and ident (the value that would be returned from ident($userid)
 */
function enumerate_users($include_email_only)
{
  $sql  = "SELECT userid, username, first_name, last_name, primary_email ";
  $sql .= "FROM users ";
  if (!$include_email_only) {
    $sql .= "WHERE username is not NULL ";
  }
  $sql .= "ORDER BY first_name, last_name, primary_email, username ";
  $stmt = $_ENV['dbi']->prepare($sql);
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute();
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $data = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();

  $rv = array();
  foreach ($data as $row) {
    $row['ident'] = _mkident($row);
    $rv[] = $row;
  }

  return $rv;
}
/* }}} */

/* {{{ Function enumerate_users_group */
/**
 * Retrieve a list of all users for a group, optionally including email-only users, sorted by
 * last name, first name, and email.
 *
 * @param boolean $include_email_only should the results include email-only users?
 * @return array of arrays with keys userid, username, first_name, last_name,
 *   primary_email, and ident (the value that would be returned from ident($userid)
 */
function enumerate_users_group($gid, $include_email_only)
{
  $sql  = "SELECT u.userid, u.username, u.first_name, u.last_name, u.primary_email ";
  $sql .= "FROM users u, group_users gu ";
  $sql .= "WHERE u.userid = gu.userid ";
  $sql .= "AND gu.gid = $gid  ";
  if (!$include_email_only) {
      $sql .= "AND u.username is not NULL ";
  }
  $sql .= "ORDER BY u.first_name, u.last_name ";
  $stmt = $_ENV['dbi']->prepare($sql);
  $result = $stmt->execute();
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  $data = $result->fetchAll(MDB2_FETCHMODE_ASSOC);
  $result->free();
  $stmt->free();
 
  $rv = array();
  foreach ($data as $row) {
    $row['ident'] = _mkident($row);
    $rv[] = $row;
  }

  return $rv;
}
/* }}} */




////
// Functions waiting on more subsystems

// XXX what is this function for?
/* {{{ Function init_user_groups */
/**
 * Retrive list of groups the user belongs to (propagated)
 *
 * @param integer $uid ID of user
 * @return array
 */
function init_user_groups($uid)
{
  $user = &get_user($uid);
  if (!$user) { session_destroy(); die("User not found"); }
  $groups = $user->groups();

  // Get parent and child groups (walk the tree).
  foreach ($groups as $gid) {
    // This is upward (parent) retrieval.
    $sql = "SELECT parent_gid AS gid
                FROM sub_groups WHERE child_gid=?
                AND prop_user='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    if ($gid = $result->fetchOne()) {
      if (!in_array($gid,$groups)) {
        array_push($groups,$gid);
      }
    }
    $result->free();
    $stmt->free();

    // This is downward (child) retrieval.
    $sql = "SELECT s.child_gid AS gid
        FROM sub_groups AS s
          JOIN groups AS g ON g.gid=s.parent_gid
          WHERE s.parent_gid=?
          AND g.prop_user='t'";
    $stmt = $_ENV['dbi']->prepare($sql); 
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($gid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
    while ($gid = $result->fetchOne()) {
      if (!in_array($gid,$groups)) {
          array_push($groups,$gid);
      }
    }
    $result->free();
    $stmt->free();
  }

  return $groups;
}
/* }}} */

/* {{{ Function: set_user_preference */
/**
 * Set this user's preference.
 *
 * @param string $preference to set
 * @param string $value to set it to
 */
function set_user_preference($preference, $value)
{
  $userid = $_SESSION['userid'];

  if (array_key_exists($preference, $_SESSION['prefs'])) {
    $result = $_ENV['dbi']->extended->autoExecute("user_prefs",
      array("value" => $value),
      MDB2_AUTOQUERY_UPDATE,
      sprintf("userid=%s and preference=%s", 
            $_ENV['dbi']->quote($userid, 'integer'), 
            $_ENV['dbi']->quote($preference, 'text')),
      array('text')); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  } else {
    $insert = array();
    $types = array();
    $insert['userid'] = $userid;
    $types[] = 'integer';
    $insert['preference'] = $preference;
    $types[] = 'text';
    $insert['value'] = $value;
    $types[] = 'text';
    $result = $_ENV['dbi']->extended->autoExecute("user_prefs",
      $insert, MDB2_AUTOQUERY_INSERT, null, $types); 
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  }

  $_SESSION['prefs'][$preference] = $value;
}
/* }}} */

/* {{{ Function: unset_user_preference */
/**
 * Delete this preference for this user.
 *
 * @param string $preference to unset
 */
function unset_user_preference($preference)
{
  $userid = $_SESSION['userid'];

  $result = $_ENV['dbi']->extended->autoExecute("user_prefs",
    null,
    MDB2_AUTOQUERY_DELETE,
    sprintf("userid=%s and preference=%s", 
            $_ENV['dbi']->quote($userid, 'integer'), 
            $_ENV['dbi']->quote($preference, 'text'))); 
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__);
  if (isset($_SESSION['prefs'][$preference])) {
    unset($_SESSION['prefs'][$preference]);
  }
}
/* }}} */

/* {{{ Function: user_preferences */
/**
 * Retrieve array of user preferences
 *
 * @param integer $userid ID of user
 * @return array
 */
function user_preferences($userid = null)
{
  $curr_prefs = array();

  if (is_null($userid)) {
    $userid = $_SESSION['userid'];
  }

  $stmt = $_ENV['dbi']->prepare("SELECT preference,value FROM user_prefs WHERE userid=?");
  $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
  $result = $stmt->execute(array($userid));
  $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 
  $prefs = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
  $result->free();
  $stmt->free();
  if (is_array($prefs)) {
    foreach ($prefs as $pref) {
      $curr_prefs[$pref['preference']] = $pref['value'];
    } 
  }

  if (empty($curr_prefs['sort_by'])) {
    $curr_prefs['sort_by'] = "issueid";
  }

  if (!empty($curr_prefs['show_fields'])) {
    $curr_prefs['show_fields'] = explode(",",$curr_prefs['show_fields']);
  } else {
    $curr_prefs['show_fields'] = array();
  }

  $_ENV['i18n']->setup_date_format(isset($curr_prefs['date_format'])?$curr_prefs['date_format'] : null);
  $_ENV['i18n']->setup_timezone(isset($curr_prefs['timezone'])?$curr_prefs['timezone'] : null);

  return $curr_prefs;
}
/* }}} */


/* {{{ Function: user_exists */
/**
 * Determine if a user actually exists
 *
 * @param integer $userid ID of user to look for
 * @return boolean
 */
function user_exists($userid)
{
	$exists = FALSE;

	$sql  = "SELECT userid ";
	$sql .= "FROM users ";
	$sql .= "WHERE userid=? ";

    $stmt = $_ENV['dbi']->prepare($sql);
    $_ENV['error']->check_pear_error($stmt, __FILE__, __LINE__);
    $result = $stmt->execute(array($userid));
    $_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

	$rows = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
	if( count( $rows ) ) {
		$exists = TRUE;
	}

    $result->free();
    $stmt->free();

	return $exists;
}
/* }}} */


/* {{{ Function: email */
/**
 * Takes userid and returns associated email
 *
 * @param integer $userid ID of user to retrieve email for
 * @return string
 */
function email($userid)
{
	// form query to get email
	$sql  = "SELECT primary_email ";
	$sql .= "FROM users ";
	$sql .= "WHERE userid=? ";

	$stmt = $_ENV['dbi']->prepare($sql);
	$result = $stmt->execute(array($userid));
	$_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

	#$rows = $result->fetchAll(MDB2_FETCHMODE_ASSOC); 
	$rows = $result->fetchRow(MDB2_FETCHMODE_ASSOC); 
	$email = $rows['primary_email'];
        $email = stripslashes($email);
	return $email;
}
/* }}} */


/* {{{ Function: gen_passwd */
/**
 * Generate password string
 *
 * @return string
 */
function gen_passwd()
{
	$pass = substr(md5(rand(1,1000000)),0,rand(6,12));
	return $pass;
}
/* }}} */


/* {{{ Funtion: reset_passwd */
/**
 * Reset a user's password and email the new one to them
 *
 * @param integer $userid ID of user to reset password for
 */
function reset_passwd($userid)
{
	if (!user_exists($userid)) {
		return FALSE;
	}

	$pass = gen_passwd();
	$md5_pass = md5($pass);

	// Update password
	$result = $_ENV['dbi']->extended->autoExecute("users",
	  array("password" => $md5_pass),
	  MDB2_AUTOQUERY_UPDATE,
	  sprintf("userid=%s", 
		  $_ENV['dbi']->quote($userid, 'integer')),
	  array("text")); 
	$_ENV['error']->check_pear_error($result, __FILE__, __LINE__); 

	$subject = $_ENV['config']['display']['company']." Support Portal Alert";
	$message  = "Your password to the ".$_ENV['config']['display']['company']." Support Portal has been reset.\n\n";
	$message .= "Your new password is: $pass\n\n";
	$message .= "You may login to the interface at "._URL_." ";
	$message .= "and change this password at any time.  If you do not know the username ";
	$message .= "that you were given at the time of your account creation, you can use ";
	$message .= "your email address as a login name with the password that has been ";
	$message .= "sent to you.  If you have any questions please contact ";
	$message .= $_ENV['config']['email']['admin'].".\n\n\n";
	$message .= "            - ".$_ENV['config']['display']['company'];

  // Make sure we have mail class and initialize it
  include_once(_CLASSES_."mail.class.php");
  if (!is_object($mailer)) {
    $mailer = new MAILER();
    $mailer->set("email_from",$_ENV['config']['email']['sender']);
  }
  $mailer->subject($subject);
  $mailer->to(email($userid));
  $mailer->message($message);
  $mailer->send();

  return TRUE;
}
/* }}} */

