<?php
// Run a series of tests to be sure EnterTrack will function.

define("_PATH_",dirname(dirname(__FILE__)));
include(_PATH_."/setup/util/common.php");
include(_PATH_."/conf/paths.php");

// make sure this user has authenticated for setup
if (!isset($_SESSION['authenticated']) || !$_SESSION['authenticated']) {
  redirect("index.php");
}

/*** Constants for test status, ordered from best to worst. ***/

define("TEST_GOOD", 1);
define("TEST_WARN", 2);
define("TEST_FAIL", 3);

/*** List of tests which should be invoked; filled in by assignments
 *** after each test function. ***/

$all_tests = array();

/* {{{ Function test_version */
function test_version()
{
  $min_php_version = "4.3.0";
  if (version_compare(phpversion(), $min_php_version, ">=")) {
    $result = TEST_GOOD;
    $notes = "PHP version " . phpversion() . " is OK.";
  } else {
    $result = TEST_BAD;
    $notes = "PHP version " . phpversion() . " is too old.";
  }
  $summary = "Your web server must run PHP version $min_php_version or above.";

  test_box($result, $summary, $notes);
  return $result;
}
$all_tests[] = "test_version";
/* }}} */

/* {{{ Function test_php_settings */
function test_php_settings()
{
  $summary = "Certain PHP settings must be configured correctly.";
  $notes_parts = array();
  $result = TEST_GOOD;

  if (ini_get("register_globals") == 1) {
    $notes_parts[] = "<tt>register_globals</tt> must be <i>off</i>";
    $result = TEST_FAIL;
  }

  if (ini_get("magic_quotes_gpc") == 1) {
    $notes_parts[] = "<tt>magic_quotes_gpc</tt> must be <i>off</i>";
    $result = TEST_FAIL;
  }

  if (ini_get("magic_quotes_runtime") == 1) {
    $notes_parts[] = "<tt>magic_quotes_runtime</tt> must be <i>off</i>";
    $result = TEST_FAIL;
  }

  if ($result == TEST_GOOD) {
    $notes = "All settings are correct.";
  } else {
    $notes  = "The following settings are incorrect:\n<ul align=\"left\">\n";
    foreach ($notes_parts as $notes_part) {
      $notes .= "<li>$notes_part\n";
    }
    $notes .= "</ul>\nTo correct this, please configure these settings appropriately in PHP.INI, or via your web server's configuration files.  If you are using Apache HTTPD, configuring it to recognize EnterTrack's <tt>.htaccess</tt> file should suffice.";
  }

  test_box($result, $summary, $notes);
  return $result;
}
$all_tests[] = "test_php_settings";
/* }}} */

/* {{{ Function test_mysql */
function test_mysql()
{
  $has_mysql = extension_loaded('mysql');
  if ($has_mysql) {
    $result = TEST_GOOD;
    $notes = "MySQL extension found.";
    $_SESSION['tests']['db'][] = 'mysql';
  } else {
    $result = TEST_WARN;
    $notes = "The PHP MySQL extension was not found.  This is OK, as long as you have at least one database extension loaded.";
  }
  $summary = "EnterTrack can use MySQL for its database backend.";

  test_box($result, $summary, $notes);
  return $result;
}
$all_tests[] = "test_mysql";
/* }}} */

/* {{{ Function test_pgsql */
function test_pgsql()
{
  $has_pgsql = extension_loaded('pgsql');
  if ($has_pgsql) {
    $result = TEST_GOOD;
    $notes = "PostgreSQL extension found.";
    $_SESSION['tests']['db'][] = 'pgsql';
  } else {
    $result = TEST_WARN;
    $notes = "The PHP PostgreSQL extension was not found.  This is OK, as long as you have at least one database extension loaded.";
  }
  $summary = "EnterTrack can use PostgreSQL for its database backend.";

  test_box($result, $summary, $notes);
  return $result;
}
$all_tests[] = "test_pgsql";
/* }}} */

/* {{{ Function test_somedb */
function test_somedb()
{
  // TODO: there should be a way to test this via MDB2?
  if (isset($_SESSION['tests']['db'])) {
    $result = TEST_GOOD;
    $count = sizeof($_SESSION['tests']['db']);
    if ($count == 1) { $count = "One"; }
    $notes = "$count database extension" . ($count>1?"s":"") . " found.";
  } else {
    $result = TEST_FAIL;
    $notes = "No database extension was found.  EnterTrack uses a database backend, and
              thus requires that PHP have at least one supported database extension installed.";
  }
  $summary = "You will need at least one database extension (MySQL or PostgreSQL) to run EnterTrack.";

  test_box($result, $summary, $notes);
  return $result;
}
$all_tests[] = "test_somedb";
/* }}} */

/* {{{ Function test_pear */
function test_pear()
{
  // test PEAR itself
  if (@include_once('PEAR.php')) {
    $result = TEST_GOOD;
    $notes = "PEAR is installed.";
    $SESSION['tests']['pear'] = true;
  } else {
    $result = TEST_FAIL;
    $notes = "PEAR is not installed.  See <a href=\"http://pear.php.net\">pear.php.net</a> for help installing PEAR.";
    $SESSION['tests']['pear'] = false;
  }
  $summary = "EnterTrack depends on PEAR, the PHP Extension and Application Repository, which is installed with most recent PHP packages.";

  test_box($result, $summary, $notes);
  return $result;
}
$all_tests[] = "test_pear";
/* }}} */

/* {{{ Function test_mdb2 */
function test_mdb2()
{
  $db_modules = array(
    'mysql' => array("MDB2_Driver_mysql", "MDB2/Driver/mysql.php"),
    'pgsql' => array("MDB2_Driver_pgsql", "MDB2/Driver/pgsql.php"),
    );

  if (@include_once("MDB2.php")) {
    $result = TEST_GOOD;
  } else {
    $result = TEST_FAIL;
    $notes = "The PEAR MDB2 package was not found.";
  } 

  if ($result == TEST_GOOD) {
    $apiVersionStr = MDB2::apiVersion();
    $apiVersion = explode('.', $apiVersionStr);
    if ($apiVersion[0]+0 < 2 || ($apiVersion[0]+0 == 2 && $apiVersion[1]+0 < 3)) {
      $result = TEST_FAIL;
      $notes = "MDB version 2.3.0 or later is required; version $apiVersionStr is installed.";
    }
  }

  if ($result == TEST_GOOD) {
    // test individual driver sub-packages
    $failed_modules = array();
    foreach ($_SESSION['tests']['db'] as $dbext) {
      list($name, $filename) = $db_modules[$dbext];
      if (!@include($filename)) {
        $failed_modules[] = $name;
      }
    }
    if (sizeof($failed_modules) > 0) {
      $result = TEST_WARN;
      $notes = "The following MDB2 drivers were not found, although PHP itself supports them: " . implode($failed_modules, ', ');
    } 
  }

  if ($result != TEST_GOOD) {
    $notes .= "  See <a href=\"http://pear.php.net\">pear.php.net</a> for help installing PEAR modules.";
  } else {
    $notes = "The PEAR MDB2 package and its drivers are installed";
  }

  $summary = "MDB2, the PEAR database package, allows EnterTrack to safely access MySQL and Postgres.";

  test_box($result, $summary, $notes);
  return $result;
}
$all_tests[] = "test_mdb2";
/* }}} */

/* {{{ Function test_pear_modules */
function test_pear_modules()
{
  /* this method of testing for PEAR modules was derived from the Horde project */
  $required_modules = array(
    'Mail' => 'Mail/RFC822.php',
    'Mail_Mime' => 'Mail/mimeDecode.php',
    'Date' => 'Date.php',
    );

  // test PEAR itself
  if (!@include_once('PEAR.php')) {
    $result = TEST_FAIL;
    $notes = "PEAR is not installed.  See <a href=\"http://pear.php.net\">pear.php.net</a> for help installing PEAR.";
  } else {
    // test the modules
    $good_modules = array();
    $bad_modules = array();
    foreach ($required_modules as $name => $filename) {
      $result = @include_once($filename);
      if ($result) {
        $good_modules[] = $name;
      } else {
        $bad_modules[] = $name;
      }
    }

    if (!$bad_modules) {
      $result = TEST_GOOD;
      $notes = "All required PEAR modules were found.";
    } else {
      $result = TEST_FAIL;
      if (sizeof($bad_modules) == 1) {
        $notes = "One PEAR module, <b>" . $bad_modules[0] . "</b>, was not found in your PHP installation.";
      } else {
        $notes = "The following PEAR modules were not found in your PHP installation: ";
        $bad_modules_b = array();
        foreach ($bad_modules as $module) { $bad_modules_b[] = "<b>$module</b>"; }
        $notes .= implode($bad_modules_b, ", ");
        $notes .= ".";
      }

      $notes .= "  See <a href=\"http://pear.php.net\">pear.php.net</a> for help installing PEAR modules.";
    }
  }
  $summary = "EnterTrack requires several PEAR modules to function properly.";

  test_box($result, $summary, $notes);
  return $result;
}
$all_tests[] = "test_pear_modules";
/* }}} */

/* {{{ Function test_smarty */
function test_smarty()
{
  if (!@include_once((defined('SMARTY_DIR')? SMARTY_DIR:'')."Smarty.class.php")) {
    $result = TEST_FAIL;

    $notes = "Smarty is not installed.  See <a href=\"http://smarty.php.net\">smarty.php.net</a>
        and <a href=\"http://entertrack.sourceforge.net/wiki\">the EnterTrack
        documentation</a> for help installing it.  If you are upgrading a previous
        installation of EnterTrack, and had defined <tt>SMARTY_DIR</tt> in
        <tt>conf/config.php</tt>, please move that definition to
        <tt>conf/paths.php</tt> and reload this page. ";
  } else {
    $smarty = &new Smarty;
    $notes = "Using Smarty version {$smarty->_version}.  ";
    if (version_compare($smarty->_version, '2.6.14', '<')) {
      $result = TEST_BAD;
      $notes .= "EnterTrack is not known to work with versions of Smarty before 2.6.14.";
    } else if (version_compare($smarty->_version, '2.6.16', '<')) {
      $result = TEST_WARN;
      $notes .= "There have been problems reported with versions of Smarty before 2.6.16; if possible, please upgrade Smarty.";
    } else {
      $result = TEST_GOOD;
    }
  }
  $summary = "EnterTrack uses the Smarty template engine to render pages.  This library must
     be installed separately from EnterTrack.";

  test_box($result, $summary, $notes);
  return $result;
}
$all_tests[] = "test_smarty";
/* }}} */

/* {{{ Function mbstring installed */
function test_mbstring()
{
  if (extension_loaded('mbstring')) {
    $result = TEST_GOOD;
    $notes = "mbtring extension loaded";
  } else {
    $result = TEST_FAIL;
    $notes = "Could not load mbstring extension.You must install it.";
  }
  $summary = "mbstring extension is required for work with multibyte strings like UTF-8 ones";

  test_box($result, $summary, $notes);
  return $result;
}
$all_tests[] = "test_mbstring";
/* }}} */

/* {{{ Function test_fs_permissions */
function test_fs_permissions()
{
  // +W: writeable (mandatory)
  // +w: writeable (advisory)
  // -w: not writeable (advisory)
  // -W: not writeable (mandatory)
  $directories = array(
    _FILES_ => array("+W"),
    _TPLCOMPILE_ => array("+W"),
    _LOGS_ => array("+W"),
    _CACHE_ => array("+W"),
    _SESSIONS_ => array("+w"),
    _THEMES_ => array("-w"),
    _SHARED_CODE_ => array("-w"),
    _PATH_ . '/download' => array("+w"),
  );

  $result = TEST_GOOD;
  $bad_perms = array();
  foreach ($directories as $dir => $requirements) {
    foreach ($requirements as $rq) {
      switch ($rq) {
        case '+W':
          if (!is_writeable($dir)) {
            $result = TEST_FAIL;
            $bad_perms[] = "<tt>$dir</tt> is not writeable (required writeable)";
          }
          break;

        case '+w':
          if (!is_writeable($dir)) {
            if ($result != TEST_FAIL) { $result = TEST_WARN; }
            $bad_perms[] = "<tt>$dir</tt> is not writeable (recommended writeable)";
          }
          break;

        case '-w':
          if (is_writeable($dir)) {
            if ($result != TEST_FAIL) { $result = TEST_WARN; }
            $bad_perms[] = "<tt>$dir</tt> is writeable (recommended not writeable)";
          }
          break;

        case '-W':
          if (is_writeable($dir)) {
            $result = TEST_FAIL;
            $bad_perms[] = "<tt>$dir</tt> is writeable (required not writeable)";
          }
          break;
      }
    }
  }

  // write up some notes for the results
  if ($result == TEST_GOOD) {
    $notes = "All required directories are writeable.";
  } else {
    $notes = "The following directories are not writeable:\n<ul>";
    foreach ($bad_perms as $dir) {
      $notes .= "<li>$dir\n";
    }
    $notes .= "</ul>\n";
    if ($result == TEST_WARN) {
      $notes .= "None of these permissions are required, so EnterTrack will function, but some features may not work.";
    }
  }

  $summary = "EnterTrack requires that certain subdirectories be writeable.";

  test_box($result, $summary, $notes);
  return $result;
}
$all_tests[] = "test_fs_permissions";
/* }}} */

/*** Format a box for the results of a test ***/
function test_box($result, $summary, $notes)
{
  $result_classes = array(
    TEST_GOOD => "good",
    TEST_WARN => "warn",
    TEST_FAIL => "fail"
  );
  $result_class = $result_classes[$result];

  echo "<div class=\"test\">\n";
  echo " <div class=\"$result_class\">\n";
  echo "  <div class=\"summary\">\n";
  echo $summary;
  echo "  </div><div class=\"notes\">\n";
  echo $notes;
  echo "  </div>\n";
  echo " </div>\n";
  echo "</div>\n";
}

/*** Format a summary of the results of all tests ***/
function results_summary($message, $phpinfo_link)
{
  echo "<div class=\"results_summary\">\n";
  echo $message;
  if ($phpinfo_link) {
    echo "<br />";
    echo "Consult your server's <a href=\"tests.php?show=phpinfo\">PHP Info</a> for current settings.";
  }
  echo "</div>\n";
}

/*** Draw the test page (also runs the tests) ***/
function test_page()
{
  global $all_tests;

  // Clear out the results array
  $_SESSION['tests'] = array();

  $worst_result = TEST_GOOD;
  foreach ($all_tests as $test) {
    $result = $test();

    // Record the worst result for a summary
    $worst_result = max($worst_result, $result);
  }

  switch ($worst_result) {
    case TEST_GOOD:
      results_summary("All tests passed.  You're ready to begin installation!", false);
      return true;

    case TEST_WARN:
      results_summary("Some tests resulted in warning.  This may or may not be ok -- see above for more information.", true);
      return true;

    case TEST_FAIL:
      results_summary("Some tests failed.  You'll need to address the problems above before you continue with the installation.", true);
      return false;
  }
}

// If this is a POST
if ($_SERVER['REQUEST_METHOD'] == 'POST') {
  // Check to see if this was a request to start over
  if (isset($_POST['_start_over'])) {
    redirect("index.php");
  }

  // head on to the database page
  redirect("database.php");
}

// If we're showing PHP info
if (isset($_GET['show']) && $_GET['show'] == 'phpinfo') {
  phpinfo();
  exit;
}

// Draw the install page
page_header("Setup :: Tests", "Use this page to check that your web server is ready to run EnterTrack.");

$tests_ok = test_page();
navigation_box($tests_ok);

page_footer();
