<?php

/* constants for use below */
define("CONFIG_OLD", 1);
define("CONFIG_ET_1_1_0", 2);
define("CONFIG_ET_1_2_0", 5);
define("CURRENT_CONFIG_VERSION", 5);

/* {{{ Class Configuration */
/**
 * Configuration
 *
 * This class is used both to examine and retrieve data from the current config
 * file (for upgrades), and to create a new config file (for upgrades and installs)
 */
class Configuration
{
  /* {{{ Variables */
  /* the original text of the config file */
  var $original_text;

  /* the version of original_text (or 'none' if none existed) */
  var $original_version;

  /* the config version of $data */
  var $version;

  /* the $_ENV['config'] array from $text */
  var $data;

  /* the absolute filename of conf/config.php */
  var $config_filename;
  /* }}} */

  /* {{{ Function Configuration */
  /**
   * Constructor.
   *
   * If it exists, load the existing configuration; otherwise,
   * create a new one and load that.
   */
  function Configuration()
  {
    // load up the config file
    $this->config_filename = _PATH_."/conf/config.php";

    if (file_exists($this->config_filename)) {
      $this->load($this->config_filename);
    } else {
      $this->make_default_config();
    }
  }
  /* }}} */

  /* {{{ Function load */
  /**
   * Load the configuration from the given file.  This function
   * automatically detects and upgrades "old" (i.e., not using $_ENV)
   * configuration files and represents them as if an equivalent new
   * configuration file were loaded.
   *
   * @param string $filename the file to load
   */
  function load($filename)
  {
    $text = file_get_contents($filename);

    $this->original_text = $this->strip_php_tags($text);
    $this->original_version = $this->get_config_version($this->original_text);

    // convert an "old-style" (non-$_ENV['config']) config
    if ($this->original_version == CONFIG_OLD) {
      $this->data = $this->load_old_config($this->original_text);
      $this->version = CONFIG_ET_1_1_0;
    } else {
      $this->data = $this->load_new_config($this->original_text);
      $this->version = $this->original_version; // version has not changed
    }
  }
  /* }}} */

  /* {{{ Function strip_php_tags */
  /**
   * Remove all PHP tags from the given text
   */
  function strip_php_tags($text)
  {
    $text = str_replace('<?php', '', $text);
    $text = str_replace('?>', '', $text);
    $text = trim($text);
    return $text;
  }
  /* }}} */

  /* {{{ Function load_old_config */
  /**
    * Create a "new" configuration file from the old one; in
    * this case, we construct an et-1.0.1 config file which can
    * then be upgraded as below.  We have local variables here, so
    * we can't do this in another function.
    */
  function load_old_config($text)
  {
    // load the config into local variables
    eval($text);

    // now construct an et-1.1.0 config from that data
    // (it will be upgraded from there)
    $c = array();

    $c['db'] = array();
    $c['db']['type'] = $db['type'];
    $c['db']['host'] = $db['host'];
    $c['db']['port'] = $db['port'];
    $c['db']['name'] = $db['name'];
    $c['db']['user'] = $db['user'];
    $c['db']['pass'] = $db['pass'];

    $c['email'] = array();
    $c['email']['admin'] = _ADMINEMAIL_;
    $c['email']['sender'] = _EMAIL_;
    $c['email']['default_groupid'] = null;
    $c['email']['shared_secret'] = uniqid('shared');

    $c['display'] = array();
    $c['display']['title'] = _TITLE_;
    $c['display']['company'] = _COMPANY_;
    $c['display']['default_theme'] = $default_theme;

    $c['i18n'] = array();
    $c['i18n']['locale'] = 'en_US';
    $c['i18n']['use_browser_timezone'] = TRUE;
    $c['i18n']['company_timezone'] = 'US/Eastern';
    $c['i18n']['company_date_format'] = 'intl';

    $c['debug'] = array();
    $c['debug']['et_handler'] = $error_handler;
    $c['debug']['backtrace'] = $backtrace;

    $c['session'] = array();
    $c['session']['et_handler'] = $session_handler;
    $c['session']['name'] = 'EnterTrack';

    $c['features'] = array();
    $c['features']['disable_edit'] = $disable_edit;
    $c['features']['auto_register'] = $allow_register;
    $c['features']['auto_register_group'] = _DEFGRP_;

    $c['limits'] = array();
    $c['limits']['max_event'] = MAXLENGTH;

    $c['bad_mime'] = array();
    $c['bad_mime'] = $bad_mime;

    $c['cache'] = array();
    $keys = array("users", "groups", "statuses", "categories", "products", "permissions");
    foreach ($keys as $k) {
      $c['cache'][$k] = in_array($k, $cache_data);
    }

    return $c;
  }
  /* }}} */

  /* {{{ Function load_new_config */
  /**
    * Create a "new" configuration file from the new one; in
    * this case, we construct an et-1.0.1 config file which can
    * then be upgraded as below.
    */
  function load_new_config($text)
  {
    // cleanup
    if (isset($_ENV['config'])) { unset($_ENV['config']); }

    // load the config into $_ENV['config']
    eval($text);
    $c = $_ENV['config'];

    // more cleanup
    unset($_ENV['config']);

    // and return that
    return $c;
  }
  /* }}} */

  /* {{{ Function make_default_config */
  /**
   * Create a default configuration
   */
  function make_default_config()
  {
    $this->original_text = '';
    $this->original_version = 'none';

    $this->version = CURRENT_CONFIG_VERSION;

    $this->data = array();
    $this->data['db'] = array();
    $this->data['db']['type'] = '';
    $this->data['db']['host'] = 'localhost';
    $this->data['db']['port'] = '';
    $this->data['db']['name'] = 'entertrack';
    $this->data['db']['user'] = '';
    $this->data['db']['pass'] = '';
    $this->data['email'] = array();
    $this->data['email']['admin'] = 'webmaster@localhost';
    $this->data['email']['sender'] = 'nobody@localhost';
    $this->data['email']['default_groupid'] = null;
    $this->data['email']['shared_secret'] = uniqid('shared');
    $this->data['display'] = array();
    $this->data['display']['title'] = 'EnterTrack';
    $this->data['display']['company'] = 'Test Install';
    $this->data['display']['default_theme'] = 'default';
    $this->data['i18n'] = array();
    $this->data['i18n']['locale'] = 'en_US';
    $this->data['i18n']['use_browser_timezone'] = true;
    $this->data['i18n']['company_timezone'] = 'US/Eastern';
    $this->data['i18n']['company_date_format'] = 'intl';
    $this->data['debug'] = array();
    $this->data['debug']['display_errors'] = true;
    $this->data['debug']['error_reporting'] = E_ERROR;
    $this->data['session'] = array();
    $this->data['session']['et_handler'] = false;
    $this->data['session']['name'] = 'EnterTrack';
    $this->data['features'] = array();
    $this->data['features']['disable_edit'] = false;
    $this->data['features']['auto_register'] = false;
    $this->data['features']['auto_register_group'] = null;
    $this->data['limits'] = array( 256000 );
    $this->data['bad_mime'] = array( 'php', 'php3', 'php4', 'cgi', 'pl', 'htm', 'html', 'shtml', 'dhtml', 'asp', 'py', 'xml', 'wmls', 'css', 'sh' );
    $this->data['cache'] = array();
    $this->data['cache']['users'] = true;
    $this->data['cache']['groups'] = true;
    $this->data['cache']['statuses'] = true;
    $this->data['cache']['categories'] = true;
    $this->data['cache']['products'] = true;
    $this->data['cache']['permissions'] = true;
  }
  /* }}} */

  /* {{{ Function generate_new_text */
  /**
   * Generate the textual representation of $this->data.
   *
   * @return string the textual representation
   */
  function generate_new_text()
  {
    $conf = $this->gen('$_ENV[\'config\']', $this->data);
    return "//FILE VERSION>> {$this->version} <<DO NOT EDIT\n$conf";
  }
  /* {{{ Helper functions */
  function scalar($v) {
    if (is_integer($v) || is_float($v)) {
      return $v . '';
    } elseif (is_bool($v)) {
      return $v?'true':'false';
    } elseif (is_null($v)) {
      return 'null';
    } else {
      return "'" . str_replace("'", "\\'", str_replace("\\", "\\\\", $v)) . "'";
    }
  }

  function gen($prefix, $value) {
    if (is_array($value)) {
      // first determine if it's associative or not, and whether
      // it's just a list of scalars or not.
      $associative = false;
      $all_scalars = true;
      $i = 0;
      foreach ($value as $k => $v) {
        if ($k != $i++) { $associative = true; }
        if (!is_scalar($v)) { $all_scalars = false; }
      }

      // now do a different thing for each possibility
      if ($all_scalars && !$associative) {
        // list of scalars
        $vals = array();
        foreach ($value as $v) {
          $vals[] = $this->scalar($v);
        }
        return "$prefix = array( " . implode($vals, ", ") . " );\n";
      } else {
        // an array or complex values gets put on multiple lines
        $rv = array( "$prefix = array();\n" );
        foreach ($value as $k => $v) {
          $kstr = $this->scalar($k);
          $rv[] = $this->gen($prefix.'['.$kstr.']', $v);
        }
        return implode($rv, '');
      }
    } else {

      // this isn't an array
      return "$prefix = " . $this->scalar($value) . ";\n";
    }
  }
  /* }}} */
  /* }}} */

  /* {{{ Function default_filename */
  /**
   * Get the default filename for the configuration
   *
   * @return string
   */
  function default_filename()
  {
    return $this->config_filename;
  }
  /* }}} */

  /* {{{ Function save */
  /**
   * Save the configuration file in $text to $this->config_filename,
   * making a backup of any existing file.  Returns a boolean
   * indicating success; failure is most likely due to permissions
   * problems.
   *
   * @param string $text the text of the configuration
   * @param string $filename (optional)
   * @return boolean
   */
  function save($text, $filename=null)
  {
    // get a filename
    if (!$filename) { $filename = $this->config_filename; }

    // make a backup filename
    $backup_filename = preg_replace('/.php$/', '-back.php', $filename);

    // test writeability
    $directory = dirname($filename);
    if (!is_writeable($directory)) { return false; }

    // make a backup
    if (file_exists($filename)) {
      if (!copy($filename, $backup_filename)) {
        return false;
      }
    }

    // Add an opening PHP tag (closing is not required)
    // (php tag is obfuscated here so as not to confuse text editors)
    $text = "<?p"."hp\n$text\n";

    // write it out
    if (!($fd = @fopen($filename, 'w'))) { return false; }
    if (@fwrite($fd, $text) < strlen($text)) { return false; }
    @fclose($fd);

    return true;
  }
  /* }}} */

  /* {{{ Function get_config_version */
  /**
   * Dig the config version out of the configuration text, or
   * specially detect et-1.1.0 and "old" configurations.
   *
   * ET uses a comment for the version number so that it's not
   * necessary to 'eval' the config file to determine its version.
   *
   * @param $text the text to analyze
   * @return the configuration version
   */
  function get_config_version($text)
  {
    // if we have a config version in the file, use that
    if (preg_match('/\/\/FILE VERSION>>([^<]*)<<DO NOT EDIT/', $text, $matches)) {
      return trim($matches[1])+0;

    // if it has $_ENV['config'] in it, then it's got to be et-1.1.0, which
    // is the only release with a new-style config file and no config version
    } elseif (strpos($text, '$_ENV[\'config\']')) {
      return CONFIG_ET_1_1_0;

    // otherwise, it's 'old'.
    } else {
      return CONFIG_OLD; // et-1.0.1 or itv4
    }
  }
  /* }}} */

  /* {{{ Function needs_upgrade */
  /**
   * Does this configuration file need to be updated?
   *
   * @return boolean
   */
  function needs_upgrade()
  {
    return !($this->version == CURRENT_CONFIG_VERSION);
  }
  /* }}} */

  /* {{{ Function upgrade */
  /**
   * Upgrade the configuration object to the current version of
   * EnterTrack.
   */
  function upgrade()
  {
    switch ($this->version) {
      // an "old" config file will already have been upgraded
      // as it was loaded, so just start with ET_1_1_0.
      case CONFIG_ET_1_1_0:
        $this->upgrade_from_et_1_1_0();
      case 3:
        $this->upgrade_auto_register_changes();
      case 4:
        $this->upgrade_remove_error_et_handler();
      case CURRENT_CONFIG_VERSION:
        $this->version = CURRENT_CONFIG_VERSION;
    }
  }
  /* }}} */

  /* {{{ Function upgrade_from_et_1_1_0 */
  function upgrade_from_et_1_1_0()
  {
    // we have a few new configuration variables
    echo "<div class=\"step\">\n";
    echo "Adding new configuration variables to upgrade from EnterTrack-1.1.0:";

    echo <<<EOT
<pre class="config">
\$_ENV['config']['debug']['display_errors'] = false;
\$_ENV['config']['debug']['error_reporting'] = E_ERROR;
</pre>
EOT;

    $this->data['debug']['display_errors'] = false;
    $this->data['debug']['error_reporting'] = E_ERROR;
    echo "</div>\n";
  }
  /* }}} */

  /* {{{ Function upgrade_auto_register_changes */
  function upgrade_auto_register_changes()
  {
    // we have a few new configuration variables
    echo "<div class=\"step\">\n";
    echo "Upgrading auto_registration configuration into two variables:";

    echo "<pre class=\"config\">\n";
    if ($this->data['features']['auto_register_group']) {
      echo "\$_ENV['config']['features']['auto_register'] = true;\n";
      $this->data['features']['auto_register'] = true;
    } else {
      echo "\$_ENV['config']['features']['auto_register'] = false;\n";
      echo "\$_ENV['config']['features']['auto_register_group'] = null;\n";
      $this->data['features']['auto_register'] = false;
      $this->data['features']['auto_register_group'] = null;
    }
    echo "</div>\n";
  }
  /* }}} */

  /* {{{ Function upgrade_remove_error_et_handler */
  function upgrade_remove_error_et_handler()
  {
    echo "<div class=\"step\">\n";
    echo "Removing <tt>\$_ENV['debug']['et_handler']</tt>.";
    unset($this->data['debug']['et_handler']);
    echo "</div>\n";
  }
  /* }}} */
}
/* }}} */
