<?php

/* Definitions for the EnterTracak schema */

define("CURRENT_SCHEMA_VERSION", 15);

/* {{{ Class definitions */
/* {{{ Class Table */
class Table
{
  var $name;
  var $comment = '';
  var $indices;
  var $columns;

  function Table($name, $comment='')
  {
    $this->name = $name;
    $this->comment = $comment;
    $this->indices = array();
    $this->columns = array();
  }

  function sql()
  {
    $rv = array(); // the array of statements we'll return

    // set up the CREATE TABLE statement
    $create = "CREATE table {$this->name} (\n  ";
    $col_defs = array();
    foreach ($this->columns as $col_name => $column) {
      $col_defs[] = "$col_name " . $column->sql();
    }
    $create .= implode($col_defs, ",\n  ");
    $create .= "\n)";
    $rv[] = $create;

    // Add a comment (postgresql only)
    if ($this->comment and $_SESSION['database']['type'] == 'pgsql') {
      $rv[] = "COMMENT on table {$this->name} is '" . pg_escape_string($this->comment) . "'";
    }

    // Add permissions if we're logged in as an admin user (postgresql only)
    if ($_SESSION['database']['type'] == 'pgsql'
        && $_SESSION['database']['admin_user']
        && $_SESSION['database']['admin_user'] != $_SESSION['database']['user']) {
      $rv[] = "GRANT SELECT, UPDATE, INSERT, DELETE on table {$this->name} to {$_SESSION['database']['user']}";
      // Postgres have problems with permission denied in the sequences and the next solves
      $rv[] = "ALTER TABLE {$this->name} OWNER TO {$_SESSION['database']['user']}";
    }

    // Add any indices
    foreach ($this->indices as $index) {
      $rv[] = $index->sql($this->name);
    }

    // return the array of queries
    return $rv;
  }
}
/* }}} */

/* {{{ Class Index */
class Index
{
  // no need for anything here at present
}
/* }}} */

/* {{{ Class UniqueIndex */
class UniqueIndex extends Index
{
  var $name;
  var $columns;

  function UniqueIndex($name, $columns)
  {
    $this->name = $name;
    $this->columns = $columns;
  }

  function sql($table)
  {
    $columns = implode($this->columns, ", ");
    switch ($_SESSION['database']['type']) {
      case 'mysql':
        return "ALTER TABLE $table ADD UNIQUE INDEX {$this->name} ($columns)";

      case 'pgsql':
        return "CREATE UNIQUE INDEX {$this->name} on $table ($columns)";

      default:
        die("unknown db type");
    }
  }
}
/* }}} */

/* {{{ Class PrimaryKey */
/* NOTE: Do not set an explicit PrimaryKey for a ColSerial
 * column -- it will be set automatically. */
class PrimaryKey extends Index
{
  var $name;
  var $columns;

  function PrimaryKey($name, $columns)
  {
    $this->name = $name;
    $this->columns = $columns;
  }

  function sql($table)
  {
    $columns = implode($this->columns, ", ");
    switch ($_SESSION['database']['type']) {
      case 'mysql':
        return "ALTER TABLE $table ADD PRIMARY KEY {$this->name} ($columns)";

      case 'pgsql':
        return "ALTER TABLE $table ADD CONSTRAINT {$this->name} PRIMARY KEY ($columns)";

      default:
        die("unknown db type");
    }
  }
}
/* }}} */

/* {{{ Class SortIndex */
class SortIndex extends Index
{
  var $name;
  var $columns;

  function SortIndex($name, $columns)
  {
    $this->name = $name;
    $this->columns = $columns;
  }

  function sql($table)
  {
    $columns = implode($this->columns, ", ");
    switch ($_SESSION['database']['type']) {
      case 'mysql':
        return "ALTER TABLE $table ADD INDEX {$this->name} ($columns)";

      case 'pgsql':
        return "CREATE INDEX {$this->name} on $table ($columns)";

      default:
        die("unknown db type");
    }
  }
}
/* }}} */

/* {{{ Class Column */
class Column
{
  var $notnull;
  var $defaultvalue;

  function Column($notnull=false, $defaultvalue='')
  {
    $this->notnull = $notnull;
    $this->defaultvalue = $defaultvalue;
  }

  /**
   * Return the SQL for this column
   *
   * Return the SQL to specify this column, e.g.,
   *  CHAR(17) NOT NULL DEFAULT 'himom'
   */
  function sql()
  {
    $rv = $this->sql_type();
    if ($this->notnull) {
      $rv .= " NOT NULL";
    }
    if ($this->defaultvalue) {
      $rv .= " DEFAULT '" . addslashes($this->defaultvalue) . "'";
    }

    return $rv;
  }

  /**
   * Return SQL for this column, without DEFAULT or NOT NULL
   *
   * Return the SQL for the column type only, e.g., CHAR(17)
   */
  function sql_type() { }
}
/* }}} */

/* {{{ Class ColSmallInteger */
class ColSmallInteger extends Column
{
  function sql_type()
  {
    return 'SMALLINT';
  }
}
/* }}} */

/* {{{ Class ColInteger */
class ColInteger extends Column
{
  function sql_type()
  {
    return 'INTEGER';
  }
}
/* }}} */

/* {{{ Class ColSerial */
class ColSerial extends Column
{
  function sql_type()
  {
    switch ($_SESSION['database']['type']) {
      case 'mysql': return 'INTEGER AUTO_INCREMENT PRIMARY KEY';
      case 'pgsql': return 'SERIAL PRIMARY KEY';
    }
  }
}
/* }}} */

/* {{{ Class ColNumeric */
class ColNumeric extends Column
{
  var $left;
  var $right;

  function ColNumeric($left, $right, $notnull=false, $defaultvalue='')
  {
    $this->Column($notnull, $defaultvalue);
    $this->left = $left;
    $this->right = $right;
  }

  function sql_type()
  {
    return "NUMERIC({$this->left}, {$this->right})";
  }
}
/* }}} */

/* {{{ Class ColString */
class ColString extends Column
{
  var $width;

  function ColString($width, $notnull=false, $defaultvalue='')
  {
    $this->Column($notnull, $defaultvalue);
    $this->width = $width;
  }

  function sql_type()
  {
    return "VARCHAR({$this->width})";
  }
}
/* }}} */

/* {{{ Class ColText */
class ColText extends Column
{
  function sql_type()
  {
    switch ($_SESSION['database']['type']) {
      case 'mysql': return 'MEDIUMTEXT';
      case 'pgsql': return 'TEXT';
    }
  }
}
/* }}} */

/* {{{ Class ColBoolean */
class ColBoolean extends Column
{
  function sql_type()
  {
    return 'CHAR(1)';
  }
}
/* }}} */

/* {{{ Class ColBadBoolean (DO NOT USE) */
/* This exists to support upgrading columsn that *used*
   to use this type for boolean columns.  Don't use it
   for any new columns! */
class ColBadBoolean extends Column
{
  function sql_type()
  {
    switch ($_SESSION['database']['type']) {
      case 'mysql': return "enum('0','1')";
      case 'pgsql': return "boolean";
    }
  }
}
/* }}} */
/* }}} */

/* {{{ The schema itself */
/**
 * The schema is stored in $_ENV['schema'] for easy access inside and outside of functions.
 * It is represented as an array of tables.
 */
$_ENV['schema'] = array();

/* {{{ Table group_istatuses */
$_ENV['schema']['group_istatuses'] = new Table("group_istatuses");
$_ENV['schema']['group_istatuses']->columns['gid'] = new ColInteger(true);
$_ENV['schema']['group_istatuses']->columns['sid'] = new ColInteger(true);
/* }}} */

/* {{{ Table permission_sets */
$_ENV['schema']['permission_sets'] = new Table("permission_sets");
$_ENV['schema']['permission_sets']->columns['permsetid'] = new ColSerial(true);
$_ENV['schema']['permission_sets']->columns['name'] = new ColString(32, true);
$_ENV['schema']['permission_sets']->columns['description'] = new ColText(true);
$_ENV['schema']['permission_sets']->columns['permissions'] = new ColText(true);
$_ENV['schema']['permission_sets']->columns['system'] = new ColBoolean(false, 'f');
/* }}} */

/* {{{ Table logs */
$_ENV['schema']['logs'] = new Table("logs");
$_ENV['schema']['logs']->columns['lid'] = new ColSerial(true);
$_ENV['schema']['logs']->columns['log_type'] = new ColString(32);
$_ENV['schema']['logs']->columns['log_message'] = new ColText(true);
$_ENV['schema']['logs']->columns['log_time'] = new ColInteger(true);
$_ENV['schema']['logs']->columns['log_user'] = new ColInteger(true);
/* }}} */

/* {{{ Table group_fields */
$_ENV['schema']['group_fields'] = new Table("group_fields");
$_ENV['schema']['group_fields']->columns['gfid'] = new ColSerial(true);
$_ENV['schema']['group_fields']->columns['gid'] = new ColInteger(true);
$_ENV['schema']['group_fields']->columns['typeid'] = new ColInteger(true);
$_ENV['schema']['group_fields']->columns['name'] = new ColString(32, true);
$_ENV['schema']['group_fields']->columns['label'] = new ColString(32);
$_ENV['schema']['group_fields']->indices[] = new UniqueIndex('group_fields_gid_name_uniq', array('gid', 'name'));
/* }}} */

/* {{{ Table event_modifications */
$_ENV['schema']['event_modifications'] = new Table("event_modifications");
$_ENV['schema']['event_modifications']->columns['eid'] = new ColInteger(true);
$_ENV['schema']['event_modifications']->columns['modified'] = new ColInteger(true);
$_ENV['schema']['event_modifications']->columns['userid'] = new ColInteger(true);
$_ENV['schema']['event_modifications']->columns['changes'] = new ColText(true);
/* }}} */

/* {{{ Table issue_groups */
$_ENV['schema']['issue_groups'] = new Table("issue_groups");
$_ENV['schema']['issue_groups']->columns['issueid'] = new ColInteger(true);
$_ENV['schema']['issue_groups']->columns['gid'] = new ColInteger(true);
$_ENV['schema']['issue_groups']->columns['assigned_to'] = new ColInteger();
$_ENV['schema']['issue_groups']->columns['opened'] = new ColInteger();
$_ENV['schema']['issue_groups']->columns['first_response'] = new ColInteger(false, '0');
$_ENV['schema']['issue_groups']->columns['show_issue'] = new ColBoolean(false, 't');
$_ENV['schema']['issue_groups']->columns['deescalated'] = new ColBoolean(false, 'f');
$_ENV['schema']['issue_groups']->indices[] = new SortIndex("idx_issue_groups_issueid", array("issueid"));
/* }}} */

/* {{{ Table issue_log */
$_ENV['schema']['issue_log'] = new Table("issue_log");
$_ENV['schema']['issue_log']->columns['ilogid'] = new ColSerial(true);
$_ENV['schema']['issue_log']->columns['issueid'] = new ColInteger(true);
$_ENV['schema']['issue_log']->columns['userid'] = new ColInteger(true);
$_ENV['schema']['issue_log']->columns['logged'] = new ColInteger(true);
$_ENV['schema']['issue_log']->columns['message'] = new ColText();
$_ENV['schema']['issue_log']->columns['private'] = new ColBoolean(false, 'f');
$_ENV['schema']['issue_log']->indices[] = new SortIndex("idx_issue_log_issueid", array("issueid"));
/* }}} */

/* {{{ Table user_permissions */
$_ENV['schema']['user_permissions'] = new Table("user_permissions");
$_ENV['schema']['user_permissions']->columns['permid'] = new ColInteger();
$_ENV['schema']['user_permissions']->columns['userid'] = new ColInteger();
/* }}} */

/* {{{ Table motd */
$_ENV['schema']['motd'] = new Table("motd");
$_ENV['schema']['motd']->columns['motd'] = new ColText();
/* }}} */

/* {{{ Table issues */
$_ENV['schema']['issues'] = new Table("issues");
$_ENV['schema']['issues']->columns['issueid'] = new ColSerial(true);
$_ENV['schema']['issues']->columns['primary_gid'] = new ColInteger(true);
$_ENV['schema']['issues']->columns['opened_by'] = new ColInteger(true);
$_ENV['schema']['issues']->columns['opened'] = new ColInteger(true);
$_ENV['schema']['issues']->columns['closed'] = new ColInteger();
$_ENV['schema']['issues']->columns['modified'] = new ColInteger();
$_ENV['schema']['issues']->columns['summary'] = new ColText(true);
$_ENV['schema']['issues']->columns['problem'] = new ColText(true);
$_ENV['schema']['issues']->columns['status'] = new ColSmallInteger(true);
$_ENV['schema']['issues']->columns['category'] = new ColSmallInteger();
$_ENV['schema']['issues']->columns['product'] = new ColInteger();
$_ENV['schema']['issues']->columns['severity'] = new ColSmallInteger(true);
$_ENV['schema']['issues']->columns['private'] = new ColBoolean(false, 'f');
$_ENV['schema']['issues']->columns['flags'] = new ColText();
$_ENV['schema']['issues']->columns['istatus'] = new ColSmallInteger();
$_ENV['schema']['issues']->columns['iseverity'] = new ColSmallInteger();
$_ENV['schema']['issues']->columns['due_date'] = new ColInteger();
$_ENV['schema']['issues']->columns['real_opened'] = new ColInteger();
$_ENV['schema']['issues']->columns['real_closed'] = new ColInteger();
$_ENV['schema']['issues']->columns['modification_note'] = new ColString(150);
$_ENV['schema']['issues']->columns['adjusted_by'] = new ColInteger();
$_ENV['schema']['issues']->indices[] = new SortIndex("idx_issues_status", array("status"));
/* }}} */

/* {{{ Table group_field_values */
$_ENV['schema']['group_field_values'] = new Table("group_field_values");
$_ENV['schema']['group_field_values']->columns['gfid'] = new ColInteger(true);
$_ENV['schema']['group_field_values']->columns['value_idx'] = new ColInteger(true);
$_ENV['schema']['group_field_values']->columns['value'] = new ColText();
$_ENV['schema']['group_field_values']->columns['is_default'] = new ColBoolean(true, 'f');
$_ENV['schema']['group_field_values']->indices[] = new PrimaryKey('group_field_values_pkey', array('gfid', 'value_idx'));
/* }}} */

/* {{{ Table sub_groups */
$_ENV['schema']['sub_groups'] = new Table("sub_groups");
$_ENV['schema']['sub_groups']->columns['parent_gid'] = new ColInteger(true);
$_ENV['schema']['sub_groups']->columns['child_gid'] = new ColInteger(true);
$_ENV['schema']['sub_groups']->columns['prop_user'] = new ColBoolean(false, 'f');
$_ENV['schema']['sub_groups']->columns['prop_category'] = new ColBoolean(false, 'f');
$_ENV['schema']['sub_groups']->columns['prop_product'] = new ColBoolean(false, 'f');
$_ENV['schema']['sub_groups']->columns['prop_status'] = new ColBoolean(false, 'f');
$_ENV['schema']['sub_groups']->columns['prop_issue'] = new ColBoolean(false, 'f');
$_ENV['schema']['sub_groups']->columns['prop_announce'] = new ColBoolean(false, 'f');
$_ENV['schema']['sub_groups']->indices[] = new SortIndex("idx_sub_groups_parent_gid", array("parent_gid"));
$_ENV['schema']['sub_groups']->indices[] = new SortIndex("idx_sub_groups_child_gid", array("child_gid"));
/* }}} */

/* {{{ Table issue_group_fields */
$_ENV['schema']['issue_group_fields'] = new Table("issue_group_fields");
$_ENV['schema']['issue_group_fields']->columns['issueid'] = new ColInteger(true);
$_ENV['schema']['issue_group_fields']->columns['gfid'] = new ColInteger(true);
$_ENV['schema']['issue_group_fields']->columns['value_idx'] = new ColInteger();
$_ENV['schema']['issue_group_fields']->columns['value'] = new ColText();
$_ENV['schema']['issue_group_fields']->indices[] = new PrimaryKey('issue_group_fields_pkey', array('issueid', 'gfid'));
/* }}} */

/* {{{ Table menus */
$_ENV['schema']['menus'] = new Table("menus");
$_ENV['schema']['menus']->columns['mid'] = new ColSerial(true);
$_ENV['schema']['menus']->columns['userid'] = new ColInteger(true);
$_ENV['schema']['menus']->columns['text'] = new ColString(24, true);
$_ENV['schema']['menus']->columns['url'] = new ColText(true);
/* }}} */

/* {{{ Table statuses */
$_ENV['schema']['statuses'] = new Table("statuses");
$_ENV['schema']['statuses']->columns['sid'] = new ColSerial(true);
$_ENV['schema']['statuses']->columns['status'] = new ColString(32, true);
$_ENV['schema']['statuses']->columns['status_type'] = new ColSmallInteger();
/* }}} */

/* {{{ Table group_products */
$_ENV['schema']['group_products'] = new Table("group_products");
$_ENV['schema']['group_products']->columns['gid'] = new ColInteger(true);
$_ENV['schema']['group_products']->columns['pid'] = new ColInteger(true);
$_ENV['schema']['group_products']->indices[] = new SortIndex("idx_group_products_gid", array("gid"));
/* }}} */

/* {{{ Table group_prefs */
$_ENV['schema']['group_prefs'] = new Table("group_prefs");
$_ENV['schema']['group_prefs']->columns['gid'] = new ColInteger(true);
$_ENV['schema']['group_prefs']->columns['preference'] = new ColString(255);
$_ENV['schema']['group_prefs']->columns['value'] = new ColString(255);
/* }}} */

/* {{{ Table products_levels */ 
$_ENV['schema']['products_levels'] = new Table("products_levels");
$_ENV['schema']['products_levels']->columns['pid'] = new ColInteger(true);
$_ENV['schema']['products_levels']->columns['level'] = new ColInteger(true);
$_ENV['schema']['products_levels']->columns['color'] = new ColString(32, true);
$_ENV['schema']['products_levels']->indices[] = new SortIndex("products_levels_pkey", array("pid","level"));
/* }}} */

/* {{{ Table group_prefs */
$_ENV['schema']['group_unnasigned'] = new Table("group_unnasigned");
$_ENV['schema']['group_unnasigned']->columns['gid'] = new ColInteger(true);
$_ENV['schema']['group_unnasigned']->columns['show_group'] = new ColBoolean(false, 't');
/* }}} */

/* {{{ Table user_prefs */
$_ENV['schema']['user_prefs'] = new Table("user_prefs");
$_ENV['schema']['user_prefs']->columns['userid'] = new ColInteger(true);
$_ENV['schema']['user_prefs']->columns['preference'] = new ColString(255);
$_ENV['schema']['user_prefs']->columns['value'] = new ColString(255);
$_ENV['schema']['user_prefs']->indices[] = new SortIndex("idx_user_prefs_userid", array("userid"));
/* }}} */

/* {{{ Table announcements */
$_ENV['schema']['announcements'] = new Table("announcements");
$_ENV['schema']['announcements']->columns['aid'] = new ColSerial(true);
$_ENV['schema']['announcements']->columns['title'] = new ColString(128, true);
$_ENV['schema']['announcements']->columns['message'] = new ColText(true);
$_ENV['schema']['announcements']->columns['posted'] = new ColInteger();
$_ENV['schema']['announcements']->columns['userid'] = new ColInteger();
$_ENV['schema']['announcements']->columns['is_global'] = new ColBoolean(false, 'f');
/* }}} */

/* {{{ Table announce_permissions */
$_ENV['schema']['announce_permissions'] = new Table("announce_permissions");
$_ENV['schema']['announce_permissions']->columns['aid'] = new ColInteger(true);
$_ENV['schema']['announce_permissions']->columns['gid'] = new ColInteger(true);
/* }}} */

/* {{{ Table events */
$_ENV['schema']['events'] = new Table("events");
$_ENV['schema']['events']->columns['eid'] = new ColSerial(true);
$_ENV['schema']['events']->columns['issueid'] = new ColInteger(true);
$_ENV['schema']['events']->columns['status'] = new ColSmallInteger();
$_ENV['schema']['events']->columns['userid'] = new ColInteger();
$_ENV['schema']['events']->columns['performed_on'] = new ColInteger(true);
$_ENV['schema']['events']->columns['duration'] = new ColNumeric(6,2,false,'0.00');
$_ENV['schema']['events']->columns['fid'] = new ColInteger();
$_ENV['schema']['events']->columns['private'] = new ColBoolean(false, 'f');
$_ENV['schema']['events']->columns['action'] = new ColText(true);
$_ENV['schema']['events']->columns['gid'] = new ColInteger(); # track group at time of event creation
$_ENV['schema']['events']->indices[] = new SortIndex("idx_events_issueid", array("issueid"));
$_ENV['schema']['events']->indices[] = new SortIndex("idx_events_fid", array("fid"));
$_ENV['schema']['events']->indices[] = new SortIndex("idx_events_gid", array("gid"));
/* }}} */

/* {{{ Table files */
$_ENV['schema']['files'] = new Table("files");
$_ENV['schema']['files']->columns['fid'] = new ColSerial(true);
$_ENV['schema']['files']->columns['userid'] = new ColInteger();
$_ENV['schema']['files']->columns['typeid'] = new ColInteger(true);
$_ENV['schema']['files']->columns['uploaded_on'] = new ColInteger();
$_ENV['schema']['files']->columns['name'] = new ColText(true);
$_ENV['schema']['files']->columns['file_type'] = new ColString(16, true);
$_ENV['schema']['files']->columns['private'] = new ColBoolean(false, 'f');
/* }}} */

/* {{{ Table status_reports */
$_ENV['schema']['status_reports'] = new Table("status_reports");
$_ENV['schema']['status_reports']->columns['userid'] = new ColInteger();
$_ENV['schema']['status_reports']->columns['gid'] = new ColInteger();
$_ENV['schema']['status_reports']->columns['date_entered'] = new ColInteger(true);
$_ENV['schema']['status_reports']->columns['info'] = new ColText();
$_ENV['schema']['status_reports']->columns['standing'] = new ColSmallInteger();
$_ENV['schema']['status_reports']->indices[] = new SortIndex("status_reports_gid", array("gid"));
/* }}} */

/* {{{ Table users */
$_ENV['schema']['users'] = new Table("users");
$_ENV['schema']['users']->columns['userid'] = new ColSerial(true);
$_ENV['schema']['users']->columns['username'] = new ColString(32, false);
$_ENV['schema']['users']->columns['password'] = new ColString(32, false);
$_ENV['schema']['users']->columns['first_name'] = new ColString(32);
$_ENV['schema']['users']->columns['last_name'] = new ColString(32);
$_ENV['schema']['users']->columns['address'] = new ColString(64);
$_ENV['schema']['users']->columns['address2'] = new ColString(64);
$_ENV['schema']['users']->columns['telephone'] = new ColString(32);
$_ENV['schema']['users']->columns['primary_email'] = new ColString(64, false);
$_ENV['schema']['users']->columns['sms'] = new ColString(64);
$_ENV['schema']['users']->columns['admin'] = new ColBoolean(false, 'f');
$_ENV['schema']['users']->columns['theme'] = new ColString(32);
$_ENV['schema']['users']->columns['active'] = new ColBoolean(false, 't');
$_ENV['schema']['users']->indices[] = new UniqueIndex("idx_users_username", array("username"));
$_ENV['schema']['users']->indices[] = new UniqueIndex("idx_users_primary_email", array("primary_email"));
/* }}} */

/* {{{ Table alternate_emails */
$_ENV['schema']['alternate_emails'] = new Table("alternate_emails");
$_ENV['schema']['alternate_emails']->columns['userid'] = new ColInteger(true);
$_ENV['schema']['alternate_emails']->columns['email'] = new ColString(64, true);
$_ENV['schema']['alternate_emails']->indices[] = new UniqueIndex("idx_alternate_emails_email", array("email"));
/* }}} */

/* {{{ Table sessions */
$_ENV['schema']['sessions'] = new Table("sessions");
$_ENV['schema']['sessions']->columns['session_id'] = new ColString(32, true);
$_ENV['schema']['sessions']->columns['session_data'] = new ColText();
$_ENV['schema']['sessions']->columns['session_expires'] = new ColInteger(true);
/* }}} */

/* {{{ Table group_categories */
$_ENV['schema']['group_categories'] = new Table("group_categories");
$_ENV['schema']['group_categories']->columns['gid'] = new ColInteger(true);
$_ENV['schema']['group_categories']->columns['cid'] = new ColInteger(true);
$_ENV['schema']['group_categories']->indices[] = new SortIndex("idx_group_categories_gid", array("gid"));
/* }}} */

/* {{{ Table group_permissions */
$_ENV['schema']['group_permissions'] = new Table("group_permissions");
$_ENV['schema']['group_permissions']->columns['permid'] = new ColInteger();
$_ENV['schema']['group_permissions']->columns['gid'] = new ColInteger();
/* }}} */

/* {{{ Table group_statuses */
$_ENV['schema']['group_statuses'] = new Table("group_statuses");
$_ENV['schema']['group_statuses']->columns['gid'] = new ColInteger(true);
$_ENV['schema']['group_statuses']->columns['sid'] = new ColInteger(true);
$_ENV['schema']['group_statuses']->indices[] = new SortIndex("idx_group_statuses_gid", array("gid"));
/* }}} */

/* {{{ Table view_tracking */
$_ENV['schema']['view_tracking'] = new Table("view_tracking");
$_ENV['schema']['view_tracking']->columns['vid'] = new ColSerial(true);
$_ENV['schema']['view_tracking']->columns['issueid'] = new ColInteger(true);
$_ENV['schema']['view_tracking']->columns['userid'] = new ColInteger(true);
$_ENV['schema']['view_tracking']->columns['viewed'] = new ColInteger(true);
$_ENV['schema']['view_tracking']->indices[] = new SortIndex("idx_view_tracking_userid_issueid", array("userid", "issueid"));
/* }}} */

/* {{{ Table custom_field_types */
$_ENV['schema']['custom_field_types'] = new Table("custom_field_types");
$_ENV['schema']['custom_field_types']->columns['typeid'] = new ColSerial(true);
$_ENV['schema']['custom_field_types']->columns['type'] = new ColString(32, true);
$_ENV['schema']['custom_field_types']->columns['tpl_path'] = new ColString(255, true);
$_ENV['schema']['custom_field_types']->columns['name'] = new ColString(32, true);
/* }}} */

/* {{{ Table groups */
$_ENV['schema']['groups'] = new Table("groups");
$_ENV['schema']['groups']->columns['gid'] = new ColSerial(true);
$_ENV['schema']['groups']->columns['name'] = new ColString(255);
$_ENV['schema']['groups']->columns['address'] = new ColString(100);
$_ENV['schema']['groups']->columns['address2'] = new ColString(100);
$_ENV['schema']['groups']->columns['contact'] = new ColString(64);
$_ENV['schema']['groups']->columns['tech'] = new ColString(64);
$_ENV['schema']['groups']->columns['tao'] = new ColString(64);
$_ENV['schema']['groups']->columns['brm'] = new ColString(64);
$_ENV['schema']['groups']->columns['sales'] = new ColString(64);
$_ENV['schema']['groups']->columns['amount'] = new ColNumeric(12,2);
$_ENV['schema']['groups']->columns['bought'] = new ColInteger();
$_ENV['schema']['groups']->columns['used'] = new ColInteger();
$_ENV['schema']['groups']->columns['start_date'] = new ColString(16);
$_ENV['schema']['groups']->columns['end_date'] = new ColString(16);
$_ENV['schema']['groups']->columns['group_type'] = new ColString(12);
$_ENV['schema']['groups']->columns['active'] = new ColBoolean(false, 't');
$_ENV['schema']['groups']->columns['status_reports'] = new ColBoolean(false, 'f');
$_ENV['schema']['groups']->columns['notes'] = new ColText();
$_ENV['schema']['groups']->columns['email'] = new ColString(64);
$_ENV['schema']['groups']->columns['prop_user'] = new ColBoolean(false, 'f');
$_ENV['schema']['groups']->columns['prop_category'] = new ColBoolean(false, 'f');
$_ENV['schema']['groups']->columns['prop_product'] = new ColBoolean(false, 'f');
$_ENV['schema']['groups']->columns['prop_status'] = new ColBoolean(false, 'f');
$_ENV['schema']['groups']->columns['prop_issue'] = new ColBoolean(false, 'f');
$_ENV['schema']['groups']->columns['prop_announce'] = new ColBoolean(false, 'f');
$_ENV['schema']['groups']->columns['autoreply'] = new ColText();
$_ENV['schema']['groups']->columns['public'] = new ColBoolean(false, 'f');
$_ENV['schema']['groups']->columns['auto_reopen'] = new ColBoolean(false, 'f');
/* }}} */

/* {{{ Table configuration */
$_ENV['schema']['configuration'] = new Table("configuration");
$_ENV['schema']['configuration']->columns['variable'] = new ColString(32, true);
$_ENV['schema']['configuration']->columns['value'] = new ColText(true);
$_ENV['schema']['configuration']->columns['constant'] = new ColBoolean(false, 'f');
/* }}} */

/* {{{ Table group_issue_visibility */
$_ENV['schema']['group_issue_visibility'] = new Table("group_issue_visibility");
$_ENV['schema']['group_issue_visibility']->columns['owner_gid'] = new ColInteger(true);
$_ENV['schema']['group_issue_visibility']->columns['viewer_gid'] = new ColInteger(true);
$_ENV['schema']['group_issue_visibility']->columns['can_write'] = new ColBoolean(false, 'f');
$_ENV['schema']['group_issue_visibility']->indices[] = new PrimaryKey('group_issue_visibility_pkey', array('owner_gid', 'viewer_gid'));
/* }}} */

/* {{{ Table categories */
$_ENV['schema']['categories'] = new Table("categories");
$_ENV['schema']['categories']->columns['cid'] = new ColSerial(true);
$_ENV['schema']['categories']->columns['category'] = new ColString(64, true);
/* }}} */

/* {{{ Table permissions */
$_ENV['schema']['permissions'] = new Table("permissions");
$_ENV['schema']['permissions']->columns['permid'] = new ColSerial(true);
$_ENV['schema']['permissions']->columns['permission'] = new ColString(32, true);
$_ENV['schema']['permissions']->columns['group_perm'] = new ColBoolean(false, 'f');
$_ENV['schema']['permissions']->columns['user_perm'] = new ColBoolean(false, 'f');
$_ENV['schema']['permissions']->columns['permission_type'] = new ColSmallInteger(true, 1);
$_ENV['schema']['permissions']->columns['system'] = new ColBoolean(false, 'f');
/* }}} */

/* {{{ Table assign_groups */
$_ENV['schema']['assign_groups'] = new Table("assign_groups");
$_ENV['schema']['assign_groups']->columns['assignor_gid'] = new ColInteger(true);
$_ENV['schema']['assign_groups']->columns['assignee_gid'] = new ColInteger(true);
$_ENV['schema']['assign_groups']->indices[] = new PrimaryKey('assign_groups_pkey', array('assignor_gid', 'assignee_gid'));
/* }}} */

/* {{{ Table notification_prefs */
$_ENV['schema']['notification_prefs'] = new Table("notification_prefs");
$_ENV['schema']['notification_prefs']->columns['userid'] = new ColInteger(true);
$_ENV['schema']['notification_prefs']->columns['channelid'] = new ColInteger(true);
$_ENV['schema']['notification_prefs']->columns['reason'] = new ColInteger(true);
$_ENV['schema']['notification_prefs']->indices[] = new PrimaryKey('notification_prefs_pkey', array('userid', 'channelid', 'reason'));
/* }}} */

/* {{{ Table reports */
$_ENV['schema']['reports'] = new Table("reports");
$_ENV['schema']['reports']->columns['rid'] = new ColSerial(true);
$_ENV['schema']['reports']->columns['userid'] = new ColInteger(true);
$_ENV['schema']['reports']->columns['name'] = new ColText(true);
$_ENV['schema']['reports']->columns['options'] = new ColText(true);
$_ENV['schema']['reports']->columns['report'] = new ColText();
$_ENV['schema']['reports']->columns['savedate'] = new ColInteger();
/* }}} */

/* {{{ Table products */
$_ENV['schema']['products'] = new Table("products");
$_ENV['schema']['products']->columns['pid'] = new ColSerial(true);
$_ENV['schema']['products']->columns['product'] = new ColString(128, true);
/* }}} */

/* {{{ Table issue_subscribers */
$_ENV['schema']['issue_subscribers'] = new Table("issue_subscribers");
$_ENV['schema']['issue_subscribers']->columns['issueid'] = new ColInteger(true);
$_ENV['schema']['issue_subscribers']->columns['userid'] = new ColInteger(true);
$_ENV['schema']['issue_subscribers']->columns['is_tech'] = new ColBoolean(true, 'f');
$_ENV['schema']['issue_subscribers']->indices[] = new PrimaryKey("issue_subscribers_pkey", array("issueid", "userid"));
/* }}} */

/* {{{ Table escalation_points */
$_ENV['schema']['escalation_points'] = new Table("escalation_points");
$_ENV['schema']['escalation_points']->columns['gid'] = new ColInteger(true);
$_ENV['schema']['escalation_points']->columns['egid'] = new ColInteger(true);
$_ENV['schema']['escalation_points']->indices[] = new SortIndex("idx_escalation_points_gid", array("gid"));
$_ENV['schema']['escalation_points']->indices[] = new SortIndex("idx_escalation_points_egid", array("egid"));
/* }}} */

/* {{{ Table group_users */
$_ENV['schema']['group_users'] = new Table("group_users");
$_ENV['schema']['group_users']->columns['gid'] = new ColInteger(true);
$_ENV['schema']['group_users']->columns['userid'] = new ColInteger(true);
$_ENV['schema']['group_users']->columns['perm_set'] = new ColInteger();
$_ENV['schema']['group_users']->columns['show_group'] = new ColBoolean(false, 't');
$_ENV['schema']['group_users']->columns['subscribed'] = new ColBoolean(true, 'f');
$_ENV['schema']['group_users']->indices[] = new SortIndex("idx_group_users_gid", array("gid"));
$_ENV['schema']['group_users']->indices[] = new SortIndex("idx_group_users_userid", array("userid"));
$_ENV['schema']['group_users']->indices[] = new SortIndex("idx_group_users_gid_userid", array("gid", "userid"));
/* }}} */

/* {{{ Table schema_version */
$_ENV['schema']['schema_version'] = new Table("schema_version");
$_ENV['schema']['schema_version']->columns['schema_version'] = new ColInteger(true);
/* }}} */

/* {{{ Table email_verification */
$_ENV['schema']['email_verification'] = new Table("email_verification");
$_ENV['schema']['email_verification']->columns['verification_key'] = new ColString(32, true);
$_ENV['schema']['email_verification']->columns['action'] = new ColString(32, true);
$_ENV['schema']['email_verification']->columns['arguments'] = new ColText(true);
$_ENV['schema']['email_verification']->indices[] = new PrimaryKey('email_verification_pkey', array('verification_key'));
/* }}} */

