<?php
/* {{{ Array $named_schema_versions */
$named_schema_versions = array(
  1 => 'Issue Tracker 4.0.x',
  2 => 'EnterTrack 1.0.1',
  3 => 'EnterTrack 1.1.0',
  11 => 'EnterTrack 1.2.0',/* EnterTrack 1.2.1 is also at schema version 11 */
  13 => 'Entertrack 1.2.2RC1',
  14 => 'Entertrack 1.2.2RC2',
);

/* constants for use below */
define('SCHEMA_ITV4', 1);
define('SCHEMA_ET_1_0_1', 2);
define('SCHEMA_ET_1_1_0', 3);
/* }}} */

/* {{{ Function get_schema_version */
/**
 * Detect the currently installed schema version.
 *
 * @param object $link an active link to the database
 * @return integer the schema version, or null if it cannot be determined
 */
function get_schema_version($link)
{
  // first check for version table
  if (setup_db_column_exists($link, "schema_version", "schema_version")) {
    $data = setup_db_fetchall($link, "SELECT schema_version from schema_version");
    if ($data) {
      $tmp = $data[0]; $tmp = $tmp['schema_version'];
      echo "get $tmp";
      return $tmp+0;
    } else {
      return null;
    }
  }

  // There's no schema_version table, so try to differentiate et-1.1.0 from the rest.
  // et-1.1.0 has an 'autoreply' column in the 'groups' table
  if (setup_db_column_exists($link, "groups", "autoreply")) {
    return SCHEMA_ET_1_1_0;
  }

  // next, try to differentiate et-1.0.1 from it-4.0.x.
  // et-1.0.1 has a 'real_opened' column in the 'issues' table
  if (setup_db_column_exists($link, "issues", "real_opened")) {
    return SCHEMA_ET_1_0_1;
  }

  // now we know it's some version of IT.  All ITv4 releases have
  // the same schema, but ITv3 had a "tickets" table instead of
  // an "issues" table, so we can use that to differentiate.  We
  // don't support upgrades from ITv3, so we're really just detecting
  // ITv4 here.
  if (setup_db_column_exists($link, "issues", "summary")) {
    return SCHEMA_ITV4;
  }

  // otherwise, bail entirely
  return null;
}
/* }}} */

/* {{{ Function get_schema_version_name */
/**
 * Get a name for the given schema version
 *
 * @param integer $version
 * @return string the name
 */
function get_schema_version_name($version)
{
  global $named_schema_versions;
  // count down schema versions until we find a name.
  $plus_revisions = false;
  while ($version > 0) {
    if (isset($named_schema_versions[$version])) {
      return $named_schema_versions[$version] . ($plus_revisions?" plus changes":"");
    }

    $plus_revisions = true;
    $version--;
  }

  return "(unknown)";
}
/* }}} */

/* {{{ Function get_all_schema_version_names */
/**
 * Get all named schema versions
 *
 * @return array (schema version => name)
 */
function get_all_schema_version_names()
{
  global $named_schema_versions;
  return $named_schema_versions;
}
/* }}} */

/* {{{ Function set_schema_version */
/**
 * Set the current schema version in the database.  This function
 * assumes that the schema_version table already exists and contains
 * one row.
 *
 * @param object $link the database connection
 * @param integer $schema_version
 */
function set_schema_version($link, $schema_version)
{
  uplog_step("Updating schema version to $schema_version");

  $sql = "UPDATE schema_version set schema_version = " . ($schema_version+0);
  uplog_sql($sql);
  $proceed = setup_db_execute($link, $sql);
  if (!$proceed) { uplog_db_error($link); }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_schema */
/**
 * Upgrade the database schema from the given schema version
 * to the current schema version.
 *
 * As a side effect, this function outputs a "log" of its activity
 * to the user.
 *
 * @param object $link a link to the database to be upgraded
 * @param integer $schema_version the schema version in place
 * @return boolean, with false indicating failure
 */
function upgrade_schema($link, $schema_version)
{
  if (is_null($schema_version)) { return false; } // can't upgrade this!

  echo "<div class=\"setup-steps\">\n";
  $rv = upgrade_schema_core($link, $schema_version);
  echo "</div>\n";
  return $rv;
}
/* }}} */

/* {{{ Function upgrade_schema_core */
/* For internal use by upgrade_schema() */
function upgrade_schema_core($link, $schema_version)
{
  // the "core" schema-upgrading switch statement.  Each label in this statement
  // falls through to the next, so that a whole cascade of schema changes will take
  // place.
  switch ($schema_version) {
    case SCHEMA_ITV4:
      if (!upgrade_add_time_cooking_columns($link)) return false;
      if (!upgrade_add_expanded_report_columns($link)) return false;
      if (!upgrade_add_custom_field_tables($link)) return false;
    case SCHEMA_ET_1_0_1:
      if (!upgrade_add_group_email_columns($link)) return false;
    case SCHEMA_ET_1_1_0:
      if (!upgrade_add_schema_version_table($link)) return false;
      if (!upgrade_rename_gid_to_primary_gid($link)) return false;
      if (!upgrade_remove_xmlrpc_connections($link)) return false;
      if (!upgrade_remove_configuration_table($link)) return false;
      if (!upgrade_add_motd_table($link)) return false;
      if (!upgrade_fix_group_field_values($link)) return false;
      if (!upgrade_add_and_transfer_notification_prefs($link)) return false;
    case 4:
      if (!upgrade_add_style_prefs_to_db($link)) return false;
    case 5:
      if (!upgrade_fix_notification_prefs_gaffe($link)) return false;
    case 6:
      if (!upgrade_remove_group_users_severity_column($link)) return false;
      if (!upgrade_delete_parser_rejects($link)) return false;
    case 7:
      if (!upgrade_username_email_not_null($link)) return false;
      if (!upgrade_userid_not_null($link)) return false;
      if (!upgrade_convert_issue_requesters_table($link)) return false;
      if (!upgrade_add_owners_to_requesters($link)) return false;
      if (!upgrade_eliminate_parser_user($link)) return false;
    case 8:
      if (!upgrade_rename_email_to_primary_email($link)) return false;
      if (!upgrade_add_user_indices($link)) return false;
      if (!upgrade_add_and_populate_alternate_emails($link)) return false;
      if (!upgrade_add_email_verification_table($link)) return false;
    case 9:
      if (!upgrade_add_is_tech_to_issue_requesters($link)) return false;
      if (!upgrade_merge_subscriptions_into_issue_requesters($link)) return false;
      if (!upgrade_rename_issue_requesters_to_issue_subscribers($link)) return false;
      if (!upgrade_add_assignees_to_subscribers($link)) return false;
    case 10:
      if (!upgrade_add_group_subscribers($link)) return false;
      if (!upgrade_remove_groups_from_notification_prefs($link)) return false;
    case 11:
      if (!upgrade_move_group_subscribers_to_group_users($link)) return false;
    case 12:
      if (!upgrade_password_not_null($link)) return false;
      if (!upgrade_add_group_unnasigned_table($link)) return false; 
      if (!upgrade_add_unnasigned_permission($link)) return false;
      if (!upgrade_add_products_levels_table($link)) return false; 
    	case 13:
		if (!upgrade_alter_issue_group_table($link)) return false;
	case 14:
		if (!upgrade_add_gid_to_events($link)) return false;
    case CURRENT_SCHEMA_VERSION:
      set_schema_version($link, CURRENT_SCHEMA_VERSION);
      /* this is the current version; nothing to do */
  }

  return true;
}
/* }}} */

////
// Logging functions
/* {{{ */
function uplog_step($title) { step_header(true); echo "<div class=\"title\">$title</div>\n"; }
function uplog_end_step() { step_footer(); }
function uplog_sql($sql) { echo "<pre class=\"sql\">$sql</pre>\n"; }
function uplog_error($error) { echo "<div class=\"error\">$error</div>\n"; }
function uplog_warn($warning) { echo "<div class=\"warning\">$warning</div>\n"; }
function uplog_db_error($link) { uplog_error(setup_db_error($link)); }
/* }}} */

////
// Upgrade functions
/* {{{ Function upgrade_add_time_cooking_columns */
function upgrade_add_time_cooking_columns($link)
{
  uplog_step("Adding time-cooking columns to <tt>issues</tt>");
  $proceed = true;

  $new_cols = array(
    "real_opened" => new ColInteger(),
    "real_closed" => new ColInteger(),
    "modification_note" => new ColString(150),
    "adjusted_by" => new ColInteger(),
  );

  foreach ($new_cols as $col_name => $col) {
    $sqls = sqls_add_column($link, "issues", $col_name, $col);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }

    if (!$proceed) { break; }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_expanded_report_columns */
function upgrade_add_expanded_report_columns($link)
{
  uplog_step("Adding additional columns to <tt>reports</tt>");
  $proceed = true;

  // report column
  if ($proceed) {
    $col = new ColText(false, '');
    $sqls = sqls_add_column($link, "reports", "report", $col);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  // savedate column
  if ($proceed) {
    $col = new ColInteger(false, 0);
    $sqls = sqls_add_column($link, "reports", "savedate", $col);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_custom_field_tables */
function upgrade_add_custom_field_tables($link)
{
  uplog_step("Adding tables to support custom fields");
  $proceed = true;

  // lots of new tables
  $tables = array();

  $tables['custom_field_types'] = new Table("custom_field_types");
  $tables['custom_field_types']->columns['typeid'] = new ColSerial(true);
  $tables['custom_field_types']->columns['type'] = new ColString(32, true);
  $tables['custom_field_types']->columns['tpl_path'] = new ColString(255, true);
  $tables['custom_field_types']->columns['name'] = new ColString(32, true);

  $tables['group_fields'] = new Table("group_fields");
  $tables['group_fields']->columns['gfid'] = new ColSerial(true);
  $tables['group_fields']->columns['gid'] = new ColInteger(true);
  $tables['group_fields']->columns['typeid'] = new ColInteger(true);
  $tables['group_fields']->columns['name'] = new ColString(32, true);
  $tables['group_fields']->columns['label'] = new ColString(32);
  $tables['group_fields']->indices[] = new UniqueIndex('group_fields_gid_name_uniq', array('gid', 'name'));

  $tables['group_field_values'] = new Table("group_field_values");
  $tables['group_field_values']->columns['gfid'] = new ColInteger(true);
  $tables['group_field_values']->columns['value_idx'] = new ColInteger(true);
  $tables['group_field_values']->columns['value'] = new ColText();
  $tables['group_field_values']->columns['is_default'] = new ColBadBoolean(true, '0');
  $tables['group_field_values']->indices[] = new PrimaryKey('group_field_values_pkey', array('gfid', 'value_idx'));

  $tables['issue_group_fields'] = new Table("issue_group_fields");
  $tables['issue_group_fields']->columns['issueid'] = new ColInteger(true);
  $tables['issue_group_fields']->columns['gfid'] = new ColInteger(true);
  $tables['issue_group_fields']->columns['value_idx'] = new ColInteger();
  $tables['issue_group_fields']->columns['value'] = new ColText();
  $tables['issue_group_fields']->indices[] = new PrimaryKey('issue_group_fields_pkey', array('issueid', 'gfid'));

  $tables['assign_groups'] = new Table("assign_groups");
  $tables['assign_groups']->columns['assignor_gid'] = new ColInteger(true);
  $tables['assign_groups']->columns['assignee_gid'] = new ColInteger(true);
  $tables['assign_groups']->indices[] = new PrimaryKey('assign_groups_pkey', array('assignor_gid', 'assignee_gid'));

  $tables['group_issue_visibility'] = new Table("group_issue_visibility");
  $tables['group_issue_visibility']->columns['owner_gid'] = new ColInteger(true);
  $tables['group_issue_visibility']->columns['viewer_gid'] = new ColInteger(true);
  $tables['group_issue_visibility']->columns['can_write'] = new ColBoolean(false, 'f');
  $tables['group_issue_visibility']->indices[] = new PrimaryKey('group_issue_visibility_pkey', array('owner_gid', 'viewer_gid'));

  // execute all of that
  foreach ($tables as $name => $table) {
    $sqls = sql_create_table($link, $table);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
    if (!$proceed) {
      break;
    }
  }

  // now add some new data to one of those tables
  if ($proceed) {
    $new_data = array(
      "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (1, 'checkbox', 'default/tpl/fields/checkbox.tpl', 'checkbox')",
      "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (2, 'date', 'default/tpl/fields/date.tpl', 'date')",
      "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (3, 'select', 'default/tpl/fields/select.tpl', 'select')",
      "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (4, 'select multiple', 'default/tpl/fields/select_multiple.tpl', 'select multiple')",
      "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (5, 'text', 'default/tpl/fields/text.tpl', 'textfield')",
      "INSERT INTO custom_field_types (typeid, type, tpl_path, name) VALUES (6, 'textarea', 'default/tpl/fields/textarea.tpl', 'textbox')",
    );

    foreach ($new_data as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_group_email_columns */
function upgrade_add_group_email_columns($link)
{
  uplog_step("Adding email-related columns to <tt>groups</tt>");
  $proceed = true;

  // autoreply column
  if ($proceed) {
    $col = new ColText(false, '');
    $sqls = sqls_add_column($link, "groups", "autoreply", $col);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    $sql = "UPDATE groups set autoreply=notes";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) { uplog_db_error($link); }
  }

  // public column
  if ($proceed) {
    $col = new ColBoolean(false, 'f');
    $sqls = sqls_add_column($link, "groups", "public", $col);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    $sql = "UPDATE groups set public='t' where email != ''";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) { uplog_db_error($link); }
  }

  // auto_reopen column
  if ($proceed) {
    $col = new ColBoolean(false, 'f');
    $sqls = sqls_add_column($link, "groups", "auto_reopen", $col);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_schema_version_table */
function upgrade_add_schema_version_table($link)
{
  uplog_step("Adding table <tt>schema_version</tt>");
  $proceed = true;

  if ($proceed) {
    $table = new Table("schema_version");
    $table->columns['schema_version'] = new ColInteger(true);
    $sqls = sql_create_table($link, $table);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    $sql = "INSERT into schema_version (schema_version) values (4)";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_rename_gid_to_primary_gid  */
function upgrade_rename_gid_to_primary_gid($link)
{
  uplog_step("Renaming <tt>issues.gid</tt> to <tt>issues.primary_gid</tt>");

  $sql = sql_rename_column($link, "issues", "gid", "primary_gid", new ColInteger(true, 0));
  uplog_sql($sql);
  $proceed = setup_db_execute($link, $sql);
  if (!$proceed) { uplog_db_error($link); }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_remove_xmlrpc_connections */
function upgrade_remove_xmlrpc_connections($link)
{
  uplog_step("Removing table <tt>xmlrpc_connections</tt>");

  $sql = sql_drop_table($link, "xmlrpc_connections");
  uplog_sql($sql);
  $proceed = setup_db_execute($link, $sql);
  if (!$proceed) { uplog_db_error($link); }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_remove_configuration_table */
function upgrade_remove_configuration_table($link)
{
  uplog_step("Removing table <tt>configuration</tt>");

  $sql = sql_drop_table($link, "configuration");
  uplog_sql($sql);
  $proceed = setup_db_execute($link, $sql);
  if (!$proceed) { uplog_db_error($link); }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_motd_table */
function upgrade_add_motd_table($link)
{
  uplog_step("Adding table <tt>motd</tt>");

  $table = new Table("motd");
  $table->columns['motd'] = new ColText();
  $sqls = sql_create_table($link, $table);
  foreach ($sqls as $sql) {
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
      break;
    }
  }

  if ($proceed) {
    // try to read the motd file
    $filename = _PATH_."/includes/motd";
    if (file_exists($filename)) {
      $fd = @fopen($filename, "r");
      if ($fd) {
        $motd = @fread($fd, filesize($filename));
        @fclose($filename);

        $sql = "INSERT into motd (motd) values ('" . setup_db_escape($motd) . "')";
        uplog_sql($sql);
        $proceed = setup_db_execute($link, $sql);
      }
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_fix_group_field_values */
function upgrade_fix_group_field_values($link)
{
  uplog_step("Upgrading column type of <tt>group_field_values.is_default</tt>");
  $colspec = new ColBoolean(true, 'f');

  // we have to do a little switcheroo here -- older versions of Postgres can't change
  // the type of a column in situ
  $proceed = true;
  if ($proceed) {
    $sqls = sqls_add_column($link, 'group_field_values', 'tempfield', $colspec);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  $proceed = true;
  if ($proceed) {
    $sql = "UPDATE group_field_values set tempfield = 'f' where not is_default";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) { uplog_db_error($link); }
  }

  $proceed = true;
  if ($proceed) {
    $sql = "UPDATE group_field_values set tempfield = 't' where is_default";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) { uplog_db_error($link); }
  }

  $proceed = true;
  if ($proceed) {
    $sql = sql_drop_column($link, 'group_field_values', 'is_default');
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) { uplog_db_error($link); }
  }

  $proceed = true;
  if ($proceed) {
    $sql = sql_rename_column($link, 'group_field_values', 'tempfield', 'is_default', $colspec);
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) { uplog_db_error($link); }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_and_transfer_notification_prefs */
function upgrade_add_and_transfer_notification_prefs($link)
{
  uplog_step("Creating and populating new <tt>notifications</tt> table");
  $proceed = true;

  if ($proceed) {
    $sql = "SELECT gid, userid, type from notifications";
    uplog_sql($sql);
    $notifications = setup_db_fetchall($link, $sql);
    if (!is_array($notifications)) {
      uplog_db_error($link);
      $proceed = false;
    }
  }

  if ($proceed) {
    $sql = sql_drop_table($link, "notifications");
    uplog_sql($sql);
    if (!setup_db_execute($link, $sql)) {
      uplog_db_error($link);
      $proceed = false;
    }
  }

  if ($proceed) {
    $table = new Table("notifications");
    $table->columns['userid'] = new ColInteger(true);
    $table->columns['channelid'] = new ColInteger(true);
    $table->columns['chtypeid'] = new ColInteger(true);
    $table->columns['gid'] = new ColInteger(true);
    $table->indices[] = new PrimaryKey('notification_prefs_pkey', array('userid', 'channelid', 'chtypeid', 'gid'));

    $sqls = sql_create_table($link, $table);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    foreach ($notifications as $notif) {
      $userid = $notif['userid'];
      $gid = $notif['gid'];
      $type = $notif['type'];

      if ($type == 'E') {
        $sql_base = "INSERT into notifications (userid, channelid, chtypeid, gid) values ($userid, 101, %s, $gid)";
      } else {
        $sql_base = "INSERT into notifications (userid, channelid, chtypeid, gid) values ($userid, 102, %s, $gid)";
        continue;
      }

      foreach (array(101, 102, 103) as $chtypeid) {
        $sql = sprintf($sql_base, $chtypeid);
        uplog_sql($sql);
        $proceed = setup_db_execute($link, $sql);
        if (!$proceed) {
          uplog_db_error($link);
          break;
        }
      }

      if (!$proceed) {
        break;
      }
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_style_prefs_to_db */
function upgrade_add_style_prefs_to_db($link)
{
  uplog_step("Transferring style settings from <tt>themes/*.theme.php</tt> files to the <tt>user_prefs</tt> table");
  $proceed = true;

  /* loop over the files */
  foreach (glob(_PATH_."/themes/*.theme.php") as $pathname) {
    $filename = basename($pathname);

    /* extract the userid */
    $matches = array();
    if (preg_match("/([0-9]*)\\.theme\\.php/", $filename, $matches)) {
      $userid = $matches[1]+0;

      /* now get the prefs -- this opens the file and grabs 'em into
       * an array */
      $prefs = _upgrade_add_style_prefs_to_db_load_prefs($pathname);
      foreach ($prefs as $preference => $value) {
        $preference = setup_db_escape($link, $preference);
        $value = setup_db_escape($link, $value);
        $sql = "INSERT into user_prefs (userid, preference, value) values ($userid, '$preference', '$value')";
        uplog_sql($sql);
        $proceed = setup_db_execute($link, $sql);
        if (!$proceed) {
          uplog_db_error($link);
          break 2;
        }
      }

      // and delete it, if possible
      @unlink($pathname);
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
function _upgrade_add_style_prefs_to_db_load_prefs($pathname)
{
  @include($pathname);

  $rv = array();
  if (isset($font_size)) { $rv['font_size'] = $font_size; }
  if (isset($font_unit)) { $rv['font_unit'] = $font_unit; }
  if (isset($font_family)) { $rv['font_family'] = $font_family; }
  if (isset($border_color)) { $rv['border_color'] = $border_color; }
  if (isset($link_hover)) { $rv['link_hover'] = $link_hover; }
  if (isset($underline)) { $rv['underline'] = $underline; }
  if (isset($fgcolor)) { $rv['fgcolor'] = $fgcolor; }
  if (isset($bgcolor)) { $rv['bgcolor'] = $bgcolor; }
  if (isset($table_head_fgcolor)) { $rv['table_head_fgcolor'] = $table_head_fgcolor; }
  if (isset($table_head_bgcolor)) { $rv['table_head_bgcolor'] = $table_head_bgcolor; }
  if (isset($titlebar_fgcolor)) { $rv['titlebar_fgcolor'] = $titlebar_fgcolor; }
  if (isset($titlebar_bgcolor)) { $rv['titlebar_bgcolor'] = $titlebar_bgcolor; }
  if (isset($subtitle_fgcolor)) { $rv['subtitle_fgcolor'] = $subtitle_fgcolor; }
  if (isset($subtitle_bgcolor)) { $rv['subtitle_bgcolor'] = $subtitle_bgcolor; }
  if (isset($row1_fgcolor)) { $rv['row1_fgcolor'] = $row1_fgcolor; }
  if (isset($row1_bgcolor)) { $rv['row1_bgcolor'] = $row1_bgcolor; }
  if (isset($row2_fgcolor)) { $rv['row2_fgcolor'] = $row2_fgcolor; }
  if (isset($row2_bgcolor)) { $rv['row2_bgcolor'] = $row2_bgcolor; }
  if (isset($label_fgcolor)) { $rv['label_fgcolor'] = $label_fgcolor; }
  if (isset($label_bgcolor)) { $rv['label_bgcolor'] = $label_bgcolor; }
  if (isset($data_fgcolor)) { $rv['data_fgcolor'] = $data_fgcolor; }
  if (isset($data_bgcolor)) { $rv['data_bgcolor'] = $data_bgcolor; }
  if (isset($menu_fgcolor)) { $rv['menu_fgcolor'] = $menu_fgcolor; }
  if (isset($menu_bgcolor)) { $rv['menu_bgcolor'] = $menu_bgcolor; }
  if (isset($submenu_fgcolor)) { $rv['submenu_fgcolor'] = $submenu_fgcolor; }
  if (isset($submenu_bgcolor)) { $rv['submenu_bgcolor'] = $submenu_bgcolor; }
  if (isset($error_fgcolor)) { $rv['error_fgcolor'] = $error_fgcolor; }
  if (isset($error_bgcolor)) { $rv['error_bgcolor'] = $error_bgcolor; }

  return $rv;
}

/* }}} */

/* {{{ Function upgrade_fix_notification_prefs_gaffe */
function upgrade_fix_notification_prefs_gaffe($link)
{
  uplog_step("Oops, that should be the <tt>notification_prefs</tt> table");
  $proceed = true;

  /* Here's the deal: if we upgraded to schema version 4, then we
   * have a 'notifications' table and no 'notification_prefs'
   * table; if we installed schema version 4 from scratch (which can
   * only happen from CVS), then we have both tables, but only the
   * 'notification_prefs' table is the one we want.  */

  $sql = "select * from notifications limit 1";
  uplog_sql($sql);
  $notifications_exists = setup_db_execute($link, $sql);

  $sql = "select * from notification_prefs limit 1";
  uplog_sql($sql);
  $notification_prefs_exists = setup_db_execute($link, $sql);

  if ($notifications_exists && $notification_prefs_exists) {
    $sql = sql_drop_table($link, "notifications");
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql); // ignore the error
    if (!$proceed) {
      uplog_db_error($link);
    }
  } else if ($notifications_exists && !$notification_prefs_exists) {
    $sql = sql_rename_table($link, "notifications", "notification_prefs");
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  } else {
    uplog_error("This situation shouldn't be able to occur! " .
                "<tt>notifications</tt> " . ($notifications_exists?"exists":"doesn't exist") .
                " and <tt>notification_prefs</tt> " . ($notification_prefs_exists?"exists":"doesn't exist"));
    $proceed = false;
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_remove_group_users_severity_column */
function upgrade_remove_group_users_severity_column($link)
{
  uplog_step("Removing the vestigial <tt>severity</tt> column from table <tt>group_users</tt>");

  $sql = sql_drop_column($link, "group_users", "severity");
  uplog_sql($sql);
  $proceed = setup_db_execute($link, $sql);
  if (!$proceed) {
    uplog_db_error($link);
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_delete_parser_rejects */
function upgrade_delete_parser_rejects($link)
{
  uplog_step("Deleting the group named \"Parser Rejects\".");

  $proceed = true;

  if ($proceed) {
    $sql = "delete from groups where name = 'Parser Rejects'";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_username_email_not_null */
function upgrade_username_email_not_null($link)
{
  uplog_step("Marking <tt>password</tt>, <tt>username</tt> and <tt>email</tt> as NULL OK in table <i>users</i>");

  $proceed = true;

  if ($proceed) {
    $sql = sql_set_not_null($link, 'users', 'password', new ColString(32, false));
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    $sql = sql_set_not_null($link, 'users', 'username', new ColString(32, false));
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    $sql = sql_set_not_null($link, 'users', 'email', new ColString(64, false));
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_userid_not_null */
function upgrade_userid_not_null($link)
{
  uplog_step("Marking <tt>userid</tt> as NOT NULL in table <i>users</i>");

  $proceed = true;

  if ($proceed) {
    $sql = sql_set_not_null($link, 'users', 'userid', new ColSerial(true));
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      // silently accept a failure here; I've seen this happen both ways
      // --dustin
      $proceed = true;
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_convert_issue_requesters_table */
function upgrade_convert_issue_requesters_table($link)
{
  uplog_step("Converting <tt>issue_requesters</tt> table to reference userids instead of email addresses");
  $proceed = true;

  // first gather the info in the existing issue_requesters table, with userids where
  // they exist
  if ($proceed) {
    $sql = "SELECT distinct requester, userid
          from issue_requesters left join users on requester = email";
    uplog_sql($sql);
    $requester_rows = setup_db_fetchall($link, $sql);
    if (!is_array($requester_rows)) {
      uplog_db_error($link);
      $proceed = false;
    } else {
      // put that into an associative array
      $requesters = array();
      foreach ($requester_rows as $row) {
        $rq = $row['requester'];
        $userid = $row['userid'];
        $requesters[$rq] = $userid;
      }
      unset($requester_rows);
    }
  }

  // now add users rows for any non-existent email addresses
  if ($proceed) {
    foreach ($requesters as $requester => $userid) {
      if (!$userid) {
        $sql = "INSERT into users (email) values ('" . setup_db_escape($link, $requester) . "')";
        uplog_sql($sql);
        $proceed = setup_db_execute($link, $sql);
        if (!$proceed) {
          uplog_db_error($link);
          break;
        } else {
          $requesters[$requester] = setup_db_last_id($link, "users_userid_seq");
        }
      }
    }
  }

  // drop the primary key on the table
  if ($proceed) {
    $sql = sql_drop_primary_key($link, 'issue_requesters', 'issue_requesters_pkey');
    uplog_sql($sql);
    setup_db_execute($link, $sql); /* proceed regardless of error (sometimes pkey doesn't exist) */
  }

  // add the new column to the table
  if ($proceed) {
    $col = &new ColInteger();
    $sqls = sqls_add_column($link, 'issue_requesters', 'userid', $col);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      if (!setup_db_execute($link, $sql)) {
        uplog_db_error($link);
        $proceed = false;
        break;
      }
    }
  }

  // issue a huge number of updates to get the right values in the new column
  if ($proceed) {
    foreach ($requesters as $requester => $userid) {
      $sql = "UPDATE issue_requesters set userid=$userid
              where requester='" . setup_db_escape($link, $requester) . "'";
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  // drop the now-unneeded columns from the table
  if ($proceed) {
    $sql = sql_drop_column($link, "issue_requesters", "requester");
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    $sql = sql_drop_column($link, "issue_requesters", "list");
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  // and add a new primary key, spanning userid and issueid
  if ($proceed) {
    $sql = sql_add_index($link, 'issue_requesters', new PrimaryKey('issue_requesters_pkey', array('userid', 'issueid')));
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_owners_to_requesters */
function upgrade_add_owners_to_requesters($link)
{
  uplog_step("Adding issue owners to <tt>issue_requesters</tt> table");
  $proceed = true;
  $non_fatal = false;

  // get the parser's user id
  if ($proceed) {
    $sql = "select userid from users where first_name = 'Email' and last_name = 'Parser'";
    uplog_sql($sql);
    $res = setup_db_fetchall($link, $sql);
    if (!is_array($res)) {
      uplog_db_error($link);
      $proceed = false;
    } else {
      if (!sizeof($res)) {
        uplog_warn("'Email Parser' was not found; EnterTrack should still function, although older tickets may have incorrect parameters.");
        $proceed = false;
        $non_fatal = true;
        $client_userid = -1; // dummy value
      } else {
        $client_userid = $res[0]['userid'];
      }
    }
  }

  // get all issues not opened by that user
  if ($proceed) {
    $sql = "select issueid, opened_by
       from issues ";
    if ($client_userid != -1) {
      $sql .= "where issues.opened_by != $client_userid";
    }
    uplog_sql($sql);
    $rows = setup_db_fetchall($link, $sql);
    if (!is_array($rows)) {
      uplog_db_error($link);
      $proceed = false;
    }
  }

  // process that data
  if ($proceed) {
    foreach ($rows as $row) {
      // a requester was found; update the opened_by in this issue
      $sql = "INSERT into issue_requesters (issueid, userid) values ({$row['issueid']}, {$row['opened_by']})";
      uplog_sql($sql);
      setup_db_execute($link, $sql);
      // ignore errors here, as they are probably from a duplicate key, which we want to ignore.
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed || $non_fatal;
}
/* }}} */

/* {{{ Function upgrade_eliminate_parser_user */
function upgrade_eliminate_parser_user($link)
{
  uplog_step("Removing user 'Email Parser' and changing issues to indicate the actual owner.");
  $proceed = true;
  $non_fatal = false;

  // get the parser's user id
  if ($proceed) {
    $sql = "select userid from users where first_name = 'Email' and last_name = 'Parser'";
    uplog_sql($sql);
    $res = setup_db_fetchall($link, $sql);
    if (!is_array($res)) {
      uplog_db_error($link);
      $proceed = false;
    } else {
      if (!sizeof($res)) {
        uplog_warn("'Email Parser' was not found; EnterTrack should still function, although older tickets may have incorrect parameters.");
        $proceed = false;
        $non_fatal = true;
      } else {
        $client_userid = $res[0]['userid'];
      }
    }
  }

  // get all issues opened by that user, along with any requesters
  if ($proceed) {
    $sql = "select issues.issueid, max(userid) as requester_userid
       from issues left join issue_requesters using(issueid)
       where issues.opened_by = $client_userid
       group by issues.issueid";
    uplog_sql($sql);
    $rows = setup_db_fetchall($link, $sql);
    if (!is_array($rows)) {
      uplog_db_error($link);
      $proceed = false;
    }
  }

  // process that data
  if ($proceed) {
    $found_null = false;
    foreach ($rows as $row) {
      if (is_null($row['requester_userid'])) {
        // no requesters existed for this issue
        $found_null = true;
      } else {
        // a requester was found; update the opened_by in this issue
        $sql = "UPDATE issues set opened_by = {$row['requester_userid']} where issueid={$row['issueid']}";
        uplog_sql($sql);
        $proceed = setup_db_execute($link, $sql);
        if (!$proceed) {
          uplog_db_error($link);
          break;
        }
      }
    }
  }

  // if we couldn't update every issue listing the parser in
  // opened_by, then bail.
  if ($proceed && $found_null) {
      uplog_warn("For at least one issue, no other user but 'Email Parser' was found; that user will not be deleted.");
      $proceed = false;
      $non_fatal = true;
  }

  // otherwrise, we are completely done with that user!
  if ($proceed) {
    $sql = "DELETE from users where userid = $client_userid";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  uplog_end_step();
  return $proceed || $non_fatal;
}
/* }}} */

/* {{{ Function upgrade_rename_email_to_primary_email */
function upgrade_rename_email_to_primary_email($link)
{
  uplog_step("Renaming <tt>email</tt> to <tt>primary_email</tt> in table <tt>users</tt>");

  $sql = sql_rename_column($link, "users", "email", "primary_email", new ColString(64, true));
  uplog_sql($sql);
  $proceed = setup_db_execute($link, $sql);
  if (!$proceed) { uplog_db_error($link); }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_user_indices */
function upgrade_add_user_indices($link)
{
  uplog_step("Adding indices for <tt>username</tt> and <tt>primary_email</tt> to table <i>users</i>");

  $username_idx = new UniqueIndex("idx_users_username", array("username"));
  $email_idx = new UniqueIndex("idx_users_primary_email", array("primary_email"));
  $proceed = true;

  if ($proceed) {
    $sql = $username_idx->sql("users");
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    $sql = $email_idx->sql("users");
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_and_populate_alternate_emails */
function upgrade_add_and_populate_alternate_emails($link)
{
  uplog_step("Adding <tt>alternate_emails</tt> table and populating it with users' email addresses");
  $proceed = true;
  $non_fatal = false;

  //  create the table
  if ($proceed) {
    $tbl = new Table("alternate_emails");
    $tbl->columns['userid'] = new ColInteger(true);
    $tbl->columns['email'] = new ColString(64, true);
    $tbl->indices[] = new UniqueIndex("idx_alternate_emails_email", array("email"));

    $sqls = sql_create_table($link, $tbl);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  // get all users' email addresses
  if ($proceed) {
    $sql = "select userid, primary_email from users";
    uplog_sql($sql);
    $rows = setup_db_fetchall($link, $sql);
    if (!is_array($rows)) {
      uplog_db_error($link);
      $proceed = false;
    }
  }

  // process that data
  if ($proceed) {
    foreach ($rows as $row) {
      // a requester was found; update the opened_by in this issue
      $sql = "INSERT into alternate_emails (userid, email) values ({$row['userid']}, '" . setup_db_escape($link, $row['primary_email']) . "')";
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed || $non_fatal;
}
/* }}} */

/* {{{ Function upgrade_add_email_verification_table */
function upgrade_add_email_verification_table($link)
{
  uplog_step("Adding table <tt>email_verification</tt>");
  $proceed = true;

  if ($proceed) {
    $table = new Table("email_verification");
    $table->columns['verification_key'] = new ColString(32, true);
    $table->columns['action'] = new ColString(32, true);
    $table->columns['arguments'] = new ColText(true);
    $table->indices[] = new PrimaryKey('email_verification_pkey', array('verification_key'));

    $sqls = sql_create_table($link, $table);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_is_tech_to_issue_requesters */
function upgrade_add_is_tech_to_issue_requesters($link)
{
  uplog_step("Adding <tt>is_tech</tt> field to table <tt>issue_requesters</tt>");
  $proceed = true;

  // autoreply column
  if ($proceed) {
    $col = new ColBoolean(false, 'f');
    $sqls = sqls_add_column($link, "issue_requesters", "is_tech", $col);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_merge_subscriptions_into_issue_requesters */
function upgrade_merge_subscriptions_into_issue_requesters($link)
{
  uplog_step("Merging issue subscribers into <tt>issue_requesters</tt> table as technicians");
  $proceed = true;

  // We expect to have a large number of issue_requesters, but a relatively
  // small number of subscribers.  Thus we iterate over the subscribers,
  // potentially executing a few queries for each one.

  // get all subscriptions
  if ($proceed) {
    $sql = "select userid, issueid from subscriptions";
    uplog_sql($sql);
    $subscriptions = setup_db_fetchall($link, $sql);
    if (!is_array($subscriptions)) {
      uplog_db_error($link);
      $proceed = false;
    }
  }

  // insert and, failing that, update the issue_requesters table
  if ($proceed) {
    foreach ($subscriptions as $subscription) {
      $userid = $subscription['userid'];
      $issueid = $subscription['issueid'];

      $sql = "insert into issue_requesters (issueid, userid, is_tech) values ($issueid, $userid, 't')";
      uplog_sql($sql);
      if (!setup_db_execute($link, $sql)) {
        $sql = "update issue_requesters set is_tech = 't' where issueid=$issueid and userid=$userid";
        uplog_sql($sql);
        if (!setup_db_execute($link, $sql)) {
          $proceed = false;
          uplog_db_error($link);
          break;
        }
      }
    }
  }

  // finally, delete the subscriptions table
  if ($proceed) {
    $sql = sql_drop_table($link, 'subscriptions');
    uplog_sql($sql);
    if (!setup_db_execute($link, $sql)) {
      $proceed = false;
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_rename_issue_requesters_to_issue_subscribers */
function upgrade_rename_issue_requesters_to_issue_subscribers($link)
{
  uplog_step("Renaming <tt>issue_requesters</tt> to <tt>issue_subscribers</tt>");
  $proceed = true;

  if ($proceed) {
    $sql = sql_rename_table($link, 'issue_requesters', 'issue_subscribers');
    uplog_sql($sql);
    if (!setup_db_execute($link, $sql)) {
      $proceed = false;
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_assignees_to_subscribers */
function upgrade_add_assignees_to_subscribers($link)
{
  uplog_step("Adding issue assignees to subscribers");
  $proceed = true;

  // get all assignments
  if ($proceed) {
    $sql = "select assigned_to, issueid from issue_groups where assigned_to is not NULL";
    uplog_sql($sql);
    $assignments = setup_db_fetchall($link, $sql);
    if (!is_array($assignments)) {
      uplog_db_error($link);
      $proceed = false;
    }
  }

  // insert and, failing that, update the issue_subscribers table (yes, this
  // is quite slow, but faster and simpler than a complex query)
  if ($proceed) {
    foreach ($assignments as $assignment) {
      $userid = $assignment['assigned_to'];
      $issueid = $assignment['issueid'];

      $sql = "insert into issue_subscribers (issueid, userid, is_tech) values ($issueid, $userid, 't')";
      uplog_sql($sql);
      if (!setup_db_execute($link, $sql)) {
        $sql = "update issue_subscribers set is_tech = 't' where issueid=$issueid and userid=$userid";
        uplog_sql($sql);
        if (!setup_db_execute($link, $sql)) {
          $proceed = false;
          uplog_db_error($link);
          break;
        }
      }
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_group_subscribers */
function upgrade_add_group_subscribers($link)
{
  uplog_step("Adding table <tt>group_subscribers</tt>");
  $proceed = true;

  if ($proceed) {
    $table = new Table("group_subscribers");
    $table->columns['userid'] = new ColInteger(32, true);
    $table->columns['gid'] = new ColInteger(32, true);
    $table->indices[] = new PrimaryKey('group_subscribers_pkey', array('userid', 'gid'));

    $sqls = sql_create_table($link, $table);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    // If the user was notified at all for a group, consider them a subscriber
    $sql = "INSERT into group_subscribers SELECT distinct userid, gid from notification_prefs";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_group_unnasigned */
function upgrade_add_products_levels_table($link)
{
  uplog_step("Adding table <tt>products_levels</tt>");
  $proceed = true;

  if ($proceed) {
    $table = new Table("products_levels");
    $table->columns['pid'] = new ColInteger(32, true);
    $table->columns['level'] = new ColInteger(32, true);
    $table->columns['color'] = new ColString(32, true);
    $table->indices[] = new PrimaryKey('products_levels_pkey', array('pid', 'level'));

    $sqls = sql_create_table($link, $table);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

function upgrade_alter_issue_group_table($link)
{
	uplog_step("Altering field type for <tt>deescalated</tt>");
	uplog_end_step();
	return true;
}

/* {{{ Function upgrade_add_group_unnasigned */
function upgrade_add_group_unnasigned_table($link)
{
  uplog_step("Adding table <tt>group_unnasigned</tt>");
  $proceed = true;

  if ($proceed) {
    $table = new Table("group_unnasigned");
    $table->columns['gid'] = new ColInteger(32, true);
    $table->columns['show_group'] = new ColBoolean(false, 't');

    $sqls = sql_create_table($link, $table);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_add_unnasigned_permission */
function upgrade_add_unnasigned_permission($link)
{
  uplog_step("Adding assign permission <tt>permissions</tt>");
  $proceed = true;

  // Add assign_issues to default permissions
  if ($proceed) {
    $sql = "INSERT INTO permissions (permission,system) VALUES('assign_issues','t')";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  // Add assign_issues to Group Administrator permission_set
  if ($proceed) {
    $sql  = "UPDATE permission_sets set permissions = ";
    $sql .= "'create_issues,create_announcements,view_private,upload_files,update_group,technician,move_issues,edit_events,field_manager,cook_date,assign_issues' ";
    $sql .= "where name = 'Group Administrator'";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  // Add assign_issues to Super Client permission_set
  if ($proceed) {
    $sql  = "UPDATE permission_sets set permissions = ";
    $sql .= "'create_issues,create_announcements,view_private,upload_files,assign_issues' ";
    $sql .= "where name = 'Super Client'";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  // Add assign_issues to Technician permission_set
  if ($proceed) {
    $sql  = "UPDATE permission_sets set permissions = ";
    $sql .= "'create_issues,create_announcements,view_private,upload_files,technician,assign_issues' ";
    $sql .= "where name = 'Technician'";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */



/* {{{ Function upgrade_remove_groups_from_notification_prefs */
function upgrade_remove_groups_from_notification_prefs($link)
{
  uplog_step("Removing group and change-type information from <tt>notification_prefs</tt>");
  $proceed = true;

  if ($proceed) {
    $table = new Table("new_notification_prefs");
    $table->columns['userid'] = new ColInteger(32, true);
    $table->columns['channelid'] = new ColInteger(32, true);
    $table->columns['reason'] = new ColInteger(32, true);
    $table->indices[] = new PrimaryKey('new_notification_prefs_pkey', array('userid', 'channelid', 'reason'));

    $sqls = sql_create_table($link, $table);
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    // If the user was notified at all for a group, consider them a subscriber (REASON_SUB_GROUP = 5
    // in include/subsys/notif.php)
    $sql = "INSERT into new_notification_prefs SELECT distinct userid, channelid, 5 from notification_prefs";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    // All users automatically get emails when they're an issue client (101 = email, 1 = REASON_ISSUE_CLIENT)
    $sql = "INSERT into new_notification_prefs SELECT distinct userid, 101, 1 from users";
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    $sql = sql_drop_table($link, 'notification_prefs');
    uplog_sql($sql);
    if (!setup_db_execute($link, $sql)) {
      $proceed = false;
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    $sql = sql_rename_table($link, 'new_notification_prefs', 'notification_prefs');
    uplog_sql($sql);
    if (!setup_db_execute($link, $sql)) {
      $proceed = false;
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_move_group_subscribers_to_group_users */
function upgrade_move_group_subscribers_to_group_users($link)
{
  uplog_step("Moving data in table <tt>group_subscribers</tt> to <tt>group_users</tt>");
  $proceed = true;

  if ($proceed) {
    $sqls = sqls_add_column($link, "group_users", "subscribed", new ColBoolean(true, 'f'));
    foreach ($sqls as $sql) {
      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    $sql = "select userid, gid from group_subscribers";
    uplog_sql($sql);
    $subscribers = setup_db_fetchall($link, $sql);
    if (!is_array($subscribers)) {
      uplog_db_error($link);
      $proceed = false;
    }
  }

  if ($proceed) {
    foreach ($subscribers as $row) {
      $gid = $row['gid'];
      $userid = $row['userid'];

      $sql = "SELECT subscribed from group_users where userid=$userid and gid=$gid";
      uplog_sql($sql);
      $rows = setup_db_fetchall($link, $sql);
      if (!is_array($rows)) {
        uplog_db_error($link);
        $proceed = false;
        break;
      }

      // do an update or an INSERT, depending on the results of the SELECT
      if (sizeof($rows) > 0)
        $sql = "UPDATE group_users set subscribed='t' where userid=$userid and gid=$gid";
      else
        $sql = "INSERT into group_users (userid, gid, subscribed) values ($userid, $gid, 't')";

      uplog_sql($sql);
      $proceed = setup_db_execute($link, $sql);
      if (!$proceed) {
        uplog_db_error($link);
        break;
      }
    }
  }

  if ($proceed) {
    $sql = sql_drop_table($link, "group_subscribers");
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
      break;
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

/* {{{ Function upgrade_password_not_null */
function upgrade_password_not_null($link)
{
  uplog_step("Marking <tt>password</tt> as NULL OK in table <i>users</i> (again)");

  $proceed = true;

  if ($proceed) {
    $sql = sql_set_not_null($link, 'users', 'password', new ColString(32, false));
    uplog_sql($sql);
    $proceed = setup_db_execute($link, $sql);
    if (!$proceed) {
      uplog_db_error($link);
    }
  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* {{{ add gid to events */
function upgrade_add_gid_to_events($link)
{
  uplog_step("Adding <tt>gid</tt> field to table <tt>events</tt>");
  $proceed = true;

  // autoreply column
  if ($proceed) {
	$col = new ColInteger();
	$sqls = sqls_add_column($link, "events", "gid", $col);
	foreach ($sqls as $sql) {
      		uplog_sql($sql);
	      $proceed = setup_db_execute($link, $sql);
		if (!$proceed) {
        		uplog_db_error($link);
		        break;
      		}
    	}
  }

  if ($proceed) {
	$gid_index=new SortIndex("idx_events_gid", array("gid"));
	$sql = sql_add_index($link, 'events', $gid_index);
	uplog_sql($sql);
	$proceed=setup_db_execute($link, $sql);
	if (!$proceed) {
        	uplog_db_error($link);
		break;
      	}

  }

  if ($proceed) {
    setup_db_commit($link);
  }

  uplog_end_step();
  return $proceed;
}
/* }}} */

